package display;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

import people.THE_WORLD;
import utility.Two;

/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

public abstract class Chart extends Canvas{
	private static final long serialVersionUID = -2373945501592620516L;
	public static Two<Integer, Integer> $4x4_MY_IMAC = new Two<Integer, Integer>(595, 310);
	public static Two<Integer, Integer> $4x5_MY_IMAC = new Two<Integer, Integer>(595, 250);
	public static Two<Integer, Integer> $5x4_MY_IMAC = new Two<Integer, Integer>(480, 310);
	public static Two<Integer, Integer> $4x3_MY_IMAC = new Two<Integer, Integer>(595, 430);
	public static Two<Integer, Integer> $3x3_MY_IMAC = new Two<Integer, Integer>(780, 430);
	public static Two<Integer, Integer> $3x4_MY_IMAC = new Two<Integer, Integer>(780, 310);
	public static Two<Integer, Integer> $3x1_MY_IMAC = new Two<Integer, Integer>(780, 1300);
	public static Two<Integer, Integer> $1x3_MY_IMAC = new Two<Integer, Integer>(2450, 430);
	public static Two<Integer, Integer> $2x2_MY_IMAC = new Two<Integer, Integer>(1200, 640);
	private final ChartKey chartKey;
	protected final int graphBorderWidth = 60;
	protected final int textBorderWidth = 5;
	protected final int textHeight = 12;
	protected final int minXTextLocationDifference = 50;
	protected final int minYTextLocationDifference = 25;
	protected final int[] intIncrementModList = new int[] {500, 100, 50, 10, 5, 1};
	
	protected final int height;
	protected final int width;
	protected final Two<Number, Number> xRange;
	protected final Two<Number, Number> yRange;
	protected final double xValueSpan;
	protected final double yValueSpan;
	protected BufferedImage bufferedImage;
	protected boolean imageDrawn = false;
	
	protected final Color[] colorsForData = new Color[] { 
			Color.BLUE, Color.RED, Color.GREEN, Color.ORANGE, Color.PINK, 
			Color.CYAN, Color.YELLOW, Color.MAGENTA, Colors.BLUE_GREEN, Colors.BROWN,
			Colors.BROWN_GREEN, Colors.BURGUNDY, Colors.DARK_BROWN_GREEN, Colors.DARK_GREEN, Colors.DEEP_GREEN,
			Colors.DEEP_PURPLE, Colors.LIGHT_BLUE, Colors.LIGHT_BROWN, Colors.LIGHT_BROWN_GREEN, Colors.LIGHT_GREEN,
			Colors.LIGHT_PURPLE, Colors.PURPLE};
	
	protected final Color labelLineColor = new Color(Color.LIGHT_GRAY.getRed(), Color.LIGHT_GRAY.getGreen(), Color.LIGHT_GRAY.getBlue(), 0xCC);
	
	public Chart(ChartKey chartKey, int width, int height, Two<Two<Number, Number>, Two<Number, Number>> range) {
		this.chartKey = chartKey;
		this.height = height;
		this.width = width;
		this.xRange = range.getFirst();
		this.yRange = range.getSecond();
		this.xValueSpan = this.xRange.getSecond().intValue() - this.xRange.getFirst().intValue();
		this.yValueSpan = this.yRange.getSecond().intValue() - this.xRange.getFirst().intValue();
		setBackground(Color.WHITE);
		setSize(width, height);
		setLocation(0, 0);
		bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_4BYTE_ABGR);
	}
	
	public void paint(Graphics graphics) {
		if(!this.imageDrawn) {
			Graphics2D bufferedImageGraphics = (Graphics2D)bufferedImage.getGraphics();
			
			//draw chart border
			bufferedImageGraphics.setColor(Color.BLACK);
			bufferedImageGraphics.drawRect(graphBorderWidth, graphBorderWidth, width - this.graphBorderWidth * 2, height - graphBorderWidth *2);

			graphData();
			
			bufferedImageGraphics.setColor(Colors.DARK_GREEN);
			labelXYValueScale(bufferedImageGraphics, THE_WORLD.UNITS_OF_TIME_IN_YEAR);
			
			this.imageDrawn = true;
			nullGraphData();
		}
		graphics.drawImage(bufferedImage, 0, 0, this);
		Font saveFont = graphics.getFont();
		graphics.setFont(saveFont.deriveFont(saveFont.getSize() + 6.0f));
		int stringWidth = graphics.getFontMetrics().stringWidth(this.chartKey.getChartTitle());
		graphics.drawString(this.chartKey.getChartTitle(), (width - stringWidth)/2, (graphBorderWidth + textHeight)/4 );
		
		graphics.setColor(Colors.DEEP_PURPLE);
		graphics.setFont(saveFont.deriveFont(saveFont.getSize() + 6.0f));
		stringWidth = graphics.getFontMetrics().stringWidth(this.chartKey.getChartXLabel());
		graphics.drawString(this.chartKey.getChartXLabel(), (width - stringWidth)/2, height - graphBorderWidth/2 + textHeight/2 );
		
		graphics.setFont(saveFont.deriveFont(saveFont.getSize() + 6.0f));
		stringWidth = graphics.getFontMetrics().stringWidth(this.chartKey.getChartYLabel());
		Graphics2D g = (Graphics2D) graphics;
		AffineTransform transform = new AffineTransform();
		transform.setToQuadrantRotation(3);
		g.setFont(g.getFont().deriveFont(transform));
		g.setFont(g.getFont().deriveFont(saveFont.getSize() + 6.0f));
		g.setColor(Colors.DEEP_PURPLE);
		g.drawString(this.chartKey.getChartYLabel(), textBorderWidth + textHeight, (height + stringWidth)/2);
		
		graphics.setColor(Color.BLACK);
		g.setFont(saveFont);
		stringWidth = graphics.getFontMetrics().stringWidth(this.chartKey.getSecondNote());
		transform = new AffineTransform();
		transform.setToQuadrantRotation(3);
		g.setFont(g.getFont().deriveFont(transform));
		g.drawString(this.chartKey.getSecondNote(), width - textBorderWidth, height/2 + stringWidth/2);
		
		g.setFont(saveFont);
		drawKey(graphics);
	}
	
	private void drawKey(Graphics graphics) {
		Color saveColor = graphics.getColor();

		for(int i = 0; i < this.chartKey.getKeys().length && i < 4; i++){
			graphics.setColor(Color.BLACK);
			int xLoc = (int)((width)*(i+0.5)/4.5);
			graphics.drawString(this.chartKey.getKeys()[i], xLoc, (graphBorderWidth + textHeight)*3/4 );
			graphics.setColor(colorsForData[i]);
			graphics.fillRect((int)(xLoc - textHeight*1.5), (graphBorderWidth + textHeight)*3/4 - textHeight, textHeight, textHeight);
		}
		graphics.setColor(saveColor);
		
		graphics.drawString(this.chartKey.getFirstNote(), graphBorderWidth/4, height - textBorderWidth );
	}
	
	public abstract void graphData();
	
	protected abstract void nullGraphData();
	
	public void labelXYValueScale(Graphics graphics, int divideXBy) {
		Color saveColor = graphics.getColor();
		int maxXLabels = (width - graphBorderWidth * 2)/minXTextLocationDifference;
		int xSpan = xRange.getSecond().intValue() - xRange.getFirst().intValue();
		xSpan /= divideXBy;
		int incrementBy = 0;
		for(int i = maxXLabels; i > 1; i--) {
			int indicationOfScale = (xSpan) / i;
			for(int j = 0; j < intIncrementModList.length; j++) {
				int potentialIncrementBase = (indicationOfScale - indicationOfScale%intIncrementModList[j]);
				if(potentialIncrementBase >= 1 && xSpan/( potentialIncrementBase + intIncrementModList[j]) <= maxXLabels) {
					if(xSpan/potentialIncrementBase <= maxXLabels) {
						incrementBy = potentialIncrementBase;
					}else{
						incrementBy = potentialIncrementBase + intIncrementModList[j];
					}
					
					break;
				}
			}
			if(incrementBy > 0) {
				break;
			}
		}
		
		if(incrementBy > 0){
			for(int i = 0; i <= maxXLabels; i++) {
				int label = xRange.getFirst().intValue() + incrementBy * i;
				double percentOfDistance = ((double)(incrementBy * i))/((double)(xSpan));
				if(label <= xRange.getSecond().intValue() && percentOfDistance < 1.01) {
					int x = graphBorderWidth + ((int)(percentOfDistance * (width - graphBorderWidth * 2)));
					int y = height - graphBorderWidth;
					graphics.drawString(""+label, graphBorderWidth + ((int)(percentOfDistance * (width - graphBorderWidth * 2))), height - graphBorderWidth + textHeight + textBorderWidth);
					if(i != 0 && i * incrementBy != xRange.getSecond().intValue()) {
						graphics.setColor(this.labelLineColor);
						graphics.drawLine(x, y, x, y - (height - graphBorderWidth * 2));
						graphics.setColor(saveColor);
					}
				}
			}
		}
		
		int maxYLabels = (height - graphBorderWidth * 2)/minYTextLocationDifference;
		int ySpan = yRange.getSecond().intValue() - yRange.getFirst().intValue();
		incrementBy = 0;
		for(int i = maxYLabels; i > 1; i--) {
			int indicationOfScale = (yRange.getSecond().intValue() - yRange.getFirst().intValue()) / i;
			for(int j = 0; j < intIncrementModList.length; j++) {
				int potentialIncrementBase = (indicationOfScale - indicationOfScale%intIncrementModList[j]);
				if(potentialIncrementBase >= 1 && ySpan/( potentialIncrementBase + intIncrementModList[j]) <= maxYLabels) {
					if(ySpan/potentialIncrementBase <= maxYLabels) {
						incrementBy = potentialIncrementBase;
					}else{
						incrementBy = potentialIncrementBase + intIncrementModList[j];
					}
					
					break;
				}
			}
			if(incrementBy > 0) {
				break;
			}
		}
		if(incrementBy > 0){
			for(int i = 0; i <= maxYLabels; i++) {
				int label = yRange.getFirst().intValue() + incrementBy * i;
				double percentOfDistance = ((double)(incrementBy * i))/((double)(yRange.getSecond().intValue()- yRange.getFirst().intValue()));
				if(label <= yRange.getSecond().intValue()) {
					int y = (height - graphBorderWidth) - ((int)(percentOfDistance * (height - graphBorderWidth * 2)));
					int x = graphBorderWidth;

					graphics.drawString(""+label, graphBorderWidth - textBorderWidth - graphics.getFontMetrics().stringWidth(""+label), (height - graphBorderWidth) - ((int)(percentOfDistance * (height - graphBorderWidth * 2))));
					if(i != 0 && i * incrementBy  + yRange.getFirst().intValue() != yRange.getSecond().intValue()) {
						graphics.setColor(this.labelLineColor);
						graphics.drawLine(x, y, x + width - graphBorderWidth * 2, y);
						graphics.setColor(saveColor);
					}
				}
			}
		}
	}
}
