package display;
import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import utility.Two;

/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/*
 *  There seems to be a rounding discrepancy with this graph where graphed results reach slightly past the top of the graph.
 */
public class FilledLineGraph extends Chart{
	private static final long serialVersionUID = 4492434745799213178L;
	private final int height;
	private Two<Number, Number>[][][] dataToGraph;
	
	public FilledLineGraph(ChartKey chartKey, int width, int height, Two<Two<Number, Number>, Two<Number, Number>> range, Two<Number, Number>[][][] dataToGraph) {
		super(chartKey, width, height, range);
		this.dataToGraph = dataToGraph;
		this.height = height;
	}
	
	public FilledLineGraph(ChartKey chartKey, int width, int height, Two<Number, Number>[][][] dataToGraph) {
		this(chartKey, width, height, determineRange(dataToGraph), dataToGraph);
	}
	
	private static Two<Two<Number, Number>, Two<Number, Number>> determineRange(Two<Number, Number>[][][] dataToGraph){
		int minXValue = Integer.MAX_VALUE;
		int maxXValue = Integer.MIN_VALUE;
		int minYValue = Integer.MAX_VALUE;
		int maxYValue = Integer.MIN_VALUE;
		for(int h = 0; h < dataToGraph.length; h++) {
			for(int i = 0; i < dataToGraph[h].length; i++) {
				for(int j = 0; j < dataToGraph[h][i].length; j++) {
					if(dataToGraph[h][i][j] != null) {
						if(dataToGraph[h][i][j].getFirst().intValue() < minXValue) {
							minXValue = dataToGraph[h][i][j].getFirst().intValue();
						}
						if(dataToGraph[h][i][j].getFirst().intValue() > maxXValue) {
							maxXValue = dataToGraph[h][i][j].getFirst().intValue();
						}
						if(dataToGraph[h][i][j].getSecond().intValue() < minYValue) {
							minYValue = dataToGraph[h][i][j].getFirst().intValue();
						}
						if(dataToGraph[h][i][j].getSecond().intValue() > maxYValue) {
							maxYValue = dataToGraph[h][i][j].getSecond().intValue();
						}
					}
				}
			}
		}
		
		int finalXMin = 0;
		int finalXMax = maxXValue;

		int finalYMin = 0;
		int finalYMax = maxYValue;
		
		return new Two<Two<Number, Number>, Two<Number, Number>>( new Two<Number, Number>(finalXMin, finalXMax), new Two<Number, Number>(finalYMin, finalYMax));
	}
	
	public void graphData() {
		if(this.dataToGraph != null) {
			Graphics graphics = this.bufferedImage.getGraphics();
			Color saveColor = graphics.getColor();
			
			Color[] colors = Colors.getTransparentColors(dataToGraph.length);
			
			for(int h = 0; h < dataToGraph.length; h++) {
				ArrayList<ArrayList<Double>> listsOfYValues = new ArrayList<ArrayList<Double>>();
				ArrayList<Double> yValues = new ArrayList<Double>();
				Collections.addAll(yValues, new Double[xRange.getSecond().intValue()]);
				listsOfYValues.add(yValues);
				
				ArrayList<Integer> xValues = new ArrayList<Integer>();
				Collections.addAll(xValues, new Integer[xRange.getSecond().intValue()]);
		
				
				for(int i = 0; i < xValues.size(); i++) {
					xValues.set(i, (int)(( (double)(i - xRange.getFirst().intValue()) ) /xValueSpan * (width - graphBorderWidth * 2) + graphBorderWidth ) );
				}
				
				ArrayList<Integer> xValues2Reverse = new ArrayList<Integer>();
				xValues2Reverse.addAll(xValues);
				Collections.reverse(xValues2Reverse);
				
				xValues.addAll(xValues2Reverse);
				
				for(int i = 0; i < dataToGraph[h].length; i++) {
					yValues = new ArrayList<Double>();
					Collections.addAll(yValues, new Double[xRange.getSecond().intValue()]);
					listsOfYValues.add(yValues);
				}
				
	
				for(int i = 0; i < xRange.getSecond().intValue(); i++) {		
					for(int j = 0; j <= dataToGraph[h].length; j++) {
						double percent = 0d;
						if(j != 0 && i < dataToGraph[h][j-1].length) {
							percent = dataToGraph[h][j-1][i].getSecond().doubleValue()/yRange.getSecond().doubleValue();
						}
						double val = (double) percent * ((double)height - (double)graphBorderWidth * 2d) ;
						listsOfYValues.get(j).set(i , (double) (height - graphBorderWidth - (int) val ) );
					}
				}
	
				for(int i = 1; i < listsOfYValues.size(); i++) {
					graphics.setColor(colors[(i - 1) % 5]);
					ArrayList<Integer> yValues4Graph = new ArrayList<Integer>();
					ArrayList<Integer> yValues2Reverse = new ArrayList<Integer>();
					
					for(Double d : listsOfYValues.get(i)) {
						if(d != null) {
							yValues4Graph.add(d.intValue());
						}
					}
					
					for(Double d : listsOfYValues.get(0)) {
						yValues2Reverse.add(d.intValue());
					}
					
					Collections.reverse(yValues2Reverse);
					
					yValues4Graph.addAll(yValues2Reverse);
		
					Integer[] xVals = new Integer[xValues.size()];
					xVals = xValues.toArray(xVals);
					int[] xArr = Arrays.stream(xVals).mapToInt(Integer::intValue).toArray();
					
					Integer[] yVals = new Integer[xValues.size()];
					yVals = yValues4Graph.toArray(yVals);
					int[] yArr = Arrays.stream(yVals).mapToInt(Integer::intValue).toArray();
					
					graphics.fillPolygon(xArr, yArr, xValues.size());
					graphics.drawPolygon(xArr, yArr, xValues.size());
				}
			}
			
			graphics.setColor(saveColor);
		}
	}
	
	protected void nullGraphData() {
		this.dataToGraph = null;
	}
}
