package display;
import java.awt.Color;
import java.awt.Graphics;
import utility.Two;

/*	Copyright 2021 Ian Stuart

This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/


public class LineGraph extends Chart{
	private static final long serialVersionUID = -169959815131265135L;
	private final int height;
	private Two<Number, Number>[][] dataToGraph;

	
	public LineGraph(ChartKey chartKey, int width, int height, Two<Two<Number, Number>, Two<Number, Number>> range, Two<Number, Number>[][] dataToGraph) {
		
		super(chartKey, width, height, range);
		this.dataToGraph = dataToGraph;
		this.height = height;

	}
	
	public LineGraph(ChartKey chartKey, int width, int height, Two<Number, Number>[][] dataToGraph) {
		this(chartKey, width, height, determineRange(dataToGraph), dataToGraph);
	}
	
	private static Two<Two<Number, Number>, Two<Number, Number>> determineRange(Two<Number, Number>[][] dataToGraph){
		int minXValue = Integer.MAX_VALUE;
		int maxXValue = Integer.MIN_VALUE;
		int minYValue = Integer.MAX_VALUE;
		int maxYValue = Integer.MIN_VALUE;
		
		for(int i = 0; i < dataToGraph.length; i++) {
			for(int j = 0; j < dataToGraph[i].length; j++) {
				if(dataToGraph[i][j] != null) {
					if(dataToGraph[i][j].getFirst().intValue() < minXValue) {
						minXValue = dataToGraph[i][j].getFirst().intValue();
					}
					if(dataToGraph[i][j].getFirst().intValue() > maxXValue) {
						maxXValue = dataToGraph[i][j].getFirst().intValue();
					}
					if(dataToGraph[i][j].getSecond().intValue() < minYValue) {
						minYValue = dataToGraph[i][j].getFirst().intValue();
					}
					if(dataToGraph[i][j].getSecond().intValue() > maxYValue) {
						maxYValue = dataToGraph[i][j].getSecond().intValue();
					}
				}
			}
		}
		
		int finalXMin = 0;
		int finalXMax = maxXValue;
		
		int finalYMin = 0;
		int finalYMax = maxYValue;
		
		return new Two<Two<Number, Number>, Two<Number, Number>>( new Two<Number, Number>(finalXMin, finalXMax), new Two<Number, Number>(finalYMin, finalYMax));
	}
	
	public void labelXYValueScale(int divideXBy) {
		Graphics graphics = this.bufferedImage.getGraphics();
		Color saveColor = graphics.getColor();
		int maxXLabels = (width - graphBorderWidth * 2)/minXTextLocationDifference;
		int xSpan = xRange.getSecond().intValue() - xRange.getFirst().intValue();
		xSpan /= divideXBy;
		int incrementBy = 0;
		for(int i = maxXLabels; i > 1; i--) {
			int indicationOfScale = (xSpan) / i;
			for(int j = 0; j < intIncrementModList.length; j++) {
				int potentialIncrementBase = (indicationOfScale - indicationOfScale%intIncrementModList[j]);
				if(potentialIncrementBase >= 1 && xSpan/( potentialIncrementBase + intIncrementModList[j]) <= maxXLabels) {
					if(xSpan/potentialIncrementBase <= maxXLabels) {
						incrementBy = potentialIncrementBase;
					}else{
						incrementBy = potentialIncrementBase + intIncrementModList[j];
					}
					
					break;
				}
			}
			if(incrementBy > 0) {
				break;
			}
		}
		if(incrementBy > 0){
			for(int i = 0; i <= maxXLabels; i++) {
				int label = xRange.getFirst().intValue() + incrementBy * i;
				double percentOfDistance = ((double)(incrementBy * i))/((double)(xSpan));
				if(label <= xRange.getSecond().intValue() && percentOfDistance < 1.01) {
					int x = graphBorderWidth + ((int)(percentOfDistance * (width - graphBorderWidth * 2)));
					int y = height - graphBorderWidth;
					graphics.drawString(""+label, graphBorderWidth + ((int)(percentOfDistance * (width - graphBorderWidth * 2))), height - graphBorderWidth + textHeight + textBorderWidth);
					if(i != 0 && i * incrementBy != xRange.getSecond().intValue()) {
						graphics.setColor(this.labelLineColor);
						graphics.drawLine(x, y, x, y - (height - graphBorderWidth * 2));
						graphics.setColor(saveColor);
					}
				}
			}
		}
		
		int maxYLabels = (height - graphBorderWidth * 2)/minYTextLocationDifference;
		int ySpan = yRange.getSecond().intValue() - yRange.getFirst().intValue();
		incrementBy = 0;
		for(int i = maxYLabels; i > 1; i--) {
			int indicationOfScale = (yRange.getSecond().intValue() - yRange.getFirst().intValue()) / i;
			for(int j = 0; j < intIncrementModList.length; j++) {
				int potentialIncrementBase = (indicationOfScale - indicationOfScale%intIncrementModList[j]);
				if(potentialIncrementBase >= 1 && ySpan/( potentialIncrementBase + intIncrementModList[j]) <= maxYLabels) {
					if(ySpan/potentialIncrementBase <= maxYLabels) {
						incrementBy = potentialIncrementBase;
					}else{
						incrementBy = potentialIncrementBase + intIncrementModList[j];
					}
					
					break;
				}
			}
			if(incrementBy > 0) {
				break;
			}
		}
		if(incrementBy > 0){
			for(int i = 0; i <= maxYLabels; i++) {
				int label = yRange.getFirst().intValue() + incrementBy * i;
				double percentOfDistance = ((double)(incrementBy * i))/((double)(yRange.getSecond().intValue()- yRange.getFirst().intValue()));
				if(label <= yRange.getSecond().intValue()) {
					int y = (height - graphBorderWidth) - ((int)(percentOfDistance * (height - graphBorderWidth * 2)));
					int x = graphBorderWidth;
					graphics.drawString(""+label, graphBorderWidth - textBorderWidth - graphics.getFontMetrics().stringWidth(""+label), (height - graphBorderWidth) - ((int)(percentOfDistance * (height - graphBorderWidth * 2))));
					if(i != 0 && i * incrementBy  + yRange.getFirst().intValue() != yRange.getSecond().intValue()) {
						graphics.setColor(this.labelLineColor);
						graphics.drawLine(x, y, x + width - graphBorderWidth * 2, y);
						graphics.setColor(saveColor);
					}
				}
			}
		}
	}
	
	public void graphData() {
		if(this.dataToGraph != null) {
			Graphics graphics = this.bufferedImage.getGraphics();
			Color saveColor = graphics.getColor();
			
			for(int i = 0; i < dataToGraph.length; i++) {
				Two<Number, Number> previousPoint = null;
				graphics.setColor(colorsForData[i%22]);
				//for(Two<Number, Number> point : dataToGraph[i]) {
				for(int j = 0; j < dataToGraph[i].length; j++) {
					if(previousPoint == null || dataToGraph[i][j] == null) {
						previousPoint = dataToGraph[i][j];
					}else {					
						
						graphics.drawLine((int) (( (double)(previousPoint.getFirst().intValue() - xRange.getFirst().intValue()) ) /xValueSpan * (width - graphBorderWidth * 2) + graphBorderWidth ), 
								height - graphBorderWidth - (int) (( (double)(previousPoint.getSecond().intValue() - yRange.getFirst().intValue()) )/yValueSpan * (height - graphBorderWidth * 2) ), 
								(int) (( (double)(dataToGraph[i][j].getFirst().intValue() - xRange.getFirst().intValue()) ) /xValueSpan * (width - graphBorderWidth * 2) + graphBorderWidth ), 
								height - graphBorderWidth - (int) (( (double)(dataToGraph[i][j].getSecond().intValue() - yRange.getFirst().intValue()) )/yValueSpan * (height - graphBorderWidth * 2) ));
						previousPoint = dataToGraph[i][j];
					}
				}
			}
			
			
			
			graphics.setColor(saveColor);
		}
	}

	protected void nullGraphData() {
		this.dataToGraph = null;
	}
}
