package display;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Map.Entry;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import helper.DeterminationHelper;
import people.EConsumable;
import people.Industry;
import people.THE_WORLD;
import utility.Two;

/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or 
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

public class MyReport {	
	private final static EConsumable[] industryReportOrder = new EConsumable[] {EConsumable.NECESSITY, EConsumable.RAW_MATERIAL, EConsumable.DISCRETIONARY};
	
	
	/*
	 * This is meant to be called after a number of simulation runs. Passing in window for a nicer transition from the simulation runs in progress to displaying results. MyReport get...Data() methods only return data from the last run so an
	 * ArrayList needs to be created and added to with each run to create 'overlappingLineGraphsInfo' the title is meant to be an introduction for the
	 * resulting data. the ArrayList<String> results data is meant to be a separate data collected for display. 
	 */
	public static void displayOverlappingGraphsReport(Window window, ArrayList<OverlappingChartArguments> overlappingLineGraphsInfo, String title, ArrayList<String> results) {
		JPanel mainPanel = new JPanel();
		JPanel panel = new JPanel();
		panel.add(new TestResults(Chart.$3x3_MY_IMAC.getFirst(), Chart.$3x3_MY_IMAC.getSecond(), title, results));
		mainPanel.add(panel);
		for(int i = 0; i < overlappingLineGraphsInfo.size(); i++) {
			panel = new JPanel();		
			panel.add( getOverlappingChart(overlappingLineGraphsInfo.get(i), Chart.$3x3_MY_IMAC) );	
			mainPanel.add(panel);
		}
		
		/* display single results of last simulation run */
		panel = new JPanel();
		ArrayList<ChartArguments> graphsInfo = getChartData("(Chart based on the final (single) simulation run)");
		ArrayList<String> singleRunResults = getSingleRunResults();
		panel.add(new TestResults(Chart.$3x3_MY_IMAC.getFirst(), Chart.$3x3_MY_IMAC.getSecond(), "Single Results For Final Simulation Run", singleRunResults));
		mainPanel.add(panel);
		for(int i = 0; i < graphsInfo.size(); i++) {
			panel = new JPanel();
			panel.add( getChart(graphsInfo.get(i), Chart.$3x3_MY_IMAC) );
			mainPanel.add(panel);
		}
		
		JScrollPane jsp = new JScrollPane(mainPanel);
		jsp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		window.add(jsp);
		window.setVisible(true);
	}
		
	//List of strings with a few specific results in text...
	private static ArrayList<String> getSingleRunResults(){
		ArrayList<String> singleRunResults = new ArrayList<String>();
		
		double averagePopulationSecondDecade = getAveragePopulationWithinTimeRange(120, 240);
		double averagePopulationTwentySecondDecade = getAveragePopulationWithinTimeRange(2520, 2640);
		singleRunResults.add("Average Population for second decade : " + String.format("%4.2f", averagePopulationSecondDecade) );
		singleRunResults.add("Average Population for twenty-second decade :" + String.format("%4.2f", averagePopulationTwentySecondDecade) );
		
		double averageRawMaterialEmployeesSecondDecade = getAverageEmployeesByIndustryWithinTimeRange(EConsumable.RAW_MATERIAL, 120, 240);
		double averageRawMaterialEmployeesTwentySecondDecade = getAverageEmployeesByIndustryWithinTimeRange(EConsumable.RAW_MATERIAL, 2520, 2640);
		double averageNecessityEmployeesSecondDecade = getAverageEmployeesByIndustryWithinTimeRange(EConsumable.NECESSITY, 120, 240);
		double averageNecessityEmployeesTwentySecondDecade = getAverageEmployeesByIndustryWithinTimeRange(EConsumable.NECESSITY, 2520, 2640);
		double averageDiscretionaryEmployeesSecondDecade = getAverageEmployeesByIndustryWithinTimeRange(EConsumable.DISCRETIONARY, 120, 240);
		double averageDiscretionaryEmployeesTwentySecondDecade = getAverageEmployeesByIndustryWithinTimeRange(EConsumable.DISCRETIONARY, 2520, 2640);
		double averageTotalEmployeesSecondDecade = averageRawMaterialEmployeesSecondDecade + averageNecessityEmployeesSecondDecade + averageDiscretionaryEmployeesSecondDecade;
		double averageTotalEmployeesTwentySecondDecade = averageRawMaterialEmployeesTwentySecondDecade + averageNecessityEmployeesTwentySecondDecade + averageDiscretionaryEmployeesTwentySecondDecade;
		singleRunResults.add("Average Share of workforce in RawMaterials for the second decade :" + String.format("%3.1f", averageRawMaterialEmployeesSecondDecade/averageTotalEmployeesSecondDecade * 100) + "%");
		singleRunResults.add("Average Share of workforce in RawMaterials for the twenty second decade :" + String.format("%3.1f", averageRawMaterialEmployeesTwentySecondDecade/averageTotalEmployeesTwentySecondDecade * 100) + "%");
		singleRunResults.add("Average Share of workforce in Necessities for the second decade :" + String.format("%3.1f", averageNecessityEmployeesSecondDecade/averageTotalEmployeesSecondDecade * 100) + "%");
		singleRunResults.add("Average Share of workforce in Necessities for twenty-second decade :" + String.format("%3.1f", averageNecessityEmployeesTwentySecondDecade/averageTotalEmployeesTwentySecondDecade * 100) + "%");
		singleRunResults.add("Average Share of workforce in Discretionaries for the second decade :" + String.format("%3.1f", averageDiscretionaryEmployeesSecondDecade/averageTotalEmployeesSecondDecade * 100) + "%");
		singleRunResults.add("Average Share of workforce in Discretionaries for twenty-second decade :" + String.format("%3.1f", averageDiscretionaryEmployeesTwentySecondDecade/averageTotalEmployeesTwentySecondDecade * 100) + "%");
		
		double averageRawMaterialSalePriceSecondDecade = getAverageConsumableSalePriceWithinTimeRange(EConsumable.RAW_MATERIAL, 120, 240);
		double averageRawMaterialSalePriceTwentySecondDecade = getAverageConsumableSalePriceWithinTimeRange(EConsumable.RAW_MATERIAL, 2520, 2640);
		double averageNecessitySalePriceSecondDecade = getAverageConsumableSalePriceWithinTimeRange(EConsumable.NECESSITY, 120, 240);
		double averageNecessitySalePriceTwentySecondDecade = getAverageConsumableSalePriceWithinTimeRange(EConsumable.NECESSITY, 2520, 2640);
		double averageDiscretionarySalePriceSecondDecade = getAverageConsumableSalePriceWithinTimeRange(EConsumable.DISCRETIONARY, 120, 240);
		double averageDiscretionarySalePriceTwentySecondDecade = getAverageConsumableSalePriceWithinTimeRange(EConsumable.DISCRETIONARY, 2520, 2640);

		singleRunResults.add("Average Raw Material Unit Sale Price for the second decade :" + String.format("%3.1f", averageRawMaterialSalePriceSecondDecade));
		singleRunResults.add("Average Raw Material Unit Sale Price for twenty-second decade :" + String.format("%3.1f", averageRawMaterialSalePriceTwentySecondDecade));
		singleRunResults.add("Average Necessity Unit Sale Price for the second decade :" + String.format("%3.1f", averageNecessitySalePriceSecondDecade));
		singleRunResults.add("Average Necessity Unit Sale Price for twenty-second decade :" + String.format("%3.1f", averageNecessitySalePriceTwentySecondDecade));
		singleRunResults.add("Average Discretionary Unit Sale Price for the second decade :" + String.format("%3.1f", averageDiscretionarySalePriceSecondDecade));
		singleRunResults.add("Average Discretionary Unit Sale Price for twenty-second decade :" + String.format("%3.1f", averageDiscretionarySalePriceTwentySecondDecade));

		return singleRunResults;
	}

	//get charts with multiple results overlay no top of each other...
	public static Chart getOverlappingChart(OverlappingChartArguments data , Two<Integer, Integer> size) {
		Chart chart = null;
		switch(data.getType()) {
			case LINE_GRAPH:			chart = getOverlappingLineGraph(data, size);
										break;
			case STACKED_AREA_CHART:	chart = getOverlappingStackedAreaChart(data, size);
										break;
			case FILLED_LINE_GRAPH: 			chart = getFilledLineGraph(data, size);
										break;
		}
		return chart;
	}

	public static Chart getChart(ChartArguments data , Two<Integer, Integer> size) {
		Chart chart = null;
		switch(data.getType()) {
			case LINE_GRAPH:			chart = getLineGraph(data, size);
										break;
			case STACKED_AREA_CHART:	chart = getStackedAreaChart(data, size);
										break;
			//case FILLED_LINE_GRAPH: 	chart = getFilledLineGraph(data, size);
			//							break;
			default:
										break;
		}
		return chart;
	}
	
	public static FilledLineGraph getFilledLineGraph(OverlappingChartArguments data , Two<Integer, Integer> size) {
		return new FilledLineGraph(data.getChartKey(), size.getFirst(), size.getSecond(), data.getDataToGraph());
	}
	
	public static FilledLineGraph getFilledLineGraph(OverlappingChartArguments data , Two<Integer, Integer> size, double highestUnemployment) {
		return new FilledLineGraph(data.getChartKey(), size.getFirst(), size.getSecond(), data.getDataToGraph());
	}
	
	public static OverlappingStackedAreaChart getOverlappingStackedAreaChart(OverlappingChartArguments data , Two<Integer, Integer> size) {
		return new OverlappingStackedAreaChart(data.getChartKey(), size.getFirst(), size.getSecond(), data.getDataToGraph());
	}
	
	public static OverlappingLineGraph getOverlappingLineGraph(OverlappingChartArguments data , Two<Integer, Integer> size) {
		return new OverlappingLineGraph(data.getChartKey(), size.getFirst(), size.getSecond(), data.getDataToGraph());
	}
	
	public static LineGraph getLineGraph(ChartArguments data, Two<Integer, Integer> size) {
		return new LineGraph(data.getChartKey(), size.getFirst(), size.getSecond(), data.getDataToGraph());
	}
	
	public static StackedAreaChart getStackedAreaChart(ChartArguments data, Two<Integer, Integer> size) {
		return new StackedAreaChart(data.getChartKey(), size.getFirst(), size.getSecond(), data.getDataToGraph());
	}
	
	public static Two<Number, Number>[][] getUnemploymentData() {
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[1][THE_WORLD.getInstance().getTime()];
		
		for(int i = 0; i < THE_WORLD.getInstance().getTime(); i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getUnemploymentRatio() * 100 );
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] getEmployeesByIndustryData(){
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[3][THE_WORLD.getInstance().getTime()];
		
		for(int i = 0; i < THE_WORLD.getInstance().getTime(); i++) {
			dataToGraph[2][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getIndustry(EConsumable.RAW_MATERIAL).getIndustryRecords().get(i).getNumberOfEmployees());
			dataToGraph[0][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getIndustry(EConsumable.DISCRETIONARY).getIndustryRecords().get(i).getNumberOfEmployees());
			dataToGraph[1][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getIndustry(EConsumable.NECESSITY).getIndustryRecords().get(i).getNumberOfEmployees());
		}
		return dataToGraph;
	}
	
	public static double getAverageEmployeesByIndustryWithinTimeRange(EConsumable consumable, int firstTimeUnit, int endingTimeUnitExclusive) {
		double forCalculatingAverageEmployees = 0;
		int counter = 0;
		for(int i = firstTimeUnit; i < THE_WORLD.getInstance().getIndustry(consumable).getIndustryRecords().size() && i < endingTimeUnitExclusive; i++) {
			forCalculatingAverageEmployees += THE_WORLD.getInstance().getIndustry(consumable).getIndustryRecords().get(i).getNumberOfEmployees();
			counter++;
		}
		return forCalculatingAverageEmployees/(counter > 0 ? counter : 1);
	}
	
	public static Two<Number, Number>[][] getFinancialStandingData(){
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[4][((int)(THE_WORLD.getInstance().getTime()))];
		
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getTime())); i++) {
			for(int j = 0; j < industryReportOrder.length; j++) {
				dataToGraph[j][i] = new Two<Number, Number>(i, ((int)(THE_WORLD.getInstance().getIndustry(industryReportOrder[j]).getIndustryRecords().get(i).getRoughFinancialStanding()*1000)));
			}
			dataToGraph[3][i] = new Two<Number, Number>(i, ((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getRoughFinancialStanding()*1000)));
		}
		return dataToGraph;
	}
		
	public static Two<Number, Number>[][] getGovernmentRevenueData(){
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[4][((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size()))];
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size())); i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getRevenueFromTaxingProfit());
			dataToGraph[1][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getRevenueFromTaxingIncome());
			dataToGraph[2][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getRevenueFromSalesTax());
			dataToGraph[3][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getRevenueFromInheritanceTax());
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] getGovernmentTaxRatesData(){
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[4][((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size()))];
		
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size())); i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, (int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getProfitTaxRate() * 100));
			dataToGraph[1][i] = new Two<Number, Number>(i, (int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getIncomeTaxRate() * 100));
			dataToGraph[2][i] = new Two<Number, Number>(i, (int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getSaleTaxRate() * 100));
			dataToGraph[3][i] = new Two<Number, Number>(i, (int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getInheritanceTaxRate() * 100));
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] getTotalSenarioMoneyData(){
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[4][((int)(THE_WORLD.getInstance().getTime()))];
		
		int[] totalMoney = new int[((int)(THE_WORLD.getInstance().getTime()))];
		int[] totalMoneyPeople = new int[((int)(THE_WORLD.getInstance().getTime()))];
		int[] totalMoneyIndustry = new int[((int)(THE_WORLD.getInstance().getTime()))];
		int[] totalMoneyGovernment = new int[((int)(THE_WORLD.getInstance().getTime()))];
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getTime())); i++) {
			for(Entry<EConsumable, Industry> entry : THE_WORLD.getInstance().getIndustries().entrySet()) {
				totalMoney[i] += THE_WORLD.getInstance().getIndustry(entry.getKey()).getIndustryRecords().get(i).getMoney();
				totalMoneyIndustry[i] += THE_WORLD.getInstance().getIndustry(entry.getKey()).getIndustryRecords().get(i).getMoney();
			}
			totalMoney[i] += THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getMoney();
			totalMoneyGovernment[i] +=THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getMoney();
		}
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size())); i++) {
			totalMoney[i] += THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getTotalMoneyThatAllPeopleHave();
			totalMoneyPeople[i] += THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getTotalMoneyThatAllPeopleHave();
		}
		for(int i = 0; i < totalMoney.length; i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, totalMoney[i]);
			dataToGraph[1][i] = new Two<Number, Number>(i, totalMoneyPeople[i]);
			dataToGraph[2][i] = new Two<Number, Number>(i, totalMoneyGovernment[i]);
			dataToGraph[3][i] = new Two<Number, Number>(i, totalMoneyIndustry[i]);
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] getSenarioMoneyData(){
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[3][((int)(THE_WORLD.getInstance().getTime()))];
		
		Two<Number, Number>[][] dataForTransfer = getTotalSenarioMoneyData();
		dataToGraph[0] = dataForTransfer[1];
		dataToGraph[1] = dataForTransfer[2];
		dataToGraph[2] = dataForTransfer[3];
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][]  getMoneyBy_Industry_Government_Data() {
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][]  dataToGraph = new Two[4][((int)(THE_WORLD.getInstance().getTime()))];
		
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getIndustry(EConsumable.RAW_MATERIAL).getIndustryRecords().size())); i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getIndustry(EConsumable.RAW_MATERIAL).getIndustryRecords().get(i).getMoney());
			dataToGraph[1][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getIndustry(EConsumable.NECESSITY).getIndustryRecords().get(i).getMoney());
			dataToGraph[2][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getIndustry(EConsumable.DISCRETIONARY).getIndustryRecords().get(i).getMoney());
			dataToGraph[3][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getMoney());
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] get_Inventory_Production__Demand_UnfilledPositions_ByIndustryData(EConsumable consumable){ 
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[4][((int)(THE_WORLD.getInstance().getTime()))];
		
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getTime())); i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getIndustry(consumable).getIndustryRecords().get(i).getProductQuantity()/10);
			dataToGraph[1][i] = new Two<Number, Number>(i, (int) THE_WORLD.getInstance().getIndustry(consumable).getIndustryRecords().get(i).getTotalProduction());
			dataToGraph[2][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getIndustry(consumable).getIndustryRecords().get(i).getProductDemand());
			dataToGraph[3][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getIndustry(consumable).getIndustryRecords().get(i).getUnfilledPositions() * 
																									/*  calculate and multiply by AdjustedEmployeeProduction  */
																									consumable.getQuantityToProduceAtOnce() * 
																									THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getLengthOfWorkday() *
																									THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getTheAdjustedProductionRateOfIndustries());
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] getInventoryByIndustryData() {		
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[3][THE_WORLD.getInstance().getTime()];
		
		for(int i = 0; i < THE_WORLD.getInstance().getTime(); i++) {
			for(int j = 0; j < industryReportOrder.length; j++) {
				dataToGraph[j][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getIndustry(industryReportOrder[j]).getIndustryRecords().get(i).getProductQuantity());
			}
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] getPopulationData(){	
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[4][((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size()))];
		
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size())); i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getPopulation());
			dataToGraph[1][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getAdultPopulation());
			dataToGraph[2][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getUnder18Population());
			dataToGraph[3][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getRetirementAgePopulation());
		}
		return dataToGraph;
	}
	
	public static double getAveragePopulationWithinTimeRange(int firstTimeUnit, int endingTimeUnitExclusive) {
		double forCalculatingAveragePopulation = 0;
		int counter = 0;
		for(int i = firstTimeUnit; i < THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size() && i < endingTimeUnitExclusive; i++) {
			forCalculatingAveragePopulation += THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getPopulation();
			counter++;
		}
		return forCalculatingAveragePopulation/(counter > 0 ? counter : 1);
	}
		
	public static Two<Number, Number>[][] getPopulationFinancialData(){
		
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[3][((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size()))];
		
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size())); i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getAverageBalance());
			dataToGraph[1][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getAverageBalanceOfTopQuarter());
			dataToGraph[2][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getAverageBalanceOfBottomQuarter());
		}
		return dataToGraph;

	}
	
	public static double getAverageConsumableSalePriceWithinTimeRange(EConsumable consumable, int firstTimeUnit, int endingTimeUnitExclusive) {
		double forCalculatingAveragePrice = 0;
		int counter = 0;
		for(int i = firstTimeUnit; i < THE_WORLD.getInstance().getIndustry(consumable).getIndustryRecords().size() && i < endingTimeUnitExclusive; i++) {
			forCalculatingAveragePrice += THE_WORLD.getInstance().getIndustry(consumable).getIndustryRecords().get(i).getSalePrice();
			counter++;
		}
		return forCalculatingAveragePrice/(counter > 0 ? counter : 1);
	}
	
	public static Two<Number, Number>[][] getProductPriceData(){
		
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[3][((int)(THE_WORLD.getInstance().getTime()))];
		
		for(int i = 0; i< THE_WORLD.getInstance().getTime(); i++) {
			for(int j = 0; j < industryReportOrder.length; j++) {
				dataToGraph[j][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getIndustry(industryReportOrder[j]).getIndustryRecords().get(i).getSalePrice());
			}
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] getProductionRateData() {
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[1][((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size()))];
		
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size())); i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getTheAdjustedProductionRateOfIndustries() * 100);
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] getParentToANewChildPaymentData(){
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[1][((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size()))];
		
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size())); i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, (int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getParentToANewChildPayment()));
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] getRetirmentAgeData(){
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[1][((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size()))];
		
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size())); i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, (int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getRetirementAge()));
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] getLengthOfWorkdayData(){
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[1][((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size()))];
		
		for(int i = 0; i < ((int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().size())); i++) {
			dataToGraph[0][i] = new Two<Number, Number>(i, (int)(THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getLengthOfWorkday() * 100));
		}
		return dataToGraph;
	}
	
	public static Two<Number, Number>[][] getGDPAdjustedByPurchasingPowerBasedOnAverageConsumption() {
		@SuppressWarnings("unchecked")
		Two<Number, Number>[][] dataToGraph = new Two[1][((int)(THE_WORLD.getInstance().getTime()))];
		
		if(!DeterminationHelper.DOES_ALLOW_A_ZERO_SALE_PRICE) {
			double sampleSize = 60;
			double necessitySalePrice = 0;
			double discretionarySalePrice = 0;
			//double necessitiesSold = 0;
			//double discretionariesSold = 0;
			LinkedList<Double> list = new LinkedList<Double>();
			
			list = new LinkedList<Double>();
			for(int i = 0; i< THE_WORLD.getInstance().getTime(); i++) {
				double gdpForTimeUnit = 0;
				for(EConsumable  key : THE_WORLD.getInstance().getIndustries().keySet()) {
					double production = THE_WORLD.getInstance().getIndustry(key).getIndustryRecords().get(i).getTotalProduction();
					double value = production * THE_WORLD.getInstance().getIndustry(key).getIndustryRecords().get(i).getSalePrice();
					//value -= THE_WORLD.getInstance().getIndustry(key).getIndustryRecords().get(i).getMoneySpent() - THE_WORLD.getInstance().getIndustry(key).getIndustryRecords().get(i).getWagesPaid();
					if(key != EConsumable.RAW_MATERIAL) {
						gdpForTimeUnit += value;
					}
					if(i >= THE_WORLD.getInstance().getTime() - sampleSize) {
						if(key == EConsumable.NECESSITY) {
							necessitySalePrice += THE_WORLD.getInstance().getIndustry(key).getIndustryRecords().get(i).getSalePrice();
						}else if(key == EConsumable.DISCRETIONARY){
							discretionarySalePrice += THE_WORLD.getInstance().getIndustry(key).getIndustryRecords().get(i).getSalePrice();
						}
					}
				}
				gdpForTimeUnit /= THE_WORLD.getInstance().getGovernment().getGovernmentRecords().get(i).getPopulation();
				double gdpForYear = 0; 
				while(list.size() >= THE_WORLD.UNITS_OF_TIME_IN_YEAR) {
					list.pollLast();
				}
				for(int j = 0; j < list.size(); j++) {
					gdpForYear += list.get(j);
				}
				gdpForYear += gdpForTimeUnit;
				list.push(gdpForTimeUnit);
				dataToGraph[0][i] = new Two<Number, Number>(i, (int)gdpForYear);
			}
			necessitySalePrice /= sampleSize;
			discretionarySalePrice /= sampleSize;
			for(int i = 0; i < THE_WORLD.getInstance().getTime(); i++) {
				double baseLine = dataToGraph[0][i].getSecond().intValue();
				double totalProductSold = THE_WORLD.getInstance().getIndustry(EConsumable.NECESSITY).getIndustryRecords().get(i).getQuantitySold() + 
						THE_WORLD.getInstance().getIndustry(EConsumable.DISCRETIONARY).getIndustryRecords().get(i).getQuantitySold();
				double forAverage = necessitySalePrice/THE_WORLD.getInstance().getIndustry(EConsumable.NECESSITY).getIndustryRecords().get(i).getSalePrice() *
									((double)THE_WORLD.getInstance().getIndustry(EConsumable.NECESSITY).getIndustryRecords().get(i).getQuantitySold() / totalProductSold);																																		
				forAverage += discretionarySalePrice/THE_WORLD.getInstance().getIndustry(EConsumable.DISCRETIONARY).getIndustryRecords().get(i).getSalePrice() *
									((double)THE_WORLD.getInstance().getIndustry(EConsumable.DISCRETIONARY).getIndustryRecords().get(i).getQuantitySold() / totalProductSold);
				if(THE_WORLD.getInstance().getIndustry(EConsumable.NECESSITY).getIndustryRecords().get(i).getSalePrice() > 0 &&
						THE_WORLD.getInstance().getIndustry(EConsumable.DISCRETIONARY).getIndustryRecords().get(i).getSalePrice() > 0 ) {
					dataToGraph[0][i] = new Two<Number, Number>(i, (int)(forAverage * baseLine));
				}else {
					dataToGraph[0][i] = null;
				}				
			}
		}
		return dataToGraph;
	}
	
	//chartGetChart data for the last run (time step must still be at final value)
	public static ArrayList<ChartArguments> getChartData(String singleRunNote){
		ArrayList<ChartArguments> lineGraphArgumentsList = new ArrayList<ChartArguments>();	
		lineGraphArgumentsList.add(new ChartArguments( getFinancialStandingChartKey(singleRunNote), getFinancialStandingData(), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( get_Inventory_Production__Demand_UnfilledPositions_ByIndustryChartKey(EConsumable.NECESSITY, singleRunNote),
				get_Inventory_Production__Demand_UnfilledPositions_ByIndustryData(EConsumable.NECESSITY), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( get_Inventory_Production__Demand_UnfilledPositions_ByIndustryChartKey(EConsumable.RAW_MATERIAL, singleRunNote),
				get_Inventory_Production__Demand_UnfilledPositions_ByIndustryData(EConsumable.RAW_MATERIAL), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( get_Inventory_Production__Demand_UnfilledPositions_ByIndustryChartKey(EConsumable.DISCRETIONARY, singleRunNote),
				get_Inventory_Production__Demand_UnfilledPositions_ByIndustryData(EConsumable.DISCRETIONARY), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( getPopulationChartKey(singleRunNote), getPopulationData(), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( getProductPriceChartKey(singleRunNote), getProductPriceData(), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( getParentToANewChildPaymentChartKey(singleRunNote), getParentToANewChildPaymentData(), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( getRetirmentAgeChartKey(singleRunNote), getRetirmentAgeData(), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( getLengthOfWorkdayChartKey(singleRunNote), getLengthOfWorkdayData(), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( getGDPAdjustedByPurchasingPowerBasedOnAverageConsumptionChartKey(singleRunNote),
				getGDPAdjustedByPurchasingPowerBasedOnAverageConsumption(), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( getPopulationFinancialChartKey(singleRunNote), getPopulationFinancialData(), EChart.LINE_GRAPH));	
		lineGraphArgumentsList.add(new ChartArguments( getGovernmentTaxRatesChartKey(singleRunNote), getGovernmentTaxRatesData(), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( getUnemploymentChartKey(singleRunNote), getUnemploymentData(), EChart.LINE_GRAPH));
		lineGraphArgumentsList.add(new ChartArguments( getGovernmentRevenueChartKey(singleRunNote), getGovernmentRevenueData(), EChart.STACKED_AREA_CHART));	
		lineGraphArgumentsList.add(new ChartArguments( getEmployeesByIndustryChartKey(singleRunNote), getEmployeesByIndustryData(), EChart.STACKED_AREA_CHART));		
		lineGraphArgumentsList.add(new ChartArguments( getSenarioMoneyChartKey(singleRunNote), getSenarioMoneyData(), EChart.STACKED_AREA_CHART));		
		lineGraphArgumentsList.add(new ChartArguments( getProductionRateChartKey(singleRunNote), getProductionRateData(), EChart.LINE_GRAPH));

		return lineGraphArgumentsList;
	}
	
	
	public static ChartKey getFinancialStandingChartKey(String secondNote) {
		return new ChartKey("Rough Financial Standing* (max results truncated)", "Year", "Financial Standing", new String[] {"Necessity Industry", "Raw Material Industry", "Discretionary Industry", "Government"}, 
														"*1000 here is the target \"Financial Standing\"; 0 represents absence of money; too high above 1000 can be destabilizing", secondNote);
	}
	public static ChartKey get_Inventory_Production__Demand_UnfilledPositions_ByIndustryChartKey(EConsumable eConsumable, String secondNote) {
		return new ChartKey( eConsumable.nameOfIndustry() + " Industry", "Year", "Amount", new String[] {"Inventory (In Tens)", "Production", "Demand", "Unfilled Positions*"}, 
														"*unfilled Positions are displayed on the chart as their collective potential production", secondNote);
	}
	public static ChartKey getPopulationChartKey(String secondNote) {
		return new ChartKey("Population Data", "Years", "People", new String[] {"Total", "Adults", "Under 18", "Retirement Age*"}, "*the retirement age can be lowered by the government throughout simulations", secondNote);
	}
	public static ChartKey getProductPriceChartKey(String secondNote) {
		return new ChartKey("Consumable Sale Price Data*", "Year", "Price", new String[] {"Necessity", "Raw Material", "Discretionary"}, "*price for a \"unit\" of each consumable", secondNote);
	}
	public static ChartKey getParentToANewChildPaymentChartKey(String secondNote) {
		return new ChartKey("Parent to a New Child Payment Amount", "Year", "Amount", new String[] {}, "", secondNote);
	}
	public static ChartKey getRetirmentAgeChartKey(String secondNote) {
		return new ChartKey("Retirement Age", "Year", "Rate", new String[] {}, "", secondNote);
	}
	public static ChartKey getLengthOfWorkdayChartKey(String secondNote) {
		return new ChartKey("Length Of Workday*", "Year", "Relative Length", new String[] {}, "*relative compared to starting value", secondNote);
	}
	public static ChartKey getGDPAdjustedByPurchasingPowerBasedOnAverageConsumptionChartKey(String secondNote) {
		return new ChartKey("GDP Per capita adjusted by purchasing power*", "Years", "Money", new String[] {}, "*retroactively adjusted based on prices the final 5 years, calculation considers typical consumption ratios by time period", secondNote);
	}
	public static ChartKey getPopulationFinancialChartKey(String secondNote) {
		return new ChartKey("Population Financial Data", "Year", "Monetary Units", new String[] {"Average Balance", "Average Of Top 25%", "Average Of Bottom 25%"}, "", secondNote);
	}
	public static ChartKey getGovernmentTaxRatesChartKey(String secondNote) {
		return new ChartKey("Government Tax Rates", "Year", "Rate", new String[] {"Profit", "Income", "Sales", "Inheritance*"}, "*the inheritance tax rate is fixed", secondNote);
	}
	public static ChartKey getUnemploymentChartKey(String secondNote) {
		return new ChartKey("Unemployment", "Year", "Percent of People", new String[] {}, "", secondNote);
	}
	public static ChartKey getGovernmentRevenueChartKey(String secondNote) {
		return new ChartKey("Share of Government Revenue by Tax Type", "Year", "Revenue", new String[] {"Taxing Income", "Taxing Sales", "Taxing Inheritance*", "Taxing Profit"}, 
				"*in cases where people die without living children, this includes 100% \"dealth tax\"", secondNote);
	}
	public static ChartKey getEmployeesByIndustryChartKey(String secondNote) {
		return new ChartKey("Share of Employees by Industry", "Year", "Percent", new String[] {"Necessity", "Raw Material", "Discretionary"}, "", secondNote);
	}
	public static ChartKey getSenarioMoneyChartKey(String secondNote) {
		return new ChartKey("Proportion of total senario money held by People, Government, Industry", "Year", "Money", new String[] {"Government", "Industry", "People"}, "", secondNote);
	}
	public static ChartKey getProductionRateChartKey(String secondNote) {
		return new ChartKey("Production Rate*", "Year", "Production Rate", new String[] {}, "*a relative production rate change is imposed by the model and should be consistent between simulations", secondNote);
	}

}






















