package display;
import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import utility.Two;

/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/*
 * There is a known discrepancy between the input data order & resulting graph coloring compared to other (non stacked area) graphs
 */
public class StackedAreaChart extends Chart{
	private static final long serialVersionUID = -203076879443363082L;
	private static boolean renderChartAsPercentOutOf100 = true;
	private final int height;
	private Two<Number, Number>[][] dataToGraph;
	
	public StackedAreaChart(ChartKey chartKey, int width, int height, Two<Two<Number, Number>, Two<Number, Number>> range, Two<Number, Number>[][] dataToGraph) {
		super(chartKey, width, height, range);
		this.dataToGraph = dataToGraph;
		this.height = height;
	}
	
	public StackedAreaChart(ChartKey chartKey, int width, int height, Two<Number, Number>[][] dataToGraph) {
		this(chartKey, width, height, determineRange(dataToGraph), dataToGraph);
	}
	
	
	private static Two<Two<Number, Number>, Two<Number, Number>> determineRange(Two<Number, Number>[][] dataToGraph){
		
		int minXValue = 0;
		int maxXValue = Integer.MIN_VALUE;
		double minYValue = 0d;
		double maxYValue = 100d;
		
		if(renderChartAsPercentOutOf100) {
			for(int i = 0; i < dataToGraph.length; i++) {
				for(int j = 0; j < dataToGraph[i].length; j++) {
					if(dataToGraph[i][j] != null) {
						if(dataToGraph[i][j].getFirst().intValue() > maxXValue) {
							maxXValue = dataToGraph[i][j].getFirst().intValue();
						}
					}
				}
			}
		}

		
		return new Two<Two<Number, Number>, Two<Number, Number>>( new Two<Number, Number>(minXValue, maxXValue), new Two<Number, Number>(minYValue, maxYValue));
	}
	

	
	public void graphData() {
		if(this.dataToGraph != null) {
			Graphics graphics = this.bufferedImage.getGraphics();
			Color saveColor = graphics.getColor();
			ArrayList<ArrayList<Double>> listsOfYValues = new ArrayList<ArrayList<Double>>();
			ArrayList<Double> yValues = new ArrayList<Double>();
			Collections.addAll(yValues, new Double[xRange.getSecond().intValue()]);
			listsOfYValues.add(yValues);
			
			ArrayList<Integer> xValues = new ArrayList<Integer>();
			Collections.addAll(xValues, new Integer[xRange.getSecond().intValue()]);
	
			
			for(int i = 0; i < xValues.size(); i++) {
				xValues.set(i, (int)(( (double)(i - xRange.getFirst().intValue()) ) /xValueSpan * (width - graphBorderWidth * 2) + graphBorderWidth ) );
			}
			
			ArrayList<Integer> xValues2Reverse = new ArrayList<Integer>();
			xValues2Reverse.addAll(xValues);
			Collections.reverse(xValues2Reverse);
			
			xValues.addAll(xValues2Reverse);
			
			for(int i = 0; i < dataToGraph.length; i++) {
				yValues = new ArrayList<Double>();
				Collections.addAll(yValues, new Double[xRange.getSecond().intValue()]);
				listsOfYValues.add(yValues);
			}
			
			for(int i = 0; i < xRange.getSecond().intValue(); i++) {
				double total = 0;
				for(int j = 0; j < dataToGraph.length; j++) {
					total += dataToGraph[j][i].getSecond().doubleValue();
				}
				
				for(int j = 0; j <= dataToGraph.length; j++) {
					double percent = 0d;
					if(j == dataToGraph.length) {
						percent = 1d;
					}else if(j != 0) {
						percent = dataToGraph[j][i].getSecond().doubleValue()/total;
					}
					double val = (double) percent * ((double)height - (double)graphBorderWidth * 2d) ;
					if(j == 0 || j == dataToGraph.length) {					
						listsOfYValues.get(j).set(i , (double) (height - graphBorderWidth - (int) val ) );
					}else {
						listsOfYValues.get(j).set(i , (double) (listsOfYValues.get(j-1).get(i) - (int) val ) );
					}
				}
			}
			
			for(int i = 1; i < listsOfYValues.size(); i++) {
				graphics.setColor(colorsForData[(i-1)%22]);
				ArrayList<Integer> yValues4Graph = new ArrayList<Integer>();
				ArrayList<Integer> yValues2Reverse = new ArrayList<Integer>();
				
				for(Double d : listsOfYValues.get(i)) {
					yValues4Graph.add(d.intValue());
				}
				
				for(Double d : listsOfYValues.get(i-1)) {
					yValues2Reverse.add(d.intValue());
				}
				
				Collections.reverse(yValues2Reverse);
				
				yValues4Graph.addAll(yValues2Reverse);
	
				Integer[] xVals = new Integer[xValues.size()];
				xVals = xValues.toArray(xVals);
				int[] xArr = Arrays.stream(xVals).mapToInt(Integer::intValue).toArray();
				
				Integer[] yVals = new Integer[xValues.size()];
				yVals = yValues4Graph.toArray(yVals);
				int[] yArr = Arrays.stream(yVals).mapToInt(Integer::intValue).toArray();
				
				graphics.fillPolygon(xArr, yArr, xValues.size());
				graphics.drawPolygon(xArr, yArr, xValues.size());
			}
			
			
			graphics.setColor(saveColor);
		}
	}
	
	protected void nullGraphData() {
		this.dataToGraph = null;
	}
}
