package helper;
import java.util.Random;
import people.EConsumable;
import people.EReproductiveGender;
import people.RomanticRelationship;
import people.THE_WORLD;
import people.Government.GovernmentRecord;
import utility.IReturnsItem;
import utility.LazyUpdateUpToOncePerTick;
import utility.Three;
import utility.Two;

/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 * The DeterminationHelper class contains many of the program's key values and is also used for determining outcomes that depend on chance/probability
 * 
 * @author Ian Stuart
 */
public class DeterminationHelper {
	public static final EConsumable[] INDUSTRY_ORDER = new EConsumable[] { EConsumable.RAW_MATERIAL, EConsumable.NECESSITY, EConsumable.DISCRETIONARY };
	
	public static final double LIFE_EXPECTANCY = 77;
	public static final int LEGAL_ADULT_AGE = 18;
	
	/* Reproduction */
	public static final double EXPECTED_RATIO_OF_MALE_BIRTHS = 0.5f;
	public static final int MAX_NUMBER_OF_CHILDREN = 4;
	public static final int MAX_FERTILE_AGE = 52;
	public static final double CHANCE_OF_FERTILE_COUPLE_GETTING_PREGNANT = .012;
	public static final double INCREASE_CHANCE_OF_COUPLE_REPRODUCING_BY = 1.4;
	public static final double CHILDREN_TO_ADULT_RATIO_THRESHOLD_TO_BOOST_REPRODUCTION = 0.13;
	public static final double REDUCE_CHANCE_OF_COUPLE_REPRODUCING_BY = 0.65;
	public static final double CHILDREN_TO_ADULT_RATIO_THRESHOLD_TO_REDUCE_REPRODUCTION = 0.45;
	public static final int TIME_TO_WAIT_BETWEEN_REPRODUCTION_RATE_ADJUSTMENTS = 16;
	public static final int NUMBER_OF_GENERATIONS_INCEST_TABOO_EXTENDS = 3;
	public static final double CHANCE_OF_COMPATIBLE_PAIR_ENTERING_MUTUAL_ROMANTIC_RELATIONSHIP_DURING_SENARIO_INITIALIZATION = 0.08;
	public static final double CHANCE_OF_COMPATIBLE_PAIR_ENTERING_MUTUAL_ROMANTIC_RELATIONSHIP = .02666;
	public static final int[] AGE_DIFFERENCE_FILTERS = new int[] {3, 6, 9, 12, 15};
	public static final Two<Integer, Integer> INDEX_AND_AGE_OF_MATURITY_FOR_AGE_DIFFERENCE_FILTER = new Two<Integer, Integer>(2, 23);
	public static final double CHANCE_OF_INITIAL_COUPLE_HAVING_HAD_A_KID_IN_A_GIVEN_YEAR = 0.085;
	
	public static final double CHANCE_OF_DYING_FROM_FAILING_TO_MEET_NEEDS = 0.08333;
	public static final double CHANCE_OF_NON_ESSIENTIAL_EMPLOYEE_SWITCHING_TO_ESSENTIAL_INDUSTRY_IN_AN_EMERGENCY = 0.5;
	public static final int INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE = 6;
	
	/* Industry */
	public static final double MAX_ADJUSTED_PRODUCTION_RATE = 25d;
	public static final double ADJUST_ADJUST_PRODUCTION_RATE_BY = 0.065;
	public static final double MULTIPLIER_TO_DETERMINE_TARGET_INPUT_MATERIAL_INVENTORY = 2.5d;
	public static final double MULTIPLIER_OF_EXPENSES_TO_DETERMINE_DIVISOR_OF_MONEY_FOR_DETERMINING_FINANCIAL_STANDING = 8;
	public static final double MULTIPLIER_OF_DEMAND_TO_DETERMINE_TARGET_PRODUCT_INVENTORY = 10;
	public static final double FINANCIAL_STANDING_CAP = 5d;
	public static final double INITIAL_WAIT_BEFORE_DESCRETIONARY_INDUSTRY_CHANGES_THE_NUMBER_OF_JOBS = 36;
	public static final int TIME_TO_WAIT_BETWEEN_CHANGES_IN_THE_NUMBER_OF_JOBS = 4;
	public static final double MINIMUM_FINANACIAL_STANDING_FOR_DESCRETIONARY_INDUSTRY_TO_ADD_JOBS = 0.15;
	public static final Two<Double, Double> AVERAGE_FINANCIAL_STANDING_AND_PRODUCT_QUANTITY_THRESHOLD_TO_SIGNAL_INSUFFICIENT_ASSETS = new Two<Double, Double>(0.01, 10d);
	public static final int MINIMUM_TIME_BETWEEN_SALE_PRICE_CHANGES = 8;
	

	public static final double MULTIPLICAR_TO_DETERMINE_LOWEST_AND_SECOND_LOWEST_SALE_PRICE = 0.8;
	public static final double AMOUNT_TO_ADD_TO_DETERMINE_SECOND_LOWEST_AND_MIDDLE_SALE_PRICE = 0.5;
	
	public static final double 			LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD = 3;
	public static final double SECOND_LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD = 1.15;
	public static final double SECOND_HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD = 0.6;
	public static final double HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD = 0.45;
	
	public static final double AMOUNT_TO_ADD_TO_DETERMINE_SECOND_HIGHEST_SALE_PRICE = 1;
	public static final double AMOUNT_TO_ADD_TO_DETERMINE_HIGHEST_SALE_PRICE = 2.25;
	public static final double SECOND_HIGHEST_SALE_PRICE_PERCENT_OF_TARGET_INVENTORY_OTHER_POSSIBLE_THRESHOLD = 0.1;
	public static final double HIGHEST_SALE_PRICE_PERCENT_OF_TARGET_INVENTORY_OTHER_POSSIBLE_THRESHOLD = 0.01;
	
	public static final double RATIO_OF_TARGET_INVENTORY_FOR_THESHOLD_TO_REMOVE_JOBS = 1.3;
	public static final double RATIO_OF_TARGET_INVENTORY_FOR_THESHOLD_TO_ADD_JOBS = 0.9;
	public static final double RATIO_OF_TARGET_INVENTORY_CEILING_FOR_THESHOLD_TO_CALCULATE_JOBS_TO_ADD_WITH_CURRENT_INSTEAD_OF_AVERAGE_DEMAND = 0.7;
	public static final int DIVISOR_OF_AMOUNT_NEEDED_TO_MEET_TARGET_INVENTORY_TO_CALCULATE_ADDITIONAL_JOBS_BASED_ONLY_ON_THIS = 150;
	public static final Two<Double, Double> RATIO_OF_TARGET_INVENTORY_AND_RATIO_OF_PRODUCTION_OVER_AVERAGE_DEMAND_FOR_THESHOLD_TO_REMOVE_JOBS = new Two<Double, Double>(0.6, 1.25);
	public static final Two<Double, Integer> MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_REMOVED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES = new Two<Double, Integer>(0.04, 50);
	public static final Two<Double, Integer> MAXIMUM_PERCENT_OF_JOBS_THAT_DISCRETIONARY_INDUSTRY_CAN_ADD_AT_ONCE_IF_IT_HAS_ATLEAST_THIS_MANY_EMPLOYEES = new Two<Double, Integer>(0.006, 1000);

	public static final Two<Double, Integer> MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_ADDED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES = new Two<Double, Integer>(0.12, 50);
	
	/* Personal Consumption */
	public static final double REQUIRED_NECESSITY_CONSUMABLE_FOR_FIRST_ADULT_OF_A_HOME = 8d;
	public static final double ADDITIONAL_NECESSITY_CONSUMABLE_REQUIRED_FOR__THE_SECOND_ADULT_OF_A_HOME_WHERE_THE_TOTAL_IS_ROUNDED_DOWN = 6.4d;
	public static final double ADDITIONAL_NECESSITY_CONSUMABLE_REQUIRED_FOR_EACH_PERSON_UNDER_18_WHERE_THE_TOTAL_FOR_A_HOME_IS_ROUNDED_DOWN = 5.4d;
	public static final int AMOUNT_OF_MONEY_PEOPLE_MAKE_SURE_TO_SAVE_WHEN_SPENDING_MONEY_ON_DESCRETIONARIES = 1600;
	
	/* Government */
	public static final int AMOUNT_TO_PAY_TO_ASSIST_AND_IMPOVERISHED_INDUSTRY = 2400;
	
	public static final int INITIAL_PARENT_TO_A_NEW_CHILD_PAYMENT = 250;
	public static final int MAXIMUM_PARENT_TO_A_NEW_CHILD_PAYMENT = 1100;
	public static final int PARENT_TO_A_NEW_CHILD_PAYMENT_ADJUSTMENT_INCREMENTS = 25;
	public static final Two<Integer, Integer> PARENT_TO_A_NEW_CHILD_PAYMENT_AVERAGE_PERSONAL_BALANCE_THRESHOLDS_TO__LOWER_AND_RAISE__ = new Two<Integer, Integer>(1650, 1550);
	public static final int MINIMUM_TIME_BETWEEN_PARENT_TO_A_NEW_CHILD_PAYMENT_ADJUSTMENTS = 12;
	
	public static final double 													INITIAL_SALES_TAX_RATE = 0.04;
	public static final double MINIMUM_ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_REQUIRED_TO_RAISE_SALES_TAX = 0.3;
	public static final int 									MINIMUM_TIME_BETWEEN_CHANGES_TO_SALES_TAX = 20;
	public static final Three<Double, Double, Double> RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX = new Three<Double, Double, Double>(0.5, 0.08, 0.02);
	public static final Three<Double, Double, Double> RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX = new Three<Double, Double, Double>(4.5, 0.03, 0.01);
	
	public static final double 													INITIAL_PROFIT_TAX_RATE = 0.5;
	public static final double ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_THRESHOLD_TO_CHANGE_PROFIT_TAX_RATE = 0.6;
	public static final int 							MINIMUM_TIME_BETWEEN_CHANGES_TO_PROFIT_TAX_RATE = 16;
	public static final Three<Double, Double, Double> RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX = new Three<Double, Double, Double>(0.8, 0.8, 0.1);
	public static final Three<Double, Double, Double> RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX = new Three<Double, Double, Double>(4.5, 0.1, 0.1);
	
	public static final double 													INITIAL_INCOME_TAX_RATE = 0.1;
	public static final int 							MINIMUM_TIME_BETWEEN_CHANGES_TO_INCOME_TAX_RATE = 30;
	public static final Three<Double, Double, Double> RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX = new Three<Double, Double, Double>(0.4, 0.13, 0.02);
	public static final Three<Double, Double, Double> RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_INCOME_TAX = new Three<Double, Double, Double>(1.5, 0.07, 0.01);
	public static final double INITIAL_INHERITANCE_TAX = 0.65;
	public static final int INITIAL_RETIREMENT_AGE = 70;
	public static final int MINIMUM_RETIREMENT_AGE = 55;
	public static final int AMOUNT_TO_PAY_FOR_SOCIAL_SECURITY = 120;
	public static final int MINIMUM_WAGE = 210;
	@SuppressWarnings("unchecked")
	public static final Three<Double, Integer, Integer>[] RETIREMENT_AGE__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS = 
																													new Three[]	{ 	new Three<Double, Integer, Integer>(0.15, 18, 2),
																																	new Three<Double, Integer, Integer>(0.10, 18, 1),
																																	new Three<Double, Integer, Integer>(0.06, 36, 1)
																																};
	public static final double MINIMUM_LENGTH_OF_WORKDAY = 0.5;
	@SuppressWarnings("unchecked")
	public static final Three<Double, Integer, Double>[] WORKDAY_LENGTH__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS = 
																													new Three[]	{ 	new Three<Double, Integer, Double>(0.18, 15, 0.04),
																																	new Three<Double, Integer, Double>(0.09, 15, 0.02),
																																	new Three<Double, Integer, Double>(0.06, 30, 0.01)
																																};
	
	/* Scenario Duration Times */
	public static final int TIME_FOR_SENARIO_TO_SEE_EARLY_INDUSTRY_SHIFT = 120 * 12 + 1;
	public static final int TIME_FOR_SENARIO_IN_RANGE_OF_EXPONENTIAL_PRODUCTION_RATE_INCREASE = 230 * 12 + 1;
	public static final int TIME_FOR_SENARIO_EXTENDING_PAST_PRODUCTION_RATE_AND_POPULATION_PLATEAU = 450 * 12 + 1;
	
	/*The potential for a temporary fall from a '1' to a '0' sale price may not be especially pragmatic
	 * and it is not meant to be in the target time range of the scenario, but there is still some interesting
	 * emergence after this point (and it is useful for stability), therefore it is allowed. 
	 */
	public static final boolean DOES_ALLOW_A_ZERO_SALE_PRICE = false;
	public static final int AMOUNT_OF_DISCRETIONARY_A_PERSON_GETS_IF_FREE = 100;
	
	public static final int MINIMUM_ADULT_STARTING_MONEY = 400;
	public static final int EXPECTED_ADULT_EXTRA_STARTING_MONEY_IF_UNIFORM_DISTRIBUTION = 200;
	
	private static final Random random = new Random();

	private static final LazyUpdateUpToOncePerTick<Double> populationPlateauReproductionRateAdjustment = new LazyUpdateUpToOncePerTick<Double>(
																									new IReturnsItem<Double>() { 
																										public Double getItem() {
																											return getPopulationPlateauReproductionRateAdjustment(8000, 150, 0.011);
																								  }});
	
	private static final LazyUpdateUpToOncePerTick<Double> adjustedChanceOfFertileCoupleGettingPregnant = new LazyUpdateUpToOncePerTick<Double>(
																										new IReturnsItem<Double>() { 
																											public Double getItem() {
																												return getAdjustedChanceOfFertileCoupleGettingPregnant();
																									}});

	public DeterminationHelper() {}
	
	public static boolean getsPregnant(RomanticRelationship romanticRelationship) {
		if(romanticRelationship.canReproduce() && 
						random.nextDouble() < adjustedChanceOfFertileCoupleGettingPregnant.get()) {
			return true;
		}
		return false;
	}

	public static int getAdultStartingMoney() { 
		return MINIMUM_ADULT_STARTING_MONEY + 
				random.nextInt(EXPECTED_ADULT_EXTRA_STARTING_MONEY_IF_UNIFORM_DISTRIBUTION * 2 + 1);
	}
	
	public static EReproductiveGender getEReproductiveGender() {
		if( random.nextDouble() < EXPECTED_RATIO_OF_MALE_BIRTHS) {
			return EReproductiveGender.MALE;
		}
		return EReproductiveGender.FEMALE;
	}
	
	public static int getAdultAgeInMonths() {
		int range = (int)((LIFE_EXPECTANCY - LEGAL_ADULT_AGE) * THE_WORLD.UNITS_OF_TIME_IN_YEAR * 0.75);
		int startingAge = (LEGAL_ADULT_AGE * THE_WORLD.UNITS_OF_TIME_IN_YEAR) + random.nextInt(range);
		return startingAge;
	}
	
	public static boolean doesDieFromFailingToMeetNeeds() { return rollDice() < CHANCE_OF_DYING_FROM_FAILING_TO_MEET_NEEDS; }
		
	/*
	 * A general point of access method for returning a pseudo random value for determining chance/probability outcomes. 
	 */
	public static double rollDice() {return random.nextDouble();}
	
	private static double getPopulationPlateauReproductionRateAdjustment(int populationToStartReducingReproductionRate, int populationIncrements, double rateReductionIncrements) {	
		return Math.max(0, 1 - Math.max(0, (THE_WORLD.getInstance().getPeople().size() - populationToStartReducingReproductionRate)/populationIncrements) * rateReductionIncrements);
	}
	
	private static double getAdjustedChanceOfFertileCoupleGettingPregnant() {
		GovernmentRecord populationRecord = THE_WORLD.getInstance().getGovernment().getLastPopulationRecord();
		double chance = CHANCE_OF_FERTILE_COUPLE_GETTING_PREGNANT;
		if(populationRecord != null && populationRecord.getAdultPopulation() > 0) {
			double adultPopulation = populationRecord.getAdultPopulation();
			if((double)populationRecord.getUnder18Population()/(double)adultPopulation <= DeterminationHelper.CHILDREN_TO_ADULT_RATIO_THRESHOLD_TO_BOOST_REPRODUCTION){
				chance *= INCREASE_CHANCE_OF_COUPLE_REPRODUCING_BY;
			}else if((double)populationRecord.getUnder18Population()/(double)adultPopulation > DeterminationHelper.CHILDREN_TO_ADULT_RATIO_THRESHOLD_TO_REDUCE_REPRODUCTION){
				chance *= REDUCE_CHANCE_OF_COUPLE_REPRODUCING_BY;
			}
		}
		return populationPlateauReproductionRateAdjustment.get() * chance;
	}
	
}

