package helper;
import java.util.ArrayList;
import utility.ExtendedCountSort;
import utility.Four;

/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

public class FinancialStatisticsHelper {
	private ExtendedCountSort extendedCountSort;
	//private int total = 0;
	
	public FinancialStatisticsHelper(int commonRange) {
		this.extendedCountSort = new ExtendedCountSort(commonRange);
	}
	
	public void zeroValues() { 
		this.extendedCountSort.zeroValues();
		//this.total = 0;
	}
	
	public void submitValue(int value) {
		this.extendedCountSort.addValue(value);
		//this.total += value;
	}
	
	
	/*
	 * This has not been tested...
	 */
	public BoxplotData getBoxplotData(){
		ArrayList<Integer> lowExtremeOutliers = new ArrayList<Integer>();
		ArrayList<Integer> lowNonExtremeOutliers = new ArrayList<Integer>();
		Integer lowNonOutlier = Integer.MAX_VALUE;
		Integer lowerFourth;
		Integer median;
		Integer upperFourth;
		Integer highNonOutlier = Integer.MIN_VALUE;
		ArrayList<Integer> highNonExtremeOutliers = new ArrayList<Integer>();
		ArrayList<Integer> highExtremeOutliers = new ArrayList<Integer>();
		
		int[] array = this.extendedCountSort.getSortedArrayOfCurrentValues();
		
		if(array.length % 2 == 1) {
			median = array[array.length/2];
			if((array.length/2 + 1)%2 == 1) {
				lowerFourth = array[(array.length/2 + 1)/2];
				upperFourth = array[array.length - 1 - (array.length/2 + 1)/2];
			}else {
				lowerFourth = (array[(array.length/2 + 1)/2 - 1] + array[(array.length/2 + 1)/2])/2;
				upperFourth = (array[array.length - 1 - (array.length/2 + 1)/2 - 1] + array[array.length - 1 - (array.length/2 + 1)/2 - 1])/2;
			}
		}else {
			median = (array[array.length/2 - 1] + array[array.length/2])/2;
			if(array.length/2%2 == 1) {
				lowerFourth = array[array.length/4];
				upperFourth = array[array.length - 1 - array.length/4];
			}else {
				lowerFourth = (array[array.length/4 - 1] + array[array.length/4])/2;
				upperFourth = (array[array.length - 1 - array.length/4 - 1] + array[array.length - 1 - array.length/4 - 1])/2;
			}
		}
		
		int fourthSpread = upperFourth - lowerFourth;
		int minNonExtremeOutlierLimit = lowerFourth - fourthSpread * 3;
		int minNonOutlierLimit = lowerFourth - (int)(fourthSpread * 1.5);
		int maxNonOutLierLimit = upperFourth + (int)(fourthSpread * 1.5);
		int maxNonExtremeOutLierLimit = upperFourth + fourthSpread * 3;
		for(int i = 0; i < array.length; i++) {
			if(array[i] < minNonExtremeOutlierLimit) {
				lowExtremeOutliers.add(array[i]);
			}else if(array[i] < minNonOutlierLimit) {
				lowNonExtremeOutliers.add(array[i]);
			}else if(array[i] > maxNonExtremeOutLierLimit) {
				highExtremeOutliers.add(array[i]);
			}else if(array[i] > maxNonOutLierLimit) {
				highNonExtremeOutliers.add(array[i]);
			}else{
				if(array[i] < lowNonOutlier) {
					lowNonOutlier = array[i];
				}
				if(array[i] > highNonOutlier) {
					highNonOutlier = array[i];
				}
			}
		}
		
		
		
		return new BoxplotData(getIntegerArray(lowExtremeOutliers), getIntegerArray(lowNonExtremeOutliers), lowNonOutlier, lowerFourth, median, 
				upperFourth, highNonOutlier, getIntegerArray(highNonExtremeOutliers), getIntegerArray(highExtremeOutliers));
	}
	
	private Integer[] getIntegerArray(ArrayList<Integer> arrayList) {
		Integer[] vals = new Integer[arrayList.size()];
		vals = arrayList.toArray(vals);
		return vals;
	}
	
	public Four<Integer, Integer, Integer, Integer> getAverage_AverageOfTopQuarter_AndAverageOfBottomQuarter_TotalPersonalMoney(){
		int[] array = this.extendedCountSort.getSortedArrayOfCurrentValues();
		int forAverageOfBottomQuarter = 0;
		int average = 0;
		int forAverageOfTopQuarter = 0;
		int lTotal = 0;
		int quarterLength = array.length/4;
		if(quarterLength > 1) {
			for(int i = 0; i < array.length; i++) {
				lTotal += array[i];
				if(i < quarterLength) {
					forAverageOfBottomQuarter += array[i];
				}else if(i >= array.length - quarterLength) {
					forAverageOfTopQuarter += array[i];
				}
			}
		}else if (array.length > 0){
			forAverageOfBottomQuarter = array[0];
			forAverageOfTopQuarter = array[array.length - 1];
			for(int i = 0; i < array.length; i++) {
				lTotal += array[i];
			}
		}
		if(quarterLength == 0) {
			quarterLength = 1;//incase it is zero
		}		
		forAverageOfBottomQuarter /= quarterLength;
		average = lTotal/(array.length > 0 ? array.length : 1);
		forAverageOfTopQuarter /= quarterLength;
		
		return new Four<Integer, Integer, Integer, Integer>(average, forAverageOfTopQuarter, forAverageOfBottomQuarter, lTotal);
		
	}

	
	public class BoxplotData{
		private final Number[] lowExtremeOutliers;
		private final Number[] lowNonExtremeOutliers;
		private final Number lowNonOutlier;
		private final Number lowerFourth;
		private final Number median;
		private final Number upperFourth;
		private final Number highNonOutlier;
		private final Number[] highNonExtremeOutliers;
		private final Number[] highExtremeOutliers;

		public BoxplotData(Number[] lowExtremeOutliers, Number[] lowNonExtremeOutliers, Number lowNonOutlier, Number lowerFourth, Number median, 
										Number upperFourth, Number highNonOutlier, Number[] highNonExtremeOutliers, Number[] highExtremeOutliers) {
			this.lowExtremeOutliers = lowExtremeOutliers;
			this.lowNonExtremeOutliers = lowNonExtremeOutliers;
			this.lowNonOutlier = lowNonOutlier;
			this.lowerFourth = lowerFourth;
			this.median = median;
			this.upperFourth = upperFourth;
			this.highNonOutlier = highNonOutlier;
			this.highNonExtremeOutliers = highNonExtremeOutliers;
			this.highExtremeOutliers = highExtremeOutliers;
		}
		
		public Number[] getLowExtremeOutliers() {return this.lowExtremeOutliers; }
		public Number[] getLowNonExtremeOutliers() {return this.lowNonExtremeOutliers; }
		public Number getLowNonOutlier() { return this.lowNonOutlier; }
		public Number getLowerFourth() { return this.lowerFourth; }
		public Number getMedian() { return this.median;}
		public Number getUpperFourth() { return this.upperFourth;}
		public Number getHighNonOutlier() { return this.highNonOutlier;}
		public Number[] getHighNonExtremeOutliers() { return this.highNonExtremeOutliers;}
		public Number[] getHighExtremeOutliers() { return this.highExtremeOutliers;}

	}
	
}
