package helper;
import java.util.ArrayList;
import people.BiologicalRelationship;
import people.EReproductiveGender;
import people.Person;
import people.RomanticRelationship;
import people.THE_WORLD;
import simulate.PersonMill;
import utility.ObscureWhichIsWhich;

/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

public class RomanticRelationshipHelper {	
	public RomanticRelationshipHelper() {}
	
	/*
	 * This method is intended to be called each time step to potentially initialize new relationships, see if people become pregnant, and initialize new people born.
	 */
	public void UpdateRelationships(ArrayList<Person> people) {
		ArrayList<RomanticRelationship> romanticRelationships = new ArrayList<RomanticRelationship>();
		ArrayList<Person> singleAdultWomen = new ArrayList<Person>();
		ArrayList<Person> singleAdultMen = new ArrayList<Person>();
		for(int i = 0; i < people.size(); i++) {
			if(people.get(i).getRomanticRelationship() != null) {
				if(!romanticRelationships.contains(people.get(i).getRomanticRelationship())) {
					romanticRelationships.add(people.get(i).getRomanticRelationship());
				}
			}else if(people.get(i).isAdult()){
				if(people.get(i).getReproductiveGender() == EReproductiveGender.MALE) {
					singleAdultMen.add(people.get(i));
				}else {
					singleAdultWomen.add(people.get(i));
				}
			}
		}

		PersonMill personMill = new PersonMill();
		for(int i = 0; i < romanticRelationships.size(); i++) {
			if(romanticRelationships.get(i).getPregnancy() != null && romanticRelationships.get(i).getPregnancy().isInLabor()) {
				if( ( romanticRelationships.get(i).getPeople().getFirst().isAlive() && romanticRelationships.get(i).getPeople().getFirst().getReproductiveGender() == EReproductiveGender.FEMALE ) ||
								romanticRelationships.get(i).getPeople().getSecond().isAlive() && romanticRelationships.get(i).getPeople().getSecond().getReproductiveGender() == EReproductiveGender.FEMALE) {
					personMill.giveBirth(romanticRelationships.get(i));
				}
			}else if(romanticRelationships.get(i).getPregnancy() == null &&
					romanticRelationships.get(i).getSharedChildren().size() < DeterminationHelper.MAX_NUMBER_OF_CHILDREN &&
					romanticRelationships.get(i).canReproduce() &&
					DeterminationHelper.getsPregnant(romanticRelationships.get(i))) {
				romanticRelationships.get(i).setPregnant();
			}
		}
		ObscureWhichIsWhich<ArrayList<Person>> obscureWhichIsWhich = new ObscureWhichIsWhich<ArrayList<Person>>(singleAdultMen, singleAdultWomen);
		ArrayList<Person> singleListA = obscureWhichIsWhich.getItemA();
		ArrayList<Person> singleList1 = obscureWhichIsWhich.getItem1();	
		int limit = 4;		
		
		for(int i = singleListA.size() - 1; i >= 0; i--) {
			int opportunity = 0;
			for(int h = 0; h < DeterminationHelper.AGE_DIFFERENCE_FILTERS.length; h++) {
				if(opportunity >= limit) {
					break;
				}
				for(int k = singleList1.size() -1; k >= 0; k--) {
					if( doFallWithinEachOthersAgeFilter(singleListA.get(i), singleList1.get(k), h) ) {		
						if(DeterminationHelper.rollDice() < DeterminationHelper.CHANCE_OF_COMPATIBLE_PAIR_ENTERING_MUTUAL_ROMANTIC_RELATIONSHIP){
							if(!BiologicalRelationship.areRelated(singleListA.get(i), singleList1.get(k), DeterminationHelper.NUMBER_OF_GENERATIONS_INCEST_TABOO_EXTENDS)){
								initializePairIntoARomanticRelationship(singleListA.get(i), singleList1.get(k));
								singleList1.remove(k);
								singleListA.remove(i);
								opportunity = limit;
								break;
							}
						}else {
							opportunity++;
						}
					}
				}
			}
		}
	}
	
	/* This method is only meant to be used at time step 0.
	 * 
	 * Initializes list of adults into romanticRelationships and instantiates any resulting children
	 */
	public void initializeAdultPopulation(ArrayList<Person> people) {
		ArrayList<RomanticRelationship> romanticRelationships = new ArrayList<RomanticRelationship>();
		ArrayList<Person> singleAdultWomen = new ArrayList<Person>();
		ArrayList<Person> singleAdultMen = new ArrayList<Person>();
		for(int i = 0; i < people.size(); i++) {
			if(people.get(i).isAdult()){
				if(people.get(i).getReproductiveGender() == EReproductiveGender.MALE) {
					singleAdultMen.add(people.get(i));
				}else {
					singleAdultWomen.add(people.get(i));
				}
			}
		}
		for(int i = 0; i < DeterminationHelper.AGE_DIFFERENCE_FILTERS.length; i++) {
			for(int j = singleAdultWomen.size() - 1; j >= 0; j--) {
				for(int k = singleAdultMen.size() -1; k >= 0; k--) {
					if( doFallWithinEachOthersAgeFilter(singleAdultWomen.get(j), singleAdultMen.get(k), i) && 
								DeterminationHelper.rollDice() < DeterminationHelper.CHANCE_OF_COMPATIBLE_PAIR_ENTERING_MUTUAL_ROMANTIC_RELATIONSHIP_DURING_SENARIO_INITIALIZATION &&
								!BiologicalRelationship.areRelated(singleAdultWomen.get(j), singleAdultMen.get(k), DeterminationHelper.NUMBER_OF_GENERATIONS_INCEST_TABOO_EXTENDS)){
						initializePairIntoARomanticRelationship(singleAdultWomen.get(j), singleAdultMen.get(k));
						romanticRelationships.add(singleAdultWomen.get(j).getRomanticRelationship());
						singleAdultWomen.remove(j);
						singleAdultMen.remove(k);
						break;
					}
				}
			}
		}
		for(int i = 0; i < romanticRelationships.size(); i++) {
			int yearsIntoAdultHood = 0;
			if(romanticRelationships.get(i).getPeople().getFirst().getAge() < romanticRelationships.get(i).getPeople().getSecond().getAge()) {
				yearsIntoAdultHood = romanticRelationships.get(i).getPeople().getFirst().getAge() - DeterminationHelper.LEGAL_ADULT_AGE;
			}else{
				yearsIntoAdultHood = romanticRelationships.get(i).getPeople().getSecond().getAge() - DeterminationHelper.LEGAL_ADULT_AGE;
			}
			for(int j = 0; j < yearsIntoAdultHood - 1 && j < DeterminationHelper.LEGAL_ADULT_AGE; j++) {
				if(DeterminationHelper.rollDice() < DeterminationHelper.CHANCE_OF_INITIAL_COUPLE_HAVING_HAD_A_KID_IN_A_GIVEN_YEAR && romanticRelationships.get(i).getSharedChildren().size() < DeterminationHelper.MAX_NUMBER_OF_CHILDREN) {
					PersonMill personMill = new PersonMill();
					int months = (int)(DeterminationHelper.rollDice() * (double)THE_WORLD.UNITS_OF_TIME_IN_YEAR);
					personMill.giveBirthInThePast(romanticRelationships.get(i), j * THE_WORLD.UNITS_OF_TIME_IN_YEAR + months);
				}
			}
		}
	}
	
	private boolean doFallWithinEachOthersAgeFilter(Person firstPerson, Person secondPerson, int currentAgeFilterIndex) {
		return Math.abs(firstPerson.getAge() - secondPerson.getAge()) <= DeterminationHelper.AGE_DIFFERENCE_FILTERS[currentAgeFilterIndex] &&
				Math.abs(firstPerson.getAge() - secondPerson.getAge()) > ( currentAgeFilterIndex > 0 ? DeterminationHelper.AGE_DIFFERENCE_FILTERS[currentAgeFilterIndex - 1] : 0 ) &&
				( ( firstPerson.getAge() > DeterminationHelper.INDEX_AND_AGE_OF_MATURITY_FOR_AGE_DIFFERENCE_FILTER.getSecond() && 
						secondPerson.getAge() > DeterminationHelper.INDEX_AND_AGE_OF_MATURITY_FOR_AGE_DIFFERENCE_FILTER.getSecond() ) ||
						currentAgeFilterIndex < DeterminationHelper.INDEX_AND_AGE_OF_MATURITY_FOR_AGE_DIFFERENCE_FILTER.getFirst());
	}
	
	private void initializePairIntoARomanticRelationship(Person firstPerson, Person secondPerson) {
		RomanticRelationship romanticRelationship = new RomanticRelationship(firstPerson, secondPerson);
		firstPerson.setRomanticRelationship(romanticRelationship);
		secondPerson.setRomanticRelationship(romanticRelationship);
		firstPerson.getHome().addPerson(secondPerson);
		secondPerson.setHome(firstPerson.getHome());
	}
}











