package people;
import helper.DeterminationHelper;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 * Represents specific biological relationship between two people.
 * 
 * @author Ian Stuart
 */
public class BiologicalRelationship extends AbstractRelationship{
	private EBiologicalRelationship parentRelationship;
	private EBiologicalRelationship childRelationship;
	
	public BiologicalRelationship(Person parent, Person child) {
		super(parent, child);
		if(parent.getAge() >= child.getAge() + DeterminationHelper.LEGAL_ADULT_AGE) {
			if(child.getReproductiveGender() == EReproductiveGender.MALE) {
				if(!parent.hasBiologicalRelationship(EBiologicalRelationship.FIRST_SON)) {
					childRelationship = EBiologicalRelationship.FIRST_SON;
				}else if(!parent.hasBiologicalRelationship(EBiologicalRelationship.SECOND_SON)) {
					childRelationship = EBiologicalRelationship.SECOND_SON;
				}else if(!parent.hasBiologicalRelationship(EBiologicalRelationship.THIRD_SON)) {
					childRelationship = EBiologicalRelationship.THIRD_SON;
				}else {
					childRelationship = EBiologicalRelationship.NONE;
				}
			}else if(child.getReproductiveGender() == EReproductiveGender.FEMALE) {
				if(!parent.hasBiologicalRelationship(EBiologicalRelationship.FIRST_DAUGHTER)) {
					childRelationship = EBiologicalRelationship.FIRST_DAUGHTER;
				}else if(!parent.hasBiologicalRelationship(EBiologicalRelationship.SECOND_DAUGHTER)) {
					childRelationship = EBiologicalRelationship.SECOND_DAUGHTER;
				}else if(!parent.hasBiologicalRelationship(EBiologicalRelationship.THIRD_DAUGHTER)) {
					childRelationship = EBiologicalRelationship.THIRD_DAUGHTER;
				}else {
					childRelationship = EBiologicalRelationship.NONE;
				}
			}else {
				childRelationship = EBiologicalRelationship.NONE;
			}
			if(parent.getReproductiveGender() == EReproductiveGender.MALE) {
				parentRelationship = EBiologicalRelationship.FATHER;
			}else if(parent.getReproductiveGender() == EReproductiveGender.FEMALE){
				parentRelationship = EBiologicalRelationship.MOTHER;
			}else {
				parentRelationship = EBiologicalRelationship.NONE;
			}
		}else {
			parentRelationship = EBiologicalRelationship.NONE;
			childRelationship = EBiologicalRelationship.NONE;
		}
	}
	
	public EBiologicalRelationship getBiologicalRelationship(Person person) {
		if(person == this.firstPerson && this.firstPerson.getAge() > this.secondPerson.getAge()) {
			return childRelationship;
		}else {
			return parentRelationship;
		}
	}
	
	public static boolean areRelated(Person personOne, Person personTwo, int depthToCheck) {
		if(depthToCheck <= 0) {
			return false;
		}
		if(personOne.hasBiologicalRelationship(EBiologicalRelationship.FATHER)){
			if(personOne.getBiologicalRelationship(EBiologicalRelationship.FATHER).getTheOtherPerson(personOne) == personTwo) {
				return true;
			}
			if(personTwo.hasBiologicalRelationship(EBiologicalRelationship.FATHER)){
				if(personOne.getBiologicalRelationship(EBiologicalRelationship.FATHER).getTheOtherPerson(personOne) == 
									personTwo.getBiologicalRelationship(EBiologicalRelationship.FATHER).getTheOtherPerson(personTwo)) {
					return true;
				}
				if(areRelated(personOne.getBiologicalRelationship(EBiologicalRelationship.FATHER).getTheOtherPerson(personOne),
								personTwo.getBiologicalRelationship(EBiologicalRelationship.FATHER).getTheOtherPerson(personTwo),
								depthToCheck - 1)) {
					return true;
				}
			}
			if(personTwo.hasBiologicalRelationship(EBiologicalRelationship.MOTHER) &&
					areRelated(personOne.getBiologicalRelationship(EBiologicalRelationship.FATHER).getTheOtherPerson(personOne),
							personTwo.getBiologicalRelationship(EBiologicalRelationship.MOTHER).getTheOtherPerson(personTwo),
							depthToCheck - 1)) {
				return true;
			}
		}
		
		if(personOne.hasBiologicalRelationship(EBiologicalRelationship.MOTHER)){
			if(personOne.getBiologicalRelationship(EBiologicalRelationship.MOTHER).getTheOtherPerson(personOne) == personTwo) {
				return true;
			}
			if(personTwo.hasBiologicalRelationship(EBiologicalRelationship.MOTHER)){
				if(personOne.getBiologicalRelationship(EBiologicalRelationship.MOTHER).getTheOtherPerson(personOne) == 
								personTwo.getBiologicalRelationship(EBiologicalRelationship.MOTHER).getTheOtherPerson(personTwo)) {
					return true;
				}
				if(areRelated(personOne.getBiologicalRelationship(EBiologicalRelationship.MOTHER).getTheOtherPerson(personOne),
							personTwo.getBiologicalRelationship(EBiologicalRelationship.MOTHER).getTheOtherPerson(personTwo),
							depthToCheck - 1)) {
					return true;
				}
			}
			if(personTwo.hasBiologicalRelationship(EBiologicalRelationship.FATHER) &&
					areRelated(personOne.getBiologicalRelationship(EBiologicalRelationship.MOTHER).getTheOtherPerson(personOne),
							personTwo.getBiologicalRelationship(EBiologicalRelationship.FATHER).getTheOtherPerson(personTwo),
							depthToCheck - 1)) {
				return true;
			}
		}
		
		if(personTwo.hasBiologicalRelationship(EBiologicalRelationship.FATHER)){
			if(personTwo.getBiologicalRelationship(EBiologicalRelationship.FATHER).getTheOtherPerson(personTwo) == personOne) {
				return true;
			}
		}
		
		if(personTwo.hasBiologicalRelationship(EBiologicalRelationship.MOTHER)){
			if(personTwo.getBiologicalRelationship(EBiologicalRelationship.MOTHER).getTheOtherPerson(personTwo) == personOne) {
				return true;
			}
		}
		
		return false;
	}
}
