package people;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 * The EConsumable types represent the product type of each industry.
 * 
 * @author Ian Stuart
 */
public enum EConsumable {

	RAW_MATERIAL(7, 0, null, true, "Raw Material"),
	NECESSITY(16, 5, EConsumable.RAW_MATERIAL, true, "Necessity"),
	DISCRETIONARY(16, 1, EConsumable.RAW_MATERIAL, false, "Discretionary");
	
	private final double quantityProducedAtOnce;
	private final int inputRequiredQuantity;
	private final EConsumable inputRequiredConsumable;
	private final boolean isEssential;
	private final String nameOfIndustry;
	
	private EConsumable(double quantityProducedAtOnce, 
							int inputRequiredQuantity,
							EConsumable inputRequiredConsumable,
							boolean isEssential,
							String nameOfIndustry) {
		this.quantityProducedAtOnce = quantityProducedAtOnce;
		this.inputRequiredQuantity = inputRequiredQuantity; 
		this.inputRequiredConsumable = inputRequiredConsumable;
		this.isEssential = isEssential;
		this.nameOfIndustry = nameOfIndustry;
	}

	public double getQuantityToProduceAtOnce() {return this.quantityProducedAtOnce;}
	public int getInputRequirementQuantity() { return this.inputRequiredQuantity; }
	public EConsumable getInputRequiredConsumable() { return this.inputRequiredConsumable;}
	public boolean isEssential() {return this.isEssential;}
	public String nameOfIndustry() {return this.nameOfIndustry;}
}
