package people;
import java.util.ArrayList;
import java.util.Map.Entry;
import helper.DeterminationHelper;
import helper.FinancialStatisticsHelper;
import people.Industry.IndustryRecord;
import utility.AnItemAndTheTimeSinceChangingIt;
import utility.Four;
import utility.IReturnsItem;
import utility.LazyUpdateUpToOncePerTick;
import utility.One;
import utility.Three;
import utility.Two;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 * Represents the government. This scenario is only intended to have a single Government agent.
 * 
 * @author Ian Stuart
 */
public class Government extends AbstractAgent{
	private static final double MAGIC_NUMBER = 0.9; /* Used when determining whether to increment of decrement tax rates */
	private int balance;
	private AnItemAndTheTimeSinceChangingIt<Double> incomeTaxRate = new AnItemAndTheTimeSinceChangingIt<Double>(DeterminationHelper.INITIAL_INCOME_TAX_RATE);
	private AnItemAndTheTimeSinceChangingIt<Double> profitTaxRate = new AnItemAndTheTimeSinceChangingIt<Double>(DeterminationHelper.INITIAL_PROFIT_TAX_RATE);
	private AnItemAndTheTimeSinceChangingIt<Double> salesTax = new AnItemAndTheTimeSinceChangingIt<Double>(DeterminationHelper.INITIAL_SALES_TAX_RATE);
	private AnItemAndTheTimeSinceChangingIt<Double> inheritanceTax = new AnItemAndTheTimeSinceChangingIt<Double>(DeterminationHelper.INITIAL_INHERITANCE_TAX);
	private AnItemAndTheTimeSinceChangingIt<Integer> parentToANewChildPaymentToFabricate = new AnItemAndTheTimeSinceChangingIt<Integer>(DeterminationHelper.INITIAL_PARENT_TO_A_NEW_CHILD_PAYMENT);
	private AnItemAndTheTimeSinceChangingIt<Integer> retirementAge = new AnItemAndTheTimeSinceChangingIt<Integer>(DeterminationHelper.INITIAL_RETIREMENT_AGE);
	private ArrayList<GovernmentRecord> records = new ArrayList<GovernmentRecord>();
	private final FinancialStatisticsHelper financialStatisticsHelper = new FinancialStatisticsHelper(8000);
	public int lastTotalSenarioMoney = 0;
	public static int moneyAdded = 0;
	public static int totalMoneyAddedAllTime = 0;
	
	private int revenueFromTaxingProfit = 0;
	private int revenueFromTaxingIncome = 0;
	private int revenueFromSalesTax = 0;
	private int revenueFromInheritanceTax = 0;
	private int numberOfPeoplePaidAssistance = 0;
	private int amountPaidToAssistPeople = 0;
	private int minimumWage = DeterminationHelper.MINIMUM_WAGE;
	private int corporateAssistance = 0;
	private Two<Industry, Integer> industryAskingForHelp = new Two<Industry, Integer>(null, 0);
	private LazyUpdateUpToOncePerTick<Boolean> isRawMaterialShortageEmergency = new LazyUpdateUpToOncePerTick<Boolean>(
																					new IReturnsItem<Boolean>() { 
																						public Boolean getItem() {
																							return isProductShortage(EConsumable.RAW_MATERIAL);
																				}});
	private LazyUpdateUpToOncePerTick<Boolean> isNecessityShortageEmergency = new LazyUpdateUpToOncePerTick<Boolean>(
																					new IReturnsItem<Boolean>() { 
																						public Boolean getItem() {
																							return isProductShortage(EConsumable.NECESSITY);
																				}});
	private LazyUpdateUpToOncePerTick<Three<Double, Double, Double>> averagesOf_Unemployment_PersonalBalance_PercentOfTargetBalance = 
																				new LazyUpdateUpToOncePerTick<Three<Double, Double, Double>>(
																					new IReturnsItem<Three<Double, Double, Double>>() { 
																						public Three<Double, Double, Double> getItem() {
																							return calculateAveragesOf_Unemployment_PersonalBalance_FinancialStanding();
																				}});
		
	/* At this point, in this scenario, the "name" value isn't consequential.
	 * The "money" value is the amount of funds that the government starts with.
	 */
	public Government(int money) {
		super();
		balance = money;
	}
	
	public ArrayList<GovernmentRecord> getGovernmentRecords(){ return this.records; }	
	public GovernmentRecord getLastPopulationRecord() {
		if(this.records.isEmpty()) {
			return null;
		}
		return this.records.get(this.records.size() -1 );
	}
	public int getRetirementAge() {return this.retirementAge.get();}
	public double getProfitTaxRate() {return this.profitTaxRate.get();}
	public int getRevenueFromTaxingProfit() {return this.revenueFromTaxingProfit;}
	public double getIncomeTaxRate() { return this.incomeTaxRate.get(); }
	public int getRevenueFromTaxingIncome() {return this.revenueFromTaxingIncome;}
	public double getSalesTaxRate() {return this.salesTax.get();}
	public int getRevenueFromSalesTax() {return this.revenueFromSalesTax;}
	public double getInheritanceTaxRate() {return this.inheritanceTax.get();}
	public int getRevenueFromInheritanceTax() {return this.revenueFromInheritanceTax;}
	public int getParentToANewChildPaymentToFabricate() { return this.parentToANewChildPaymentToFabricate.get();}	
	public int getAmountPaidToAssistPeople() {return this.amountPaidToAssistPeople;}
	public boolean isRawMaterialShortageEmergency() { return this.isRawMaterialShortageEmergency.get();}
	public boolean isNecessityShortageEmergency() { return this.isNecessityShortageEmergency.get();}
	public int getBalance() {return this.balance;}
	public void setBalance(int balance) {this.balance = balance;}
	public int getMinimumWage() {return this.minimumWage;}
	
	/*
	 * This method executes the main actions of the government.
	 */
	public void executiveFunction() {
		if(this.balance < 0) {
			throw new IllegalStateException("This implementation of a government does not allow for a negative balance.");
		}
		if(this.getGovernmentRecords().size() > DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE && this.records.size() > DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE) {
			this.adjustProfitTax();
			this.adjustIncomeTax();
			this.adjustSalesTax();
			this.adjustLengthOfWorkDay();
			this.adjustRetirmentAge();
			this.adjustParentToANewChildPayment();
		}

		int previousBalance = records.size() > 0 ? records.get(records.size() - 1).getMoney() : 0;
		int currentBalance = this.balance;
		int totalRevenue = revenueFromTaxingProfit + revenueFromTaxingIncome + revenueFromSalesTax + revenueFromInheritanceTax;
		int totalCosts = amountPaidToAssistPeople + corporateAssistance;
		if(previousBalance + totalRevenue - totalCosts != currentBalance && THE_WORLD.getInstance().getTime() != 0) {
			throw new IllegalStateException("GovernmentSelfAuditError");
		}
		recordPopulationData();
	}
	

	
	/* The income argument should be set to the total amount that is to be taxed. 
	 * This method will deduct the tax, changing this amount (as it is passed by reference),
	 * so that the person can add the remaining amount to their balance.
	 * For verification and convenience the amount of tax that was paid is returned. 
	 */
	public int acceptTaxOnIncomeAndReturnAmountOfTaxPaid(One<Integer> income) {
		int amount = (int) (income.get() * incomeTaxRate.get());
		income.set(income.get() - amount);
		if(amount > 0) {
			this.revenueFromTaxingIncome += amount;
			this.balance += amount;
		}else if (amount < 0){
			throw new IllegalStateException("Paying negative income tax");
		}
		return amount;
	}
	
	/* The income argument should be set to the total amount that is to be taxed. 
	 * This method will deduct the tax, changing this amount (as it is passed by reference),
	 * so that the industry can add the remaining amount to their balance.
	 * For verification and convenience the amount of tax that was paid is returned. 
	 */
	public int payTaxOnProfitAndReturnAmountOfTaxPaid(One<Integer> income) {
		int audit = income.get();
		int amount = (int) (income.get() * profitTaxRate.get());
		
		income.set(income.get() - amount);
		if(amount > 0) {
			this.revenueFromTaxingProfit += amount;
			this.balance += amount;
		}
		
		if(audit != income.get() + amount) {
			throw new IllegalStateException("Paying profit tax error");
		}
		return amount;
	}

	/*
	 * Returns the sales tax amount for an item of the "price" amount plus the "price" amount.
	 */
	public int getTotalPriceIncludingSalesTax(int price) {
		return price + getSalesTaxAmount(price);
	}
	
	/*
	 * Returns the sales tax amount in monetary units that would need to be paid for a sale at the "price" amount.
	 */
	public int getSalesTaxAmount(int price) {
		return (int)(price * this.salesTax.get());
	}
	
	/* 
	 * The "totalPrice" argument should be set to the total price plus the resulting salesTax. 
	 * This method will deduct the tax from the "totalPrice" variable, changing this amount (as it is passed by reference),
	 * so that the industry can add the remaining amount to their balance. 
	 * The "originalPrice" value is passed in to verify that the math on the other end is correct.
	 * For verification and convenience the quantity of tax that was paid is returned. 
	 */
	public int paySalesTaxAndReturnAmountOfTaxPaid(int originalPrice, One<Integer> totalPrice) {
		int salesTaxAmount = getSalesTaxAmount(originalPrice);
		if(salesTaxAmount + originalPrice != totalPrice.get()) {
			throw new IllegalArgumentException("There is missmatch between original price, sales tax, and total price!");
		}
		this.revenueFromSalesTax += getSalesTaxAmount(originalPrice);
		this.balance += getSalesTaxAmount(originalPrice);
		totalPrice.set(totalPrice.get() - getSalesTaxAmount(originalPrice));
		return salesTaxAmount;
	}
	
	/* The "amount" argument should be set to the total amount that is to be taxed. 
	 * This method will deduct the tax from the amount variable, changing this amount (as it is passed by reference),
	 * so that the remaining amount can be distributed.
	 * For verification and convenience the quantity of tax that was paid is returned. 
	 */
	public int payInheritanceTaxAndReturnAmountOfTaxPaid(One<Integer> amount) {
		//int originalAmount = amount.get();
		int amountOfTaxBeingCollected = (int)(amount.get() * this.inheritanceTax.get());
		this.revenueFromInheritanceTax += (int)(amount.get() * this.inheritanceTax.get());
		this.balance += (int)(amount.get() * this.inheritanceTax.get());
		amount.set(amount.get() - (int)(amount.get() * this.inheritanceTax.get()));
		return amountOfTaxBeingCollected;
	}
	
	/* The amount argument should be set to the total amount that is submitted for government revenue. 
	 * This method will zero the amount variable changing this variable as it is passed by reference.
	 * The entire amount passed in should be returned as the quantity of tax paid. 
	 */
	public int pay100PercentInheritanceTaxBecauseYouDidntHaveLivingKidsAndReturnAmountOfTaxPaid(One<Integer> amount) {
		int amountPaid = amount.get();
		this.revenueFromInheritanceTax += amount.get();
		this.balance += amount.get();
		amount.set(0);
		return amountPaid;
	}
	
	/*
	 * As long as the government has enough money it will call the acceptAssistance method of the person argument that was passed in,
	 * which will add the designated amount paid for social security to the balance of the person (and the method will return true for success).
	 * If the government did not have enough money this method will return false. 
	 */	
	public boolean doesPaySocialSecurity(Person person) {
		if(person == null) {
			throw new NullPointerException("null cannot be paid social security");
		}else if(!person.isAlive()){
			throw new IllegalArgumentException("A dead person cannot be paid social security");
		}
		boolean success = false;
		if(this.balance >= DeterminationHelper.AMOUNT_TO_PAY_FOR_SOCIAL_SECURITY) {
			this.balance -= DeterminationHelper.AMOUNT_TO_PAY_FOR_SOCIAL_SECURITY;
			this.amountPaidToAssistPeople += DeterminationHelper.AMOUNT_TO_PAY_FOR_SOCIAL_SECURITY;
			this.numberOfPeoplePaidAssistance++;
			One<Integer> pileOfMoney = new One<Integer>(DeterminationHelper.AMOUNT_TO_PAY_FOR_SOCIAL_SECURITY);
			person.acceptAssistance(pileOfMoney);
			success = true;
		}
		return success;
	}
	
	/*
	 * Simulations hopefully do not require this method, it is meant as a potential save to the undesirable case when
	 * an industry runs out of money and product.
	 * 
	 * Whether the government assists an industry is conditional. 
	 * The government will not help on the first call to this method.
	 * 
	 * If criteria is met the government will assist the industry with funds and return true. 
	 * If the government did not assist the industry this method returns false.
	 */
	public boolean doesHelpDyingIndustry(Industry industry) {
		boolean success = false;
		if(this.industryAskingForHelp.getFirst() == null) {
			this.industryAskingForHelp.setFirst(industry);
			this.industryAskingForHelp.setSecond(1);
		}else if(this.industryAskingForHelp.getFirst() == industry) {
			if(this.industryAskingForHelp.getSecond() >= 10 && this.getRoughFinancialStanding() > 1 && this.balance > DeterminationHelper.AMOUNT_TO_PAY_TO_ASSIST_AND_IMPOVERISHED_INDUSTRY * 2) {
				industry.setMoney(industry.getMoney() + DeterminationHelper.AMOUNT_TO_PAY_TO_ASSIST_AND_IMPOVERISHED_INDUSTRY);
				this.corporateAssistance += DeterminationHelper.AMOUNT_TO_PAY_TO_ASSIST_AND_IMPOVERISHED_INDUSTRY;
				this.balance -= DeterminationHelper.AMOUNT_TO_PAY_TO_ASSIST_AND_IMPOVERISHED_INDUSTRY;
				this.industryAskingForHelp.setSecond(0);
				this.industryAskingForHelp.setFirst(null);
				success = true;
				boolean savedOne = false;
				for(int i = industry.getJobs().size() - 1; i >= 0; i--) {
					if(!savedOne && industry.getJobs().get(i).getPerson() != null) {
						savedOne = true;
					}else{
						if(industry.getJobs().get(i).getPerson() != null) {
							industry.getJobs().get(i).getPerson().setJob(null);
							industry.getJobs().get(i).setPersonNull();
						}
						industry.getJobs().remove(i);
					}
				}
				if(!savedOne) {
					industry.doesCreateNewJob(industry.getProductType());
				}
			}else {
				this.industryAskingForHelp.setSecond(this.industryAskingForHelp.getSecond() + 1);
			}
		}
		return success;
	}
	
	/*
	 * As long as the government has enough money it will call the acceptAssistance method of the person argument that was passed in,
	 * which will add the amountNeeded to the balance of the person (and the method will return true for success).
	 * If the government did not have enough money this method will return false. 
	 */	
	public boolean doesAcceptAndProvideAssistanceUponRequest(Person person, int amountNeeded) {
		if(amountNeeded < 0) {
			throw new IllegalArgumentException("The amountNeeded value cannot be negative");
		}
		boolean success = false;
		if(this.balance > amountNeeded) {
			this.amountPaidToAssistPeople += amountNeeded;
			this.numberOfPeoplePaidAssistance++;
			this.balance -= amountNeeded;
			One<Integer> pileOfMoney = new One<Integer>(amountNeeded);
			person.acceptAssistance(pileOfMoney);
			success = true;
		}
		return success;
	}
	
	/*
	 * this is meant to be used by the person agents so the government (and final simulations results) can use this data. 
	 */
	public void submitPersonalBalanceForRecord(int balance) {
		financialStatisticsHelper.submitValue(balance);
	}
	
	/*
	 * This method returns true if the "econsumable" consumable is that of an essential industry (Raw Material or Necessity)
	 * and there is a product shortage of that type. 
	 */
	private boolean isProductShortage(EConsumable econsumable) {
		boolean isProductShortage = false;
		int sizeForSample = 4;
		if(econsumable.isEssential()) {
			ArrayList<IndustryRecord> industryRecords = THE_WORLD.getInstance().getIndustry(econsumable).getIndustryRecords();
			if(industryRecords.size() > sizeForSample) {
				for(int i = industryRecords.size() - (sizeForSample - 1); i < industryRecords.size(); i++) {
					if(industryRecords.get(i).getUnmetDemand() > 0 || 
							(industryRecords.get(i).getUnfilledPositions() > 0 && THE_WORLD.getInstance().getIndustry(econsumable).getPercentOfTargetInventory() < 0.45)) {
						isProductShortage = true;
					}
				}
			}
		}
		return isProductShortage;
	}
	
	/*
	 * This method records current government and population information.
	 */
	private void recordPopulationData() {

			int checkPeoplesMoney = 0;	
			int age = 0;
			int retirementAgePopulation = 0;
			int adultCount = 0;
			int under18Count = 0;
			int unemployedPeople = 0;
			for(Person person : THE_WORLD.getInstance().getPeople()) {
				age += person.getAge();
				if(person.isAdult()) {
					adultCount++;
					if(person.getAge() >= this.getRetirementAge()) {
						retirementAgePopulation++;
					}else if(person.getJob() == null) {
						unemployedPeople++;
					}
				}else {
					under18Count++;
				}
				checkPeoplesMoney += person.getMoney();
			}
			double unemploymentRatio = ((double)unemployedPeople)/((double)(adultCount - retirementAgePopulation));
			if(THE_WORLD.getInstance().getPeople().size() > 0) {
				age /= THE_WORLD.getInstance().getPeople().size();
			}
			
			Four<Integer, Integer, Integer, Integer> averageBalance_averageTopQuarterBalance_averageBottomQuarterBalance_totalPersonalMoney = 
																					financialStatisticsHelper.getAverage_AverageOfTopQuarter_AndAverageOfBottomQuarter_TotalPersonalMoney();
			int alreadyAddedPeoplesMoney = averageBalance_averageTopQuarterBalance_averageBottomQuarterBalance_totalPersonalMoney.getFourth();
			financialStatisticsHelper.zeroValues();
			
			records.add(new GovernmentRecord(this.balance, this.revenueFromTaxingProfit, this.revenueFromTaxingIncome, this.revenueFromSalesTax, this.revenueFromInheritanceTax, 
					this.numberOfPeoplePaidAssistance, this.amountPaidToAssistPeople, getRoughFinancialStanding(), Industry.getAdjustProductionRate(),
					this.getProfitTaxRate(), this.getIncomeTaxRate(), this.getSalesTaxRate(), this.getInheritanceTaxRate(), this.getParentToANewChildPaymentToFabricate(),
					this.getRetirementAge(), Industry.getLengthOfWorkDay(), THE_WORLD.getInstance().getPeople().size(), retirementAgePopulation, adultCount, under18Count, age, unemploymentRatio, 
					averageBalance_averageTopQuarterBalance_averageBottomQuarterBalance_totalPersonalMoney.getFirst(), averageBalance_averageTopQuarterBalance_averageBottomQuarterBalance_totalPersonalMoney.getSecond(), 
					averageBalance_averageTopQuarterBalance_averageBottomQuarterBalance_totalPersonalMoney.getThird(), averageBalance_averageTopQuarterBalance_averageBottomQuarterBalance_totalPersonalMoney.getFourth()));
			
			int forNewTotalSenarioMoney = averageBalance_averageTopQuarterBalance_averageBottomQuarterBalance_totalPersonalMoney.getFourth();
			forNewTotalSenarioMoney += this.getBalance();
			for(Entry<EConsumable, Industry> entry : THE_WORLD.getInstance().getIndustries().entrySet()){
				forNewTotalSenarioMoney += entry.getValue().getMoney();
			}
			
			if(checkPeoplesMoney != alreadyAddedPeoplesMoney) {
				throw new IllegalStateException("Descrepency with peoples total money!!");
			}

			if(forNewTotalSenarioMoney != this.lastTotalSenarioMoney + Government.moneyAdded && THE_WORLD.getInstance().getTime() != 0) {		
				throw new IllegalStateException("Error! Unauthorized Change In Total Senario Money");
				//this.lastTotalSenarioMoney = forNewTotalSenarioMoney;
			}else if(Government.moneyAdded > 0 || THE_WORLD.getInstance().getTime() == 0) {
				this.lastTotalSenarioMoney = forNewTotalSenarioMoney;
			}
			
			this.corporateAssistance = 0;
			this.revenueFromTaxingProfit = 0;
			this.revenueFromTaxingIncome = 0;
			this.revenueFromSalesTax = 0;
			this.revenueFromInheritanceTax = 0;
			this.numberOfPeoplePaidAssistance = 0;
			this.amountPaidToAssistPeople = 0;
			
			Person.totalFromAssistance = 0;
			Person.totalFromEmployerMinusIncomeTax = 0;
			Person.totalPaidInTaxes = 0;
			Person.totalPaidForNecessities = 0;
			Person.totalPaidForDiscretionaries = 0;
			Person.totalPaidForInheritanceTax = 0;
			Government.moneyAdded = 0;
	}
	
	/*
	 * The returns the governments 'rough financial standing' which will be a value between 0 and 5 (values above 5 were truncated). A 0 financial standing indicates lack of funds.
	 * A financial standing near 1 is considered ideal, which indicates that the government has enough funds for to pay for x days of its expenses.
	 * A financial standing that is too high can be troublesome as it can mean that the government is hoarding too much of the total scenarios money. 
	 */
	private double getRoughFinancialStanding() {
		double roughFinancialStanding = ((double)this.balance)/((double)(this.amountPaidToAssistPeople)*DeterminationHelper.MULTIPLIER_OF_EXPENSES_TO_DETERMINE_DIVISOR_OF_MONEY_FOR_DETERMINING_FINANCIAL_STANDING);
		if(roughFinancialStanding > DeterminationHelper.FINANCIAL_STANDING_CAP) {
			roughFinancialStanding = DeterminationHelper.FINANCIAL_STANDING_CAP;
		}
		return roughFinancialStanding;
	}
	
	/*
	 * This is conditional; if criteria is met the profit tax rate is adjusted.
	 */
	private void adjustProfitTax() {
		double percentOfTargetBalance = this.getAveragesOf_Unemployment_PersonalBalance_FinancialStanding().getThird();
		double rawMaterialIndustryFinancialStanding = THE_WORLD.getInstance().getIndustry(EConsumable.RAW_MATERIAL).getRoughFinancialStanding();
		double necessitiesIndustryFinancialStanding = THE_WORLD.getInstance().getIndustry(EConsumable.NECESSITY).getRoughFinancialStanding();
		if(percentOfTargetBalance < DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getFirst() && 
						this.profitTaxRate.getTimeSinceLastChange() >= DeterminationHelper.MINIMUM_TIME_BETWEEN_CHANGES_TO_PROFIT_TAX_RATE &&
						rawMaterialIndustryFinancialStanding >= DeterminationHelper.ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_THRESHOLD_TO_CHANGE_PROFIT_TAX_RATE && 
						necessitiesIndustryFinancialStanding >= DeterminationHelper.ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_THRESHOLD_TO_CHANGE_PROFIT_TAX_RATE &&
						this.getProfitTaxRate() < DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getSecond() - 
									DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getThird() * MAGIC_NUMBER) {
			this.profitTaxRate.set(this.profitTaxRate.get() + DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getThird());
		}else if(percentOfTargetBalance > DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getFirst() && 
						this.profitTaxRate.getTimeSinceLastChange() >= DeterminationHelper.MINIMUM_TIME_BETWEEN_CHANGES_TO_PROFIT_TAX_RATE &&
						(rawMaterialIndustryFinancialStanding < DeterminationHelper.ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_THRESHOLD_TO_CHANGE_PROFIT_TAX_RATE || 
									necessitiesIndustryFinancialStanding < DeterminationHelper.ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_THRESHOLD_TO_CHANGE_PROFIT_TAX_RATE) &&
						this.getProfitTaxRate() > DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getSecond() + 
									DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getThird() * MAGIC_NUMBER) {
			this.profitTaxRate.set(this.profitTaxRate.get() - DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getThird());
		}
	}
	
	/*
	 * This is conditional; if criteria is met the income tax rate is adjusted.
	 */
	private void adjustIncomeTax() {
		double percentOfTargetBalance = this.getAveragesOf_Unemployment_PersonalBalance_FinancialStanding().getThird();
		if(percentOfTargetBalance < DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getFirst() && 
						this.getIncomeTaxRate() < DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getSecond() - 
																DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getThird() * MAGIC_NUMBER && 
						this.incomeTaxRate.getTimeSinceLastChange() >= DeterminationHelper.MINIMUM_TIME_BETWEEN_CHANGES_TO_INCOME_TAX_RATE) {
			this.incomeTaxRate.set(this.incomeTaxRate.get() + DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getThird());
		}else if(percentOfTargetBalance > DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_INCOME_TAX.getFirst() && 
						this.getIncomeTaxRate() > DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_INCOME_TAX.getSecond() +
																DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getThird() * MAGIC_NUMBER && 
						this.incomeTaxRate.getTimeSinceLastChange() >= DeterminationHelper.MINIMUM_TIME_BETWEEN_CHANGES_TO_INCOME_TAX_RATE) {
			this.incomeTaxRate.set(this.incomeTaxRate.get() - DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getThird());
		}
	}
	
	/*
	 * This is conditional; if criteria is met the sales tax rate is adjusted.
	 */
	private void adjustSalesTax() {
		double percentOfTargetBalance = this.getAveragesOf_Unemployment_PersonalBalance_FinancialStanding().getThird();
		double rawMaterialIndustryFinancialStanding = THE_WORLD.getInstance().getIndustry(EConsumable.RAW_MATERIAL).getRoughFinancialStanding();
		double necessitiesIndustryFinancialStanding = THE_WORLD.getInstance().getIndustry(EConsumable.NECESSITY).getRoughFinancialStanding();
		if(percentOfTargetBalance < DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getFirst() && 
					this.getSalesTaxRate() < DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getSecond() - 
														DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getThird() * MAGIC_NUMBER  &&
					this.salesTax.getTimeSinceLastChange() >= DeterminationHelper.MINIMUM_TIME_BETWEEN_CHANGES_TO_SALES_TAX &&
								rawMaterialIndustryFinancialStanding >= DeterminationHelper.MINIMUM_ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_REQUIRED_TO_RAISE_SALES_TAX && 
								necessitiesIndustryFinancialStanding >= DeterminationHelper.MINIMUM_ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_REQUIRED_TO_RAISE_SALES_TAX) {
			this.salesTax.set(this.salesTax.get() + DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getThird());
		}else if(percentOfTargetBalance > DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getFirst() && 
					this.getSalesTaxRate() > DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getSecond() + 
														DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getThird() * MAGIC_NUMBER && 
					this.salesTax.getTimeSinceLastChange() >= DeterminationHelper.MINIMUM_TIME_BETWEEN_CHANGES_TO_SALES_TAX) {
			this.salesTax.set(this.salesTax.get() - DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getThird());
		}
	}
	
	/*
	 * This is conditional; if criteria is met the relative length of work day is adjusted.
	 * The length of work day starts at 1d and can only be lowered. 
	 */
	private void adjustLengthOfWorkDay() {
		double averageUnemployment = this.getAveragesOf_Unemployment_PersonalBalance_FinancialStanding().getFirst();
		double unemployment = this.getLastPopulationRecord().getUnemploymentRatio();
		if(this.getGovernmentRecords().size() > 1) {
			unemployment = (unemployment + this.getGovernmentRecords().get(this.getGovernmentRecords().size() - 2).getUnemploymentRatio())/2;
		}
		Three<Double, Integer, Double>[] unemploymentThreshold_TimeBetweenChanges_IncrementBy_list = DeterminationHelper.WORKDAY_LENGTH__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS;
		for(int i = 0; i < unemploymentThreshold_TimeBetweenChanges_IncrementBy_list.length; i++) {
			if(unemployment > unemploymentThreshold_TimeBetweenChanges_IncrementBy_list[i].getFirst() &&
						Industry.getTimeSinceLastChangeInLengthOfWorkDay() >= unemploymentThreshold_TimeBetweenChanges_IncrementBy_list[i].getSecond() &&
						Industry.getLengthOfWorkDay() >= DeterminationHelper.MINIMUM_LENGTH_OF_WORKDAY + unemploymentThreshold_TimeBetweenChanges_IncrementBy_list[i].getThird()) {
				Industry.setLengthOfWorkDay(Industry.getLengthOfWorkDay() - unemploymentThreshold_TimeBetweenChanges_IncrementBy_list[i].getThird());
				break;
			}else {
				unemployment = averageUnemployment;
			}
		}
	}
	
	/*
	 * This is conditional; if criteria is met the retirement age is adjusted.
	 * The retirement age can only be lowered. 
	 */
	private void adjustRetirmentAge() {
		double averageUnemployment = this.getAveragesOf_Unemployment_PersonalBalance_FinancialStanding().getFirst();
		Three<Double, Integer, Integer>[] unemploymentThreshold_TimeBetweenChanges_IncrementBy_list = DeterminationHelper.RETIREMENT_AGE__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS;
		for(int i = 0; i < unemploymentThreshold_TimeBetweenChanges_IncrementBy_list.length; i++) {
			if(averageUnemployment > unemploymentThreshold_TimeBetweenChanges_IncrementBy_list[i].getFirst() &&
					this.retirementAge.getTimeSinceLastChange() >= unemploymentThreshold_TimeBetweenChanges_IncrementBy_list[i].getSecond() &&
					this.retirementAge.get() >= DeterminationHelper.MINIMUM_RETIREMENT_AGE + unemploymentThreshold_TimeBetweenChanges_IncrementBy_list[i].getThird()) {
				this.retirementAge.set(this.retirementAge.get() - unemploymentThreshold_TimeBetweenChanges_IncrementBy_list[i].getThird());
				break;
			}
		}
	}
	
	/*
	 * This is conditional; if criteria is met the "parent to a new child payment" amount is adjusted.
	 */
	private void adjustParentToANewChildPayment() {
		double averagePersonalBalance = this.getAveragesOf_Unemployment_PersonalBalance_FinancialStanding().getSecond();
		if(averagePersonalBalance > DeterminationHelper.PARENT_TO_A_NEW_CHILD_PAYMENT_AVERAGE_PERSONAL_BALANCE_THRESHOLDS_TO__LOWER_AND_RAISE__.getFirst() && 
						this.parentToANewChildPaymentToFabricate.get() > 0 && this.parentToANewChildPaymentToFabricate.getTimeSinceLastChange() >= DeterminationHelper.MINIMUM_TIME_BETWEEN_PARENT_TO_A_NEW_CHILD_PAYMENT_ADJUSTMENTS) {
			if(this.parentToANewChildPaymentToFabricate.get() >= DeterminationHelper.PARENT_TO_A_NEW_CHILD_PAYMENT_ADJUSTMENT_INCREMENTS) {
				this.parentToANewChildPaymentToFabricate.set(this.parentToANewChildPaymentToFabricate.get() - DeterminationHelper.PARENT_TO_A_NEW_CHILD_PAYMENT_ADJUSTMENT_INCREMENTS);
			}
		}else if( (averagePersonalBalance <= DeterminationHelper.PARENT_TO_A_NEW_CHILD_PAYMENT_AVERAGE_PERSONAL_BALANCE_THRESHOLDS_TO__LOWER_AND_RAISE__.getSecond()) && 
						this.parentToANewChildPaymentToFabricate.get() < DeterminationHelper.MAXIMUM_PARENT_TO_A_NEW_CHILD_PAYMENT && 
						this.parentToANewChildPaymentToFabricate.getTimeSinceLastChange() >= DeterminationHelper.MINIMUM_TIME_BETWEEN_PARENT_TO_A_NEW_CHILD_PAYMENT_ADJUSTMENTS){
			this.parentToANewChildPaymentToFabricate.set(this.parentToANewChildPaymentToFabricate.get() + DeterminationHelper.PARENT_TO_A_NEW_CHILD_PAYMENT_ADJUSTMENT_INCREMENTS);
		}
	}
	
	/*
	 * returns a generic Three object with the average unemployment ratio (out of 1 for 100%), the average personal balance of people, and the governments average financial standing 
	 * in that order (first, second, third - see the "Three" class). 
	 * 
	 * averages use data from the previous (includes current time step) x time steps.
	 */
	private Three<Double, Double, Double> calculateAveragesOf_Unemployment_PersonalBalance_FinancialStanding(){
		double averageUnemployment = 0;
		double averagePersonalBalance = 0;
		int averageExpenses = this.getAmountPaidToAssistPeople();
		for(int i = this.getGovernmentRecords().size() - (DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE - 1); i < this.getGovernmentRecords().size(); i++) {
			averageUnemployment += this.getGovernmentRecords().get(i).getUnemploymentRatio();
			averagePersonalBalance += this.getGovernmentRecords().get(i).getAverageBalance();
			averageExpenses += this.getGovernmentRecords().get(i).getAmountPaidToAssistPeople();
		}		
		averageUnemployment /= DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE;
		averagePersonalBalance /= DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE;
		averageExpenses /= DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE;
		return new Three<Double, Double, Double>(averageUnemployment, averagePersonalBalance, 
						((double)this.balance)/((double)(averageExpenses * DeterminationHelper.MULTIPLIER_OF_EXPENSES_TO_DETERMINE_DIVISOR_OF_MONEY_FOR_DETERMINING_FINANCIAL_STANDING)));
	}
	
	/*
	 * returns a generic Three object with the average unemployment ratio (out of 1 for 100%), the average personal balance of people, and the governments average financial standing 
	 * in that order (first, second, third - see the "Three" class). 
	 * 
	 * averages use data from the previous (includes current time step) x time steps.
	 */
	private Three<Double, Double, Double> getAveragesOf_Unemployment_PersonalBalance_FinancialStanding(){
		return this.averagesOf_Unemployment_PersonalBalance_PercentOfTargetBalance.get();
	}
	
	public class GovernmentRecord {
		private final int money;
		private final int revenueFromTaxingProfit;
		private final int revenueFromTaxingIncome;
		private final int revenueFromSalesTax;
		private final int revenueFromInheritanceTax;
		private final int numberOfPeoplePaidAssistance;
		private final int amountPaidToAssistPeople;
		private final double roughFinancialStanding;
		private final double theAdjustedProductionRateOfIndustries;
		private final double profitTaxRate;
		private final double incomeTaxRate;
		private final double salesTaxRate;
		private final double inheritanceTaxRate;
		private final int parentToANewChildPayment;
		private final int retirementAge;
		private final double lengthOfWorkday;
		private final int population;
		private final int retirementAgePopulation;
		private final int adultPopulation;
		private final int under18Population;
		private final int averageAge;
		private final double unemploymentRatio;
		private final int averageBalance;
		private final int averageBalanceOfTopQuarter;
		private final int averageBalanceOfBottomQuarter;
		private final int totalMoneyThatAllPeopleHave;
		
		public GovernmentRecord(int money, int revenueFromTaxingProfit, int revenueFromTaxingIncome, int revenueFromSalesTax, int revenueFromInheritanceTax, 
										int numberOfPeoplePaidAssistance, int amountPaidToAssistPeople, double roughFinancialStanding, double theAdjustedProductionRateOfIndustries,
										double profitTaxRate, double incomeTaxRate, double salesTaxRate, double inheritanceTaxRate, int parentToANewChildPayment, int retirementAge, double lengthOfWorkday,
										int population, int retirementAgePopulation, int adultPopulation, int under18Population, int averageAge, double unemploymentRatio, 
										int averageBalance, int averageBalanceOfTopQuarter, int averageBalanceOfBottomQuarter, int totalMoneyThatAllPeopleHave) {
			this.money = money;
			this.revenueFromTaxingProfit = revenueFromTaxingProfit;
			this.revenueFromTaxingIncome = revenueFromTaxingIncome;
			this.revenueFromSalesTax = revenueFromSalesTax;
			this.revenueFromInheritanceTax = revenueFromInheritanceTax;
			this.numberOfPeoplePaidAssistance = numberOfPeoplePaidAssistance;
			this.amountPaidToAssistPeople = amountPaidToAssistPeople;
			this.roughFinancialStanding = roughFinancialStanding;
			this.theAdjustedProductionRateOfIndustries = theAdjustedProductionRateOfIndustries;
			this.profitTaxRate = profitTaxRate;
			this.incomeTaxRate = incomeTaxRate;
			this.salesTaxRate = salesTaxRate;
			this.inheritanceTaxRate = inheritanceTaxRate;
			this.parentToANewChildPayment = parentToANewChildPayment;
			this.retirementAge = retirementAge;
			this.lengthOfWorkday = lengthOfWorkday;
			this.population = population;
			this.retirementAgePopulation = retirementAgePopulation;
			this.adultPopulation = adultPopulation;
			this.under18Population = under18Population;
			this.averageAge = averageAge;
			this.unemploymentRatio = unemploymentRatio;
			this.averageBalance = averageBalance;
			this.averageBalanceOfTopQuarter = averageBalanceOfTopQuarter;
			this.averageBalanceOfBottomQuarter = averageBalanceOfBottomQuarter;
			this.totalMoneyThatAllPeopleHave = totalMoneyThatAllPeopleHave;
			
			
		}
		
		public int getMoney() {return this.money;}
		public int getRevenueFromTaxingProfit() {return this.revenueFromTaxingProfit;}
		public int getRevenueFromTaxingIncome() {return this.revenueFromTaxingIncome;}
		public int getRevenueFromSalesTax() {return this.revenueFromSalesTax;}
		public int getRevenueFromInheritanceTax() {return this.revenueFromInheritanceTax;}
		public int getNumberOfPeoplePaidAssistance() {return this.numberOfPeoplePaidAssistance;}
		public int getAmountPaidToAssistPeople() {return this.amountPaidToAssistPeople;}
		public double getRoughFinancialStanding() {return this.roughFinancialStanding;}
		public double getTheAdjustedProductionRateOfIndustries() {return this.theAdjustedProductionRateOfIndustries;}
		public double getProfitTaxRate() {return this.profitTaxRate;}
		public double getIncomeTaxRate() {return this.incomeTaxRate;}
		public double getSaleTaxRate() {return this.salesTaxRate;}
		public double getInheritanceTaxRate() {return this.inheritanceTaxRate;}
		public int getParentToANewChildPayment() {return this.parentToANewChildPayment;}
		public int getRetirementAge() {return this.retirementAge;}
		public double getLengthOfWorkday() {return this.lengthOfWorkday;}
		public int getPopulation() { return this.population;}
		public int getRetirementAgePopulation() {return this.retirementAgePopulation;}
		public int getAdultPopulation() {return this.adultPopulation;}
		public int getUnder18Population() {return this.under18Population;}
		public int getAverageAge() {return this.averageAge;}
		public double getUnemploymentRatio() {return this.unemploymentRatio;}
		public int getAverageBalance() {return this.averageBalance;}
		public int getAverageBalanceOfTopQuarter() {return this.averageBalanceOfTopQuarter;}
		public int getAverageBalanceOfBottomQuarter() {return this.averageBalanceOfBottomQuarter;}
		public int getTotalMoneyThatAllPeopleHave() {return this.totalMoneyThatAllPeopleHave;}
	}
}





