package people;
import java.util.ArrayList;
import helper.DeterminationHelper;
import utility.AnItemAndTheTimeSinceChangingIt;
import utility.IReturnsItem;
import utility.LazyUpdateUpToOncePerTick;
import utility.One;
import utility.Three;
import utility.Two;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

	Three Industry ABCE Model is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	Three Industry ABCE Model is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 *  There are meant to be three Industry agents in a simulation, one for each consumable type (Raw Material, Necessity, Discretionary). 
 * 
 * @author Ian Stuart
 */
public class Industry extends AbstractAgent{
	private ArrayList<Job> jobs = new ArrayList<Job>();
	private ArrayList<IndustryRecord> industryRecords= new ArrayList<IndustryRecord>();
	private static double adjustProductionRate = 1d;
	private int money;
	private double productQuantity;
	private EConsumable productType;
	private AnItemAndTheTimeSinceChangingIt<Integer> productSalePrice;
	private  Double inputProductQuantity;
	private EConsumable inputProductType;
	private int unmetDemand = 0;
	private boolean wasProductionBelowExpectation = false;
	private int timeSinceLastChangeInTheNumberOfJobs = 0;
	private double dailyProduction = 0;
	private int dailyDemand = 0;
	private int dailyCosts = 0;
	private int wagesPaid = 0;
	private int taxPaid = 0;
	private int revenue = 0;
	private int quantitySold = 0;
	
	private static AnItemAndTheTimeSinceChangingIt<Double> lengthOfWorkDay = new AnItemAndTheTimeSinceChangingIt<Double>(1d);
			
	private LazyUpdateUpToOncePerTick<Double> adjustedEmployeeProduction = new LazyUpdateUpToOncePerTick<Double>(
																				new IReturnsItem<Double>() {
																					public Double getItem() {
																						return productType.getQuantityToProduceAtOnce() * 
																								adjustProductionRate * lengthOfWorkDay.get(); }});

	private LazyUpdateUpToOncePerTick<Double> adjustedEmployeeRequiredInput = new LazyUpdateUpToOncePerTick<Double>(
																					new IReturnsItem<Double>() {
																						public Double getItem() {
																							return productType.getInputRequirementQuantity() * 
																									adjustProductionRate * lengthOfWorkDay.get(); }});

	private LazyUpdateUpToOncePerTick<Integer> calculatedDailyExpenses = new LazyUpdateUpToOncePerTick<Integer>(
																			new IReturnsItem<Integer>() {
																				public Integer getItem() {
																					return calculateDailyExpenses(); }});

	private LazyUpdateUpToOncePerTick<Job> jobOpening = new LazyUpdateUpToOncePerTick<Job>(
																new IReturnsItem<Job>() {
																	public Job getItem() {
																		return findJobOpening(); }});

	private LazyUpdateUpToOncePerTick<Three<Integer, Double, Double>> averagesOf_Demand_FinancialStanding_PercentOfTargetInventory = 
																					new LazyUpdateUpToOncePerTick<Three<Integer, Double, Double>>(
																						new IReturnsItem<Three<Integer, Double, Double>>() {
																							public Three<Integer, Double, Double> getItem() {
																								return calculateAveragesOf_Demand_FinancialStanding_PercentOfTargetInventory();
																					}});
	
	/* The simulation is sensitive to the initialization values of the scenario and will fail if, in relation to other scenario initialization attributes, appropriate and workable values are not found. 
	 * 
	 * "startingMoney" is the amount of funds the industry would start with.
	 * "product" is a Two object including the consumable product that the industry will produce and the quantity of that product that the industry will start with - in that order (first and second; see the "Two" class). 
	 * "productSalePrice" is the initial price that the industry will sell its product for.
	 * "inputProductQuantity" is the quantity of input consumable that the industry will start with (the input consumable is Raw Material for the other two industries and isn't applicable to the Raw Material industry itself (so the quantity in that case can be 0)). 
	 * 
	 * if the starting consumable quantities are too high the industry objects will fail to see the need to create more jobs. 
	 */
	public Industry(int startingMoney, Two<EConsumable, Double> product, int productSalePrice, Double inputProductQuantity) {
		super();
		this.money = startingMoney;
		this.productType = product.getFirst();
		this.productQuantity = product.getSecond();
		this.productSalePrice = new AnItemAndTheTimeSinceChangingIt<Integer>(productSalePrice);
		this.inputProductType = product.getFirst().getInputRequiredConsumable();
		this.inputProductQuantity = inputProductQuantity;
	}
	
	/* The simulation is sensitive to the initialization values of the scenario and will fail if, in relation to other scenario initialization attributes, appropriate and workable values are not found. 
	 * 
	 * "startingMoney" is the amount of funds the industry would start with.
	 * "product" is a Two object including the consumable product that the industry will produce and the quantity of that product that the industry will start with - in that order (first and second; see the "Two" class). 
	 * "productSalePrice" is the initial price that the industry will sell its product for.
	 * 
	 * if the starting consumable quantities are too high the industry objects will fail to see the need to create more jobs. 
	 */
	public Industry(int startingMoney, Two<EConsumable, Double> product, int productSalePrice) {
		this(startingMoney, product, productSalePrice, 0d);
	}
	
	public EConsumable getProductType(){ return this.productType;}
	public Double getProductQuantity(){ return this.productQuantity;}
	public int getProductSalePrice() {return this.productSalePrice.get();}
	public void setProductSalePrice(int productSalePrice) {this.productSalePrice.set(productSalePrice);}	
	public int getUnmetDemand() {return this.unmetDemand;}
	public static double getAdjustProductionRate() {return adjustProductionRate;}
	public static void setAdjustProductionRate(double newAdjustProductionRate) {adjustProductionRate = newAdjustProductionRate;}
	public static double getLengthOfWorkDay() {return lengthOfWorkDay.get();}
	public static void setLengthOfWorkDay(double newLengthOfWorkDay) {lengthOfWorkDay.set(newLengthOfWorkDay);}
	public static int getTimeSinceLastChangeInLengthOfWorkDay() {return lengthOfWorkDay.getTimeSinceLastChange();}
	public boolean isEssentialIndustry() {return this.productType.isEssential();}
	public double getPercentOfTargetInventory() {return averagesOf_Demand_FinancialStanding_PercentOfTargetInventory.get().getThird();}
	public int getMoney() {return this.money;}
	public void setMoney(int money) {this.money = money;}
	
	/*
	 * executes the main actions of the industry
	 */
	public void executiveFunction() {
		this.IfADyingIndustryAskingForHelp();
		this.adjustNumberOfJobs();
		checkForNeededMaterials();
		this.adjustProductSalePrice();
	}
	
	/*
	 * This method will pay the industry's profit tax
	 */
	public void payProfitTaxAndUpdateRecords() {
		if(industryRecords.size() > 0) {
			int profit = money - industryRecords.get(industryRecords.size() - 1).getMoney();
			int amountPaid = 0;
			if(profit > 0) {
				this.money -= profit;
				One<Integer> pileOfMoney = new One<Integer>(profit);
				amountPaid = THE_WORLD.getInstance().getGovernment().payTaxOnProfitAndReturnAmountOfTaxPaid(pileOfMoney);
				int checkTheOtherWay = profit - pileOfMoney.get();
				if(amountPaid != checkTheOtherWay || pileOfMoney.get() < 0) {
					throw new IllegalStateException("Paying profit tax discrepency!.");
				}
				this.taxPaid += profit - pileOfMoney.get();
				this.money += pileOfMoney.get();
			}
		}
		
		int numberOfEmployees = 0;
		int unfilledPositions = 0;
		for(Job job : jobs) {
			if(job.getPerson() != null) {
				numberOfEmployees++;
			}else {
				unfilledPositions++;
			}
		}

		industryRecords.add(new IndustryRecord(money, (int)this.productQuantity, this.inputProductQuantity.intValue(), this.dailyProduction, this.dailyDemand, this.dailyCosts, numberOfEmployees, unfilledPositions,
				this.unmetDemand, this.wasProductionBelowExpectation, getRoughFinancialStanding(), this.wagesPaid, this.taxPaid, this.revenue, this.productSalePrice.get(), this.quantitySold));

		this.dailyProduction = 0;
		this.dailyDemand = 0;
		this.dailyCosts = 0;
		this.unmetDemand = 0;
		this.wagesPaid = 0;
		this.taxPaid = 0;
		this.revenue = 0;
		this.quantitySold = 0;	
		this.wasProductionBelowExpectation = false;
		this.timeSinceLastChangeInTheNumberOfJobs++;

		if(this.money < 0) {
			throw new IllegalStateException("An industry cannot have a negative balance.");
		}
	}
	
	public ArrayList<Job> getJobs() { return jobs; }

	public boolean doesCreateNewJob( EConsumable typeOfProductToProduce) {
		boolean success = false;
		Job job = null;
		if(this.money >= THE_WORLD.getInstance().getGovernment().getMinimumWage()) { 
			job = new Job(this, typeOfProductToProduce);
			this.jobs.add(job);
			success = true;
		}
		return success;
	}
	
	public Job getJobOpening() {
		Job job = jobOpening.get();
		if(job != null && job.getPerson() != null)  {
			job = jobOpening.forceUpdate();
		}
		return job;
	}
	
	private Job findJobOpening() {
		Job gig = null;
		for(int i = 0; i < jobs.size(); i++) {
			if(jobs.get(i).getPerson() == null) {
				gig = jobs.get(i);
				break;
			}
		}
		return gig;
	}
	
	/*
	 * This method should be called by a person wanting a job; the person will need to pass themselves in as and argument. Returns true if the person is hired. Returns false if the person is not hired. 
	 */
	public boolean doesLikeAndAcceptApplication(Person person) {
		boolean success = false;
		if(person.isAdult() && person.getAge() < THE_WORLD.getInstance().getGovernment().getRetirementAge() && getJobOpening() != null) {
			success = getJobOpening().doesSetPerson(person);
		}
		return success;
	}
	
	/* This method should be called by people to perform their job and create product. If the industry has enough money and any required Raw Material (if needed),
	 * the doesAcceptPayment of the person is called so they can be paid and this method will return true. If the industry didn't have enough money or there wasn't the required input material 
	 *  (only the Necessity and Discretionary Industries require input consumable for production) then the method will return false.
	 * 
	 */
	public boolean doesAcceptExpectedWork(Person person) {
		boolean success = false;
		Job job;
		if(person.isAlive() && person.getJob() instanceof Job) {
			job = (Job) person.getJob();
			if(/*this.jobs.contains(job = (Job) person.getJob()) &&*/ this.money >= job.getDailyPay()) {	
				if(job.getTypeOfProductToProduce().getInputRequiredConsumable() == this.inputProductType &&
						this.inputProductQuantity >= adjustedEmployeeRequiredInput.get()) {
					this.inputProductQuantity -= adjustedEmployeeRequiredInput.get();		
					this.dailyProduction += adjustedEmployeeProduction.get();
					productQuantity += adjustedEmployeeProduction.get();
					success= true;
				}else if(this.inputProductType == null) {
					this.dailyProduction += adjustedEmployeeProduction.get();
					productQuantity += adjustedEmployeeProduction.get();
					success= true;
				}
				if(success) {
					this.wagesPaid += job.getDailyPay();
					this.money = this.money - job.getDailyPay();
					addToDailyCosts(job.getDailyPay());
					person.doesAcceptPayment( new One<Integer>(job.getDailyPay() )); 
				}
			}
		}
		if(!success && person.getJob() != null) {
			this.wasProductionBelowExpectation = true;
		}
		return success;
	}
	
	public ArrayList<IndustryRecord> getIndustryRecords(){return this.industryRecords;}

	/* The "consumable" argument should be the consumable type being purchased. This type is passed for verification and must match the product/consumable that the industry produces and sells.
	 * This method returns the total money that would be required to purchase the "quantity" amount of the "consumable" consumable type including sales tax.
	 */
	public int getTotalPriceForConsumable(EConsumable consumable, int quantity) {
		if(quantity < 0) {
			throw new IllegalArgumentException("The quantity argument cannot be negative");
		}
		if(consumable != this.productType) {
			throw new IllegalArgumentException("This industry can only sell the one product/consumable that it produces. The consumable argument value did not match.");
		}
		return THE_WORLD.getInstance().getGovernment().getTotalPriceIncludingSalesTax((int)(this.productSalePrice.get() * quantity));
	}
	
	/* The "consumable" argument should be the consumable type being purchased. This type is passed for verification and must match the product/consumable that the industry produces and sells.
	 * This method returns the total money that would be required to purchase the "quantity" amount of the "consumable" consumable type excluding sales tax.
	 */
	public int getPriceForConsumableExcludingSalesTax(EConsumable consumable, int quantity) {
		if(consumable != this.productType) {
			throw new IllegalArgumentException("This industry can only sell the one product/consumable that it produces. The consumable argument value did not match.");
		}
		return (this.productSalePrice.get() * quantity);
	}
	
	/*
	 * This method will return true if the industry has the "quantity" amount of its product/consumable, This method will return false otherwise. 
	 */
	public boolean hasQuantity(int quantity) {
		if(quantity < 0) {
			throw new IllegalArgumentException("The quantity argument cannot be negative");
		}
		boolean hasQuantity = productQuantity >= quantity;
		if(!hasQuantity) {
			this.unmetDemand += quantity;
		}
		return hasQuantity;
	}
	
	/* Agents using this method will have to have checked the price before calling this method for the arguments and math to be correct.
	 * 
	 * The "consumable" argument should be the consumable type being purchased. This type is passed for verification and must match the product/consumable that the industry produces and sells.
	 * The "amount" argument should be the quantity of the consumable type being purchased.
	 * The "pileOfMoney" argument should be set to the total cost of the purchase including sales tax.
	 * 
	 * If the sale is successful the "pileOfMoney" object will be set to 0 (it is passed by reference) and this method will return true.
	 * If the "pileOfMoney" amount does not match the total cost or the industry does not have the desired quantity this method will return false.
	 */
	public boolean doesAcceptPurchaseOfConsumable(EConsumable consumable, int amount, One<Integer> pileOfMoney) {
		if(consumable != this.productType) {
			throw new IllegalArgumentException("This industry can only sell the one product/consumable that it produces. The consumable argument value did not match.");
		}
		boolean success = false;
		int originalAmount = pileOfMoney.get();
		int salesTaxAmount = 0;
		if(consumable == productType) { 
			this.dailyDemand += amount;//wasUnmetDemand
			if(productQuantity >= amount && pileOfMoney.get() == THE_WORLD.getInstance().getGovernment().getTotalPriceIncludingSalesTax((int)(this.productSalePrice.get() * amount) ) ){			
				salesTaxAmount = THE_WORLD.getInstance().getGovernment().paySalesTaxAndReturnAmountOfTaxPaid((int)(this.productSalePrice.get() * amount), pileOfMoney);
				productQuantity -= amount;

				if(pileOfMoney.get() + salesTaxAmount != originalAmount ) {
					throw new IllegalStateException("Submitting sales tax discrepency!");
				}
				this.revenue += pileOfMoney.get();
				this.money = this.money + pileOfMoney.get();
				pileOfMoney.set(0);
				success = true;
				this.quantitySold += amount;
			}else {
				//something didn't work, put the dailyDemand back to what it was
				this.dailyDemand -= amount;
			}
		}
		return success;
	}

	/*
	 * The returns the industry's 'rough financial standing' which will be a value between 0 and 5 (values above 5 were truncated). A 0 financial standing indicates lack of funds.
	 * A financial standing near 1 is considered ideal, which indicates that the industry has enough funds for to pay for x days of its expenses.
	 * A financial standing that is too high can be troublesome as it can mean that the industry is hoarding too much of the total scenarios money. 
	 * 
	 * Conceptually industries in the model meant to be representative of multiple companies such that they would be internally competitive and thus the behavior of a whole industry does not appear to seek maximum profit.
	 */
	public double getRoughFinancialStanding() {
		double roughFinancialStanding = ((double)this.money)/
											((double)this.calculatedDailyExpenses.get() * 
													DeterminationHelper.MULTIPLIER_OF_EXPENSES_TO_DETERMINE_DIVISOR_OF_MONEY_FOR_DETERMINING_FINANCIAL_STANDING);
		if(roughFinancialStanding > DeterminationHelper.FINANCIAL_STANDING_CAP) {
			roughFinancialStanding = DeterminationHelper.FINANCIAL_STANDING_CAP;
		}
		return roughFinancialStanding;
	}
	
	/*
	 * This method checks to see if the industry needs more input material/consumable (which can only be Raw Material in this model) and
	 * will attempt to purchase the needed input material/consumable if the industry has enough money (this attempt would fail if the supplying industry didn't have enough product/consumable).
	 */
	private void checkForNeededMaterials() {
		if(this.productType.getInputRequiredConsumable() == EConsumable.RAW_MATERIAL) {
			double higherAmount = dailyProduction;
			if(dailyDemand > higherAmount) {
				higherAmount = dailyDemand;
			}
			int targetInputMaterialInventory =  (int)((higherAmount/(adjustedEmployeeProduction.get()) + 1) * 
																			(adjustedEmployeeRequiredInput.get()) * DeterminationHelper.MULTIPLIER_TO_DETERMINE_TARGET_INPUT_MATERIAL_INVENTORY);
			if(inputProductQuantity < targetInputMaterialInventory) {
				int amount = targetInputMaterialInventory - inputProductQuantity.intValue();
				this.getRawMaterials(amount);
			}
		}
	}
	
	private void addToDailyCosts(int amount) {this.dailyCosts += amount;}	
	
	/*
	 * Calculates expenses based on filled jobs. This is only used for determining the rough financial standing
	 */
	private int calculateDailyExpenses() {
		int payRollExpenses = 0;
		int materialCosts = 0;
		if(this.productType.getInputRequiredConsumable() == EConsumable.RAW_MATERIAL) {
			int materialNeeded = 0;
			for(Job job : this.jobs) {
				payRollExpenses += job.getDailyPay();
				materialNeeded += adjustedEmployeeRequiredInput.get();
			}
			materialCosts = this.getRawMaterialCost(materialNeeded);
		}else {
			for(Job job : this.jobs) {
				payRollExpenses += job.getDailyPay();
			}
		}
		return payRollExpenses + materialCosts;
	}
	/*
	 * returns a generic Three object with the average industry demand, the industry's average financial standing, and the industry's ratio of its 'target inventory' (1 would be 100%)
	 * in that order (first, second, third - see the "Three" class). 
	 * 
	 * Averages use data from the previous (includes current time step) x time steps.
	 * 
	 * This method should be used to store the result in an LazyUpdateUpToOncePerTick object so it isn't called multiple times in a time step. 
	 */
	private Three<Integer, Double, Double> calculateAveragesOf_Demand_FinancialStanding_PercentOfTargetInventory(){
		int averageMetDemand = this.dailyDemand + getUnmetDemand();
		int averageQuantity = (int)this.productQuantity;
		double averageFinancialStanding = this.getRoughFinancialStanding();
		for(int i = this.industryRecords.size() - (DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE - 1); i < this.industryRecords.size() && i >= 0; i++) {
			averageMetDemand += this.industryRecords.get(i).getProductDemand();
			averageQuantity += this.industryRecords.get(i).getProductQuantity();
			averageFinancialStanding += this.industryRecords.get(i).getRoughFinancialStanding();
		}
		averageMetDemand /= DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE;
		if(averageMetDemand < 1) {
			averageMetDemand = 1;
		}
		averageQuantity /= DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE;
		averageFinancialStanding /= DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE;
		double percentOfTargetInventory = ((double)averageQuantity)/(double)(averageMetDemand * DeterminationHelper.MULTIPLIER_OF_DEMAND_TO_DETERMINE_TARGET_PRODUCT_INVENTORY);
		return new Three<Integer, Double, Double>(averageMetDemand, averageFinancialStanding, percentOfTargetInventory);
	}
	
	/*
	 * returns a generic Three object with the average industry demand, the industry's average financial standing, and the industry's ratio of its 'target inventory' (1 would be 100%)
	 * in that order (first, second, third - see the "Three" class). 
	 * 
	 * Averages use data from the previous (includes current time step) x time steps
	 */
	private Three<Integer, Double, Double> getAveragesOf_Demand_FinancialStanding_PercentOfTargetInventory(){
		return this.averagesOf_Demand_FinancialStanding_PercentOfTargetInventory.get();
	}
	
	private void addJobs(int numberOfJobsToAdd) {
		for(int i = 0; i < numberOfJobsToAdd; i++) {
			this.doesCreateNewJob(this.productType);
			this.timeSinceLastChangeInTheNumberOfJobs = 0;
		}
	}
	
	private void removeJobs(int numberOfJobsToRemove) {
		for(int i = 0; i < numberOfJobsToRemove; i++) {
			for(int j = this.jobs.size() - 1; j >= 0; j--) {
				if(this.jobs.get(j).getPerson() != null) {
					this.jobs.get(j).getPerson().setJob(null);
					this.jobs.get(j).setPersonNull();
					this.jobs.remove(j);
					this.timeSinceLastChangeInTheNumberOfJobs = 0;
					break;
				}else {
					this.jobs.remove(j);
					this.timeSinceLastChangeInTheNumberOfJobs = 0;
				}
			}
		}
	}
	
	private int getNumberOfUnfilledPositions() {
		int unfilledPositions = 0;
		for(Job job : this.getJobs()) {
			if(job.getPerson() == null){
				unfilledPositions++;
			}
		}	
		return unfilledPositions;
	}
	
	/*
	 * If the industry has enough money this method will purchase the "quantity" amount of the raw material and return true if successful. If there was no successful purchase this method returns false. 
	 */
	private boolean getRawMaterials(int quantity) {
		if(quantity <= 0) {
			throw new IllegalArgumentException("Cannot retrieve a negative or zero amount of raw materials");
		}
		boolean success = false;
		Industry rawMaterialIndustry = THE_WORLD.getInstance().getIndustry(EConsumable.RAW_MATERIAL);
		if(rawMaterialIndustry != this && rawMaterialIndustry.hasQuantity(quantity)) {
			int lessTax = rawMaterialIndustry.getPriceForConsumableExcludingSalesTax(EConsumable.RAW_MATERIAL, quantity);
			int moneyNeeded = rawMaterialIndustry.getTotalPriceForConsumable(EConsumable.RAW_MATERIAL, quantity);
			if(moneyNeeded <= this.money) {
				if(rawMaterialIndustry.doesAcceptPurchaseOfConsumable(EConsumable.RAW_MATERIAL, quantity, new One<Integer>(moneyNeeded))){
					if(moneyNeeded <= 0) {
						System.out.println("error 0 money needed");
					}
					int taxPortion = moneyNeeded - lessTax;;
					this.taxPaid += moneyNeeded - lessTax;
					this.money = this.money - moneyNeeded;
					this.addToDailyCosts(moneyNeeded - taxPortion);
					this.inputProductQuantity += quantity;
					success = true;
				}
			}
		}
		return success;
	}

	/*
	 * This method returns the total amount of money required to purchase the "quantity" amount of the Raw Material consumable.
	 * 
	 * This method is not applicable to the Raw Material industry itself and would return 0 if called by the Raw Material industry. 
	 */
	private int getRawMaterialCost(int quantity) {
		int cost = 0;
		Industry rawMaterialIndustry = THE_WORLD.getInstance().getIndustry(EConsumable.RAW_MATERIAL);
		if(rawMaterialIndustry != this && rawMaterialIndustry.hasQuantity(quantity)) {
			cost = rawMaterialIndustry.getTotalPriceForConsumable(EConsumable.RAW_MATERIAL, quantity);
		}
		return cost;
	}
	
	/*
	 * This method is conditional and will adjust the product's sale price if criteria is met. 
	 */
	private void adjustProductSalePrice() {
		if( this.getIndustryRecords().size() > DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE && this.productSalePrice.getTimeSinceLastChange() > DeterminationHelper.MINIMUM_TIME_BETWEEN_SALE_PRICE_CHANGES ) {
			Three<Integer, Double, Double> averagesOf_Production_FinancialStanding_PercentOfTargetInventory = 
																					this.getAveragesOf_Demand_FinancialStanding_PercentOfTargetInventory();
			double averageFinancialStanding = averagesOf_Production_FinancialStanding_PercentOfTargetInventory.getSecond();
			double percentOfTargetInventory = averagesOf_Production_FinancialStanding_PercentOfTargetInventory.getThird();
			double costToProduce = ((double)(this.getRawMaterialCost(this.adjustedEmployeeRequiredInput.get().intValue()) + THE_WORLD.getInstance().getGovernment().getMinimumWage())) /
																		( this.adjustedEmployeeProduction.get() ) ;
			if(averageFinancialStanding > DeterminationHelper.LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD && 
						this.getProductSalePrice() != calculateSalePrice(costToProduce, DeterminationHelper.MULTIPLICAR_TO_DETERMINE_LOWEST_AND_SECOND_LOWEST_SALE_PRICE, 0, DeterminationHelper.DOES_ALLOW_A_ZERO_SALE_PRICE)) {
				this.setProductSalePrice(calculateSalePrice(costToProduce, DeterminationHelper.MULTIPLICAR_TO_DETERMINE_LOWEST_AND_SECOND_LOWEST_SALE_PRICE, 0, DeterminationHelper.DOES_ALLOW_A_ZERO_SALE_PRICE));
			}else if(averageFinancialStanding > DeterminationHelper.SECOND_LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD && 
								averageFinancialStanding <= DeterminationHelper.LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD && 
								this.getProductSalePrice() != calculateSalePrice(costToProduce, DeterminationHelper.MULTIPLICAR_TO_DETERMINE_LOWEST_AND_SECOND_LOWEST_SALE_PRICE, 
																												DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_SECOND_LOWEST_AND_MIDDLE_SALE_PRICE, false)) {
				this.setProductSalePrice( calculateSalePrice(costToProduce, DeterminationHelper.MULTIPLICAR_TO_DETERMINE_LOWEST_AND_SECOND_LOWEST_SALE_PRICE, DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_SECOND_LOWEST_AND_MIDDLE_SALE_PRICE, false) );
			}else if(averageFinancialStanding > DeterminationHelper.SECOND_HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD && 
								averageFinancialStanding <= DeterminationHelper.SECOND_LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD && 
								this.getProductSalePrice() != calculateSalePrice(costToProduce, 1, DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_SECOND_LOWEST_AND_MIDDLE_SALE_PRICE) ) {
				this.setProductSalePrice( calculateSalePrice(costToProduce, 1, DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_SECOND_LOWEST_AND_MIDDLE_SALE_PRICE) );
			}else if( ( averageFinancialStanding <= DeterminationHelper.SECOND_HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD || 
																				percentOfTargetInventory < DeterminationHelper.SECOND_HIGHEST_SALE_PRICE_PERCENT_OF_TARGET_INVENTORY_OTHER_POSSIBLE_THRESHOLD ) &&
								this.getProductSalePrice() != calculateSalePrice(costToProduce, 1, DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_SECOND_HIGHEST_SALE_PRICE) ) {
				this.setProductSalePrice( calculateSalePrice(costToProduce, 1, DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_SECOND_HIGHEST_SALE_PRICE) );
			}else if( ( averageFinancialStanding <= DeterminationHelper.HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD || 
																				percentOfTargetInventory < DeterminationHelper.HIGHEST_SALE_PRICE_PERCENT_OF_TARGET_INVENTORY_OTHER_POSSIBLE_THRESHOLD ) &&
								this.getProductSalePrice() != calculateSalePrice(costToProduce, 1, DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_HIGHEST_SALE_PRICE) ) {
				this.setProductSalePrice( calculateSalePrice(costToProduce, 1, DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_HIGHEST_SALE_PRICE) );
			}
		}
	}
	
	private int calculateSalePrice(double costToProduce, double multiplicarToDetermineSalePrice, double amountToAddToDetermineSalePrice, boolean doesAllowZeroPrice) {
		return doesAllowZeroPrice || calculateSalePrice(costToProduce, multiplicarToDetermineSalePrice, amountToAddToDetermineSalePrice) > 0 ? 
																									calculateSalePrice(costToProduce, multiplicarToDetermineSalePrice, amountToAddToDetermineSalePrice) : 1;
	}
	
	private int calculateSalePrice(double costToProduce, double multiplicarToDetermineSalePrice, double amountToAddToDetermineSalePrice) {
		return (int)(costToProduce * multiplicarToDetermineSalePrice + amountToAddToDetermineSalePrice);
	}

	/* 
	 * This method is conditional. If criteria is met this method will adjust the industry's number of jobs. 
	 */
	private void adjustNumberOfJobs() {
		if(this.getIndustryRecords().size() > DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE && 
								(this.getProductType() != EConsumable.DISCRETIONARY || THE_WORLD.getInstance().getTime() >= DeterminationHelper.INITIAL_WAIT_BEFORE_DESCRETIONARY_INDUSTRY_CHANGES_THE_NUMBER_OF_JOBS) &&
								this.timeSinceLastChangeInTheNumberOfJobs >= DeterminationHelper.TIME_TO_WAIT_BETWEEN_CHANGES_IN_THE_NUMBER_OF_JOBS) {
			Three<Integer, Double, Double> averagesOf_Demand_FinancialStanding_PercentOfTargetInventory = 
																	this.getAveragesOf_Demand_FinancialStanding_PercentOfTargetInventory();
			double roughFinancialStanding = averagesOf_Demand_FinancialStanding_PercentOfTargetInventory.getSecond();
			double percentOfTargetInventory = averagesOf_Demand_FinancialStanding_PercentOfTargetInventory.getThird();
			int averageDemand = averagesOf_Demand_FinancialStanding_PercentOfTargetInventory.getFirst();
			if(!this.wasProductionBelowExpectation) {
				if( (percentOfTargetInventory > DeterminationHelper.RATIO_OF_TARGET_INVENTORY_FOR_THESHOLD_TO_REMOVE_JOBS && this.dailyProduction > averageDemand ) || 
									( percentOfTargetInventory > DeterminationHelper.RATIO_OF_TARGET_INVENTORY_AND_RATIO_OF_PRODUCTION_OVER_AVERAGE_DEMAND_FOR_THESHOLD_TO_REMOVE_JOBS.getFirst() && 
											this.dailyProduction > averageDemand * DeterminationHelper.RATIO_OF_TARGET_INVENTORY_AND_RATIO_OF_PRODUCTION_OVER_AVERAGE_DEMAND_FOR_THESHOLD_TO_REMOVE_JOBS.getSecond() ) ) {
					int numberOfJobsToRemove = (int) ( (this.dailyProduction - averageDemand) / this.adjustedEmployeeProduction.get() );
					if(((double)numberOfJobsToRemove/this.getJobs().size()) > DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_REMOVED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst() && 
							this.getJobs().size() >= DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_REMOVED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getSecond()) {
						numberOfJobsToRemove = ((int)(this.getJobs().size() * DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_REMOVED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst()));
					}
					this.removeJobs(numberOfJobsToRemove);
				}else if(percentOfTargetInventory < DeterminationHelper.RATIO_OF_TARGET_INVENTORY_FOR_THESHOLD_TO_ADD_JOBS && 
						this.dailyProduction < averageDemand && 
									(this.isEssentialIndustry() || roughFinancialStanding >= DeterminationHelper.MINIMUM_FINANACIAL_STANDING_FOR_DESCRETIONARY_INDUSTRY_TO_ADD_JOBS)) {
					int numberOfJobsToAddBasedOnDemandVsProduction = 0;
					if(percentOfTargetInventory < DeterminationHelper.RATIO_OF_TARGET_INVENTORY_CEILING_FOR_THESHOLD_TO_CALCULATE_JOBS_TO_ADD_WITH_CURRENT_INSTEAD_OF_AVERAGE_DEMAND && averageDemand < this.dailyDemand) {
						numberOfJobsToAddBasedOnDemandVsProduction = (int)	(	(this.dailyDemand - this.dailyProduction) / this.adjustedEmployeeProduction.get()	) + 1;
					}else {
						numberOfJobsToAddBasedOnDemandVsProduction = (int)	(	(averageDemand - this.dailyProduction) / this.adjustedEmployeeProduction.get() ) + 1;
					}
					
					int targetInventory = (int)(averageDemand * DeterminationHelper.MULTIPLIER_OF_DEMAND_TO_DETERMINE_TARGET_PRODUCT_INVENTORY);
					double amountNeeded = ( (1d - percentOfTargetInventory) * targetInventory ) / this.adjustedEmployeeProduction.get();
					int numberOfJobsToAddBasedOnInventory = (int)(amountNeeded/DeterminationHelper.DIVISOR_OF_AMOUNT_NEEDED_TO_MEET_TARGET_INVENTORY_TO_CALCULATE_ADDITIONAL_JOBS_BASED_ONLY_ON_THIS);
					int totalNumberOfJobsToAdd = Math.max( (numberOfJobsToAddBasedOnInventory + numberOfJobsToAddBasedOnDemandVsProduction ) - this.getNumberOfUnfilledPositions(), 0);
					
					if(totalNumberOfJobsToAdd * (THE_WORLD.getInstance().getGovernment().getMinimumWage() + this.getRawMaterialCost(this.adjustedEmployeeRequiredInput.get().intValue())) > this.getMoney()) {
						boolean addingMoreJobsThanTheyCanAfford = true;
						while(addingMoreJobsThanTheyCanAfford && totalNumberOfJobsToAdd > 0) {
							totalNumberOfJobsToAdd--;
							if(! (totalNumberOfJobsToAdd * (THE_WORLD.getInstance().getGovernment().getMinimumWage() + this.getRawMaterialCost(this.adjustedEmployeeRequiredInput.get().intValue())) > this.getMoney())) {
								addingMoreJobsThanTheyCanAfford = false;
							}
						}
					}
					
					if(this.getProductType() == EConsumable.DISCRETIONARY && 
										((double)totalNumberOfJobsToAdd/this.getJobs().size()) >= DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_DISCRETIONARY_INDUSTRY_CAN_ADD_AT_ONCE_IF_IT_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst() && 
												this.getJobs().size() > DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_DISCRETIONARY_INDUSTRY_CAN_ADD_AT_ONCE_IF_IT_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getSecond()) { //0.01
						totalNumberOfJobsToAdd = ((int)(this.getJobs().size() * DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_DISCRETIONARY_INDUSTRY_CAN_ADD_AT_ONCE_IF_IT_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst() ));
					}else if(((double)totalNumberOfJobsToAdd/this.getJobs().size()) > DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_ADDED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst() && 
							this.getJobs().size() >= DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_ADDED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getSecond()) {
						totalNumberOfJobsToAdd = ((int)(this.getJobs().size() * DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_ADDED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst()));
					}
					
					this.addJobs(totalNumberOfJobsToAdd);
				}
			}
		}
	}

	/*
	 * This method is conditional. If criteria is met (zero or near zero funds and zero or near zero product to sell) the this method will ask the government for assistance. 
	 */
	private void IfADyingIndustryAskingForHelp() {
		if( this.getProductType() == EConsumable.DISCRETIONARY && this.getIndustryRecords().size() > DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE) {
			double averageFinancialStanding = this.getAveragesOf_Demand_FinancialStanding_PercentOfTargetInventory().getSecond();
			if(averageFinancialStanding < DeterminationHelper.AVERAGE_FINANCIAL_STANDING_AND_PRODUCT_QUANTITY_THRESHOLD_TO_SIGNAL_INSUFFICIENT_ASSETS.getFirst() && 
					this.getProductQuantity() < DeterminationHelper.AVERAGE_FINANCIAL_STANDING_AND_PRODUCT_QUANTITY_THRESHOLD_TO_SIGNAL_INSUFFICIENT_ASSETS.getSecond()) {
				THE_WORLD.getInstance().getGovernment().doesHelpDyingIndustry(this);
			}
		}
	}
	
	/*
	 * The IndustryRecord class is used to save industry history
	 */
	public class IndustryRecord {
		private int money;
		private int productQuantity;
		private int inputConsumableQuantity;
		private double totalProduction;
		private int productDemand;
		private int moneySpent;
		private int numberOfEmployees;
		private int unfilledPositions;
		private int unmetDemand;
		private boolean wasProductionBelowExpectation;
		public double roughFinancialStanding;
		private int wagesPaid;
		private int taxPaid;
		private int revenue;
		private int salePrice;
		private int quantitySold;
		public IndustryRecord(int money, int productQuantity, int inputConsumableQuantity, double totalProduction, int productDemand, int moneySpent, int numberOfEmployees, int unfilledPositions,
				int unmetDemand, boolean wasProductionBelowExpectation, double roughFinancialStanding, int wagesPaid, int taxPaid, int revenue, int salePrice, int quantitySold) {
			this.money = money;
			this.productQuantity = productQuantity;
			this.inputConsumableQuantity = inputConsumableQuantity;
			this.totalProduction = totalProduction;
			this.productDemand = productDemand;
			this.moneySpent = moneySpent;
			this.numberOfEmployees = numberOfEmployees;
			this.unfilledPositions = unfilledPositions;
			this.unmetDemand = unmetDemand;
			this.wasProductionBelowExpectation = wasProductionBelowExpectation;
			this.roughFinancialStanding = roughFinancialStanding;
			this.wagesPaid =  wagesPaid;
			this.taxPaid = taxPaid;
			this.revenue = revenue;
			this.salePrice = salePrice;
			this.quantitySold = quantitySold;
		}
		public int getMoney() {return this.money;}
		public int getProductQuantity() {return this.productQuantity;}
		public int getInputConsumableQuantity() {return this.inputConsumableQuantity;}
		public double getTotalProduction() {return this.totalProduction;}
		public int getProductDemand() {return this.productDemand;}
		public int getMoneySpent() {return this.moneySpent;}
		public int getNumberOfEmployees() {return this.numberOfEmployees;}
		public int getUnfilledPositions() {return this.unfilledPositions;}
		public int getUnmetDemand() {return this.unmetDemand;}
		public boolean wasProductionBelowExpectation() {return this.wasProductionBelowExpectation;}
		public double getRoughFinancialStanding() {return this.roughFinancialStanding;}
		public int getWagesPaid() {return this.wagesPaid;}
		public int getTaxPaid() {return this.taxPaid;}
		public int getRevenue() {return this.revenue;}
		public int getSalePrice() {return this.salePrice;}
		public int getQuantitySold() {return this.quantitySold;}
	}
	
	
	
}





