package people;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

	Three Industry ABCE Model is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	Three Industry ABCE Model is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 * Ties a Person to his or her industry of employment
 * 
 * @author Ian Stuart
 */
public class Job{
	private final EConsumable typeOfProductToProduce;
	private final Industry industry;
	private Person person = null;

	public Job(Industry industry, EConsumable typeOfProductToProduce) {
		this.typeOfProductToProduce = typeOfProductToProduce;
		this.industry = industry;
	}

	public EConsumable getTypeOfProductToProduce() {
		return this.typeOfProductToProduce;
	}

	public Industry getIndustry() {
		return this.industry;
	}
	
	public int getDailyPay() {
		return THE_WORLD.getInstance().getGovernment().getMinimumWage();
	}

	public Person getPerson() {
		return this.person;
	}

	public void setPersonNull() {this.person = null;}
	
	public boolean doesSetPerson(Person person) {
		boolean success = false;
		if ( (this.person == null && person != null && person.isAdult() && person != this.person)) {
			this.person = person;
			person.setJob(this);
			success = true;
		} else if(person == null){
			this.person = null;
			success = true;
		}
		return success;
	}

	public boolean isEssential() { return industry.isEssentialIndustry();}
}
