package people;
import java.util.ArrayList;

import helper.DeterminationHelper;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 * A romanticRelationship in the model is the basis from which two adults share a home and opens the possibility of the pair having children.
 * 
 * @author Ian Stuart
 */
public class RomanticRelationship extends AbstractRelationship{
	private Pregnancy pregnancy = null;
	private ArrayList<Person> sharedChildren = new ArrayList<Person>();
	private boolean heterogeneousCouple;

	public RomanticRelationship(Person first, Person second) {
		super(first, second);
		if(first.getReproductiveGender() == EReproductiveGender.MALE && second.getReproductiveGender() == EReproductiveGender.FEMALE ||
					first.getReproductiveGender() == EReproductiveGender.FEMALE && second.getReproductiveGender() == EReproductiveGender.MALE) {
			heterogeneousCouple = true;
		}else {
			heterogeneousCouple = false;
		}
	}
	
	public boolean canReproduce() {
		if(heterogeneousCouple && this.firstPerson.getAge() <= DeterminationHelper.MAX_FERTILE_AGE && this.secondPerson.getAge() <= DeterminationHelper.MAX_FERTILE_AGE && //) { //&&
									this.firstPerson.isAlive() && this.secondPerson.isAlive()) {//let people remain "in a relationship" if their significant other is dead, but they will no longer reproduce...
			return true;
		}
		return false;
	}
	
	public Pregnancy getPregnancy() { return this.pregnancy; }
	
	public void setPregnant() { this.pregnancy = new Pregnancy();}
	
	public void clearPregnancy() { this.pregnancy = null;}
	
	public ArrayList<Person> getSharedChildren(){ return this.sharedChildren;}
	
	public void addSharedChild(Person person) { this.sharedChildren.add(person);}
	
	public class Pregnancy{
		public static final int duration = THE_WORLD.UNITS_OF_TIME_IN_YEAR * 3 / 4;
		private int since;
		public Pregnancy(){
			this.since = THE_WORLD.getInstance().getTime();
		}
		
		public boolean isInLabor() {
			if(THE_WORLD.getInstance().getTime() - this.since >= Pregnancy.duration) {
				return true;
			}
			return false;
		}
	}
}
