package people;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import helper.DeterminationHelper;
import utility.Two;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

	Three Industry ABCE Model is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	Three Industry ABCE Model is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 * The THE_WORLD class keeps the current time step and is used as a global point of access in this Three Industry ABCE Model.
 * In terms of agent based modeling, this could be considered the environment of the model. 
 * 
 * @author Ian Stuart
 */
public class THE_WORLD {
	private static final THE_WORLD instance  = new THE_WORLD();
	public static final int UNITS_OF_TIME_IN_YEAR = 12;
	private final HashMap<EConsumable, Industry> industries = new HashMap<EConsumable, Industry>();
	private final ArrayList<Person> people = new ArrayList<Person>();
	private final LinkedList<Two<Integer, Person>> eraseDeadPeoplesParentsCounter = new LinkedList<Two<Integer, Person>>();
	private Government government;
	private int time = 0;
	private int year = 0;

	private THE_WORLD(){}
	
	/*
	 * Resets THE_WORLD variables to their starting point. 
	 * The reset method was introduced to allow for multiple simulation runs.
	 */
	public void reset() {
		time = 0;
		year = 0;
		industries.clear();
		Industry.setAdjustProductionRate(1d);
		Industry.setLengthOfWorkDay(1d);
		people.clear();
		eraseDeadPeoplesParentsCounter.clear();
		government = null;
		
		Person.totalFromAssistance = 0;
		Person.totalFromEmployerMinusIncomeTax = 0;
		Person.totalPaidInTaxes = 0;
		Person.totalPaidForNecessities = 0;
		Person.totalPaidForDiscretionaries = 0;
		Person.totalPaidForInheritanceTax = 0;
	}
	
	public HashMap<EConsumable, Industry> getIndustries(){return this.industries;}
	public Industry getIndustry(EConsumable econsumable) { return industries.get(econsumable);}
	public void addIndustry(Industry industry) {this.industries.put(industry.getProductType(), industry);}
	public ArrayList<Person> getPeople(){return this.people;}
	public void addPerson(Person person) {this.people.add(person);}
	
	public boolean removePerson(Person person) { 		
		if(person.getJob() != null) {
			person.getJob().setPersonNull();
			person.setJob(null);
		}

		if(person.getNumberOfKids() < 1) {
			if(person.getRomanticRelationship() == null) {
				person.setBiologicalRelationships(null);
			}
		}else {
			this.eraseDeadPeoplesParentsCounter.addLast(
					new Two<Integer, Person>(this.time + DeterminationHelper.MAX_FERTILE_AGE * DeterminationHelper.NUMBER_OF_GENERATIONS_INCEST_TABOO_EXTENDS * THE_WORLD.UNITS_OF_TIME_IN_YEAR, person));		
		}
		person.getHome().removePerson(person);
		person.setHome(null);
		person.setRomanticRelationship(null);
		return this.people.remove(person);
	}
	
	public Government getGovernment() {return this.government;}
	public void setGovernment(Government government) {this.government = government;}	
	public static THE_WORLD getInstance() {return instance;}
	public int getTime() {return this.time;}
	
	public void progressTime(){
		if(time%(UNITS_OF_TIME_IN_YEAR * 5) == 0) { //every five years we check to see if there are references of dead people that can be released
			while(!this.eraseDeadPeoplesParentsCounter.isEmpty()) {
				if(this.eraseDeadPeoplesParentsCounter.peekFirst().getFirst() <= time) {
					this.eraseDeadPeoplesParentsCounter.peekFirst().getSecond().setHome(null);
					this.eraseDeadPeoplesParentsCounter.pollFirst().getSecond().setBiologicalRelationships(null);
				}else {
					break;
				}
			}
		}
		
		if(THE_WORLD.getInstance().getTime() % 3 - 2 == 0 &&
				DeterminationHelper.MAX_ADJUSTED_PRODUCTION_RATE > Industry.getAdjustProductionRate()) {
			Industry.setAdjustProductionRate(Industry.getAdjustProductionRate() * 1.0034717485095025);
		}
		
		this.time++; 
		this.year = this.time / UNITS_OF_TIME_IN_YEAR;
	}
	
	public int getYear() {return this.year;}
}








