package simulate;
import helper.DeterminationHelper;
import helper.RomanticRelationshipHelper;
import people.BiologicalRelationship;
import people.EReproductiveGender;
import people.Government;
import people.Person;
import people.RomanticRelationship;
import people.THE_WORLD;
import utility.Two;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

	Three Industry ABCE Model is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	Three Industry ABCE Model is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 * Helps to instantiate person objects.
 * 
 * @author Ian Stuart
 *
 */
public class PersonMill {
	public PersonMill() {}
	
	/*
	 * instantiates the input number of adults (as well as children in addition to this number) as Person objects
	 */
	public void populateWorld(int numberInitialOfAdults) {
		if(numberInitialOfAdults < 0) {
			throw new IllegalArgumentException("The numberInitialOfAdults argument cannot be negative.");
		}
		Person person;
		for(int i = 0; i < numberInitialOfAdults; i++) {
			person = constructAndInstantiateAdultPerson();
			THE_WORLD.getInstance().addPerson(person);
		}		
		RomanticRelationshipHelper relationshipHelper = new RomanticRelationshipHelper();
		relationshipHelper.initializeAdultPopulation(THE_WORLD.getInstance().getPeople());
	}

	/*
	 * creates and returns a Person object that with age 0 at the current time step.
	 */
	public Person giveBirth(RomanticRelationship parents) {
		return giveBirth(parents, 0);
	}
	
	/*
	 * creates and returns a Person object the designated age at the current time step.
	 */
	public Person giveBirthInThePast(RomanticRelationship parents, int ageInMonths) {
		if(ageInMonths < 0) {
			throw new IllegalArgumentException("The ageInMonths argument cannot be negative.");
		}
		return giveBirth(parents, ageInMonths);
	}
	
	private Person giveBirth(RomanticRelationship parents, int ageInMonths) {
		if(ageInMonths < 0) {
			throw new IllegalArgumentException("The ageInMonths argument cannot be negative.");
		}
		Person dad = null;
		Person mom = null;
		if(parents.getPeople().getFirst().getReproductiveGender() == EReproductiveGender.MALE &&
				parents.getPeople().getSecond().getReproductiveGender() == EReproductiveGender.FEMALE) {
			dad = parents.getPeople().getFirst();
			mom = parents.getPeople().getSecond();
		}else if(parents.getPeople().getFirst().getReproductiveGender() == EReproductiveGender.FEMALE &&
				parents.getPeople().getSecond().getReproductiveGender() == EReproductiveGender.MALE) {
			mom = parents.getPeople().getFirst();
			dad = parents.getPeople().getSecond();
		}
		
		if(mom != null && dad != null && dad.isAlive() && mom.isAlive() && ( (parents.getPregnancy() != null && parents.getPregnancy().isInLabor()) || ageInMonths > 0 ) ) {
			Person newPerson;
			if(ageInMonths == 0){
				newPerson = constructAndInstantiateNewborn(parents.getPeople());
			}else {
				newPerson = constructAndInstantiateChild(parents.getPeople(), ageInMonths);
			}
			mom.getHome().addPerson(newPerson);
			newPerson.setHome(mom.getHome());			
			THE_WORLD.getInstance().addPerson(newPerson);
			parents.addSharedChild(newPerson);
			parents.clearPregnancy();
			return newPerson;
		}else {
			return null;
		}
	}
	
	private Person constructAndInstantiateAdultPerson() {
		Person newPerson = new Person(DeterminationHelper.getEReproductiveGender());
		newPerson.instantiate();
		return newPerson;
	}
	
	private Person constructAndInstantiateNewborn(Two<Person, Person> parents) {
		EReproductiveGender reproductiveGender = DeterminationHelper.getEReproductiveGender();
		Person newPerson = new Person(reproductiveGender, 0);
		newPerson.instantiate();
		setBiologicalRelationships(parents, newPerson);
		Government.moneyAdded += THE_WORLD.getInstance().getGovernment().getParentToANewChildPaymentToFabricate()*2;
		Government.totalMoneyAddedAllTime += THE_WORLD.getInstance().getGovernment().getParentToANewChildPaymentToFabricate()*2;
		parents.getFirst().setMoney(parents.getFirst().getMoney() + THE_WORLD.getInstance().getGovernment().getParentToANewChildPaymentToFabricate());
		parents.getSecond().setMoney(parents.getSecond().getMoney() + THE_WORLD.getInstance().getGovernment().getParentToANewChildPaymentToFabricate());
		return newPerson;
	}
	
	private Person constructAndInstantiateChild(Two<Person, Person> parents, int ageInMonths) {
		if(ageInMonths < 0) {
			throw new IllegalArgumentException("The ageInMonths argument cannot be negative.");
		}
		EReproductiveGender reproductiveGender = DeterminationHelper.getEReproductiveGender();
		Person newPerson = new Person(reproductiveGender, ageInMonths);
		newPerson.instantiate();
		setBiologicalRelationships(parents, newPerson);
		return newPerson;
	}
	
	private void setBiologicalRelationships(Two<Person, Person> parents, Person newPerson) {
		BiologicalRelationship  bioRelationshipOne = new BiologicalRelationship(parents.getFirst(), newPerson);
		newPerson.addBiologicalRelationship(bioRelationshipOne.getBiologicalRelationship(newPerson), bioRelationshipOne);
		parents.getFirst().addBiologicalRelationship(bioRelationshipOne.getBiologicalRelationship(parents.getFirst()), bioRelationshipOne);
		BiologicalRelationship  bioRelationshipTwo = new BiologicalRelationship(parents.getSecond(), newPerson);
		newPerson.addBiologicalRelationship(bioRelationshipTwo.getBiologicalRelationship(newPerson), bioRelationshipTwo);
		parents.getSecond().addBiologicalRelationship(bioRelationshipTwo.getBiologicalRelationship(parents.getSecond()), bioRelationshipTwo);
	}
}

























