package simulate;
import java.util.ArrayList;
import javax.swing.JPanel;
import people.Industry;
import people.EConsumable;
import people.Government;
import people.THE_WORLD;
import display.Chart;
import display.EChart;
import display.MyReport;
import display.OverlappingChartArguments;
import display.TestResults;
import display.Window;
import helper.DeterminationHelper;
import helper.RomanticRelationshipHelper;
import utility.Two;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

	Three Industry ABCE Model is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	Three Industry ABCE Model is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 * Instantiates and runs a simulation and also gets the results displayed.
 * 
 * @author Ian Stuart
 */
public class Scenario {
	private final int initialNumberOfAdults = 41;
	private final int initialNecessityWorkers = 20;
	private final int initialRawMaterialWorkers = 20;
	private final int initialDiscretionaryWorkers = 1;
	private final int adjustIndustryStartingMoney = (int) (80000);
	private final double multiplier = 20;
	
	/*
	 * Instantiates and initializes agents prior to simulation run. 
	 */
	public void Setup() throws Exception {	
		ScenarioValuesTest.checkDeterminationHelperValues();
		
		THE_WORLD.getInstance().setGovernment(new Government( (int)(128000 * multiplier * 0.25) ) );
		
		Industry industry = new Industry((int)(adjustIndustryStartingMoney * multiplier), new Two<EConsumable, Double>(EConsumable.NECESSITY, (4000 * multiplier)), 20, 20 * multiplier);
		int numberOfWorkers = (int)(initialNecessityWorkers * multiplier);
		for(int i = 0; i < numberOfWorkers; i++) {
			industry.doesCreateNewJob(EConsumable.NECESSITY);
		}
		THE_WORLD.getInstance().addIndustry(industry);
		
		industry = new Industry((int)(adjustIndustryStartingMoney * multiplier), new Two<EConsumable, Double>(EConsumable.RAW_MATERIAL, (1000 * multiplier)), 30);
		numberOfWorkers = (int)(initialRawMaterialWorkers * multiplier);
		for(int i = 0; i < numberOfWorkers; i++) {
			industry.doesCreateNewJob(EConsumable.RAW_MATERIAL);
		}
		THE_WORLD.getInstance().addIndustry(industry);
		
		industry = new Industry((int)(adjustIndustryStartingMoney * multiplier), new Two<EConsumable, Double>(EConsumable.DISCRETIONARY, (4 * multiplier)), 12, 40 * multiplier);
		numberOfWorkers = (int)(initialDiscretionaryWorkers * multiplier);
		for(int i = 0; i < numberOfWorkers - 1; i++) {
			industry.doesCreateNewJob(EConsumable.DISCRETIONARY);
		}
		THE_WORLD.getInstance().addIndustry(industry);
		
		PersonMill personMill = new PersonMill();
		personMill.populateWorld((int)(initialNumberOfAdults * multiplier));
	}
	
	/*
	 * Play() includes the main simulation loop.
	 */
	public void Play(int endOfTime) {
		RomanticRelationshipHelper romanticRelationshipHelper = new RomanticRelationshipHelper();	
		while(THE_WORLD.getInstance().getTime() < endOfTime && THE_WORLD.getInstance().getPeople().size() > 0) {		
			romanticRelationshipHelper.UpdateRelationships(THE_WORLD.getInstance().getPeople());
			updateAgents();
			THE_WORLD.getInstance().progressTime();
		}
	}
	
	/*
	 * This is intended to provide the prevalence rate in which simulations end early
	 */
	public void multipleRunsCheckFailRateOnly() throws Exception {
		int iterations = 1000;
		
		Window window = new Window(1100, 600);
		JPanel mainPanel = new JPanel();
		JPanel panel = new JPanel();
		TestResults testResults = new TestResults(Chart.$4x4_MY_IMAC.getFirst(), Chart.$4x4_MY_IMAC.getSecond(), "In Progress", new ArrayList<String>());
		panel.add(testResults);
		mainPanel.add(panel);
		window.add(mainPanel);
		int duration = DeterminationHelper.TIME_FOR_SENARIO_IN_RANGE_OF_EXPONENTIAL_PRODUCTION_RATE_INCREASE;
		int fails = 0;
		Scenario scenario = new Scenario();
		long timeBeforeRunningSimulations = System.currentTimeMillis();
		for(int i = 0; i < iterations; i++) {
			testResults.SetTitle("Processing Simulation Run " + (i + 1) + " out of " + iterations + "...");
			window.repaint();
			testResults.repaint();
			THE_WORLD.getInstance().reset();
			Setup();
			scenario.Play(duration);
			if(THE_WORLD.getInstance().getPeople().size() < 100) {
				fails++;
			}
		}
		long timeAfterRunningSimulations = System.currentTimeMillis();
		testResults.SetTitle("Undesired results in " + fails + " out of " + iterations + " runs. Simulations ran for " + (timeAfterRunningSimulations - timeBeforeRunningSimulations)/1000 + " seconds");
		window.repaint();
		testResults.repaint();
	}
	
	/*
	 * This is perhaps the messiest code of the program...
	 * 
	 * MultipleRuns runs simulation multiple times and collects data from simulation to simulation so that it can be displayed for viewing. 
	 */
	@SuppressWarnings("unchecked")
	public void multipleRuns() throws Exception {
		int iterations = 10;
		
		Window window = new Window(1100, 600);
		JPanel mainPanel = new JPanel();
		JPanel panel = new JPanel();
		TestResults testResults = new TestResults(Chart.$4x4_MY_IMAC.getFirst(), Chart.$4x4_MY_IMAC.getSecond(), "In Progress", new ArrayList<String>());
		panel.add(testResults);
		mainPanel.add(panel);
		window.add(mainPanel);
		int duration = DeterminationHelper.TIME_FOR_SENARIO_IN_RANGE_OF_EXPONENTIAL_PRODUCTION_RATE_INCREASE;
		
		Scenario scenario = new Scenario();
		ArrayList<OverlappingChartArguments> list = new ArrayList<OverlappingChartArguments>();
		Two<Number, Number>[][][] unemploymentDataToGraph = new Two[iterations][][];
		Two<Number, Number>[][][] percentOfLaborDataToGraph = new Two[iterations][][];
		Two<Number, Number>[][][] financialStandingDataToGraph = new Two[iterations][][];
		Two<Number, Number>[][][] industryDataToGraph = new Two[iterations][][];
		Two<Number, Number>[][][] industryDataToGraph2 = new Two[iterations][][];
		Two<Number, Number>[][][] industryDataToGraph3 = new Two[iterations][3][duration];
		Two<Number, Number>[][][] populationDataToGraph = new Two[iterations][4][duration];
		Two<Number, Number>[][][] productPriceDataToGraph = new Two[iterations][3][duration];
		Two<Number, Number>[][][] parentToANewChildPaymentDataToGraph = new Two[iterations][1][duration];
		Two<Number, Number>[][][] retirementAgeDataToGraph = new Two[iterations][1][duration];
		Two<Number, Number>[][][] lengthOfWorkdayDataToGraph = new Two[iterations][1][duration];
		Two<Number, Number>[][][] adjustedGDPDataToGraph = new Two[iterations][1][duration];		
		Two<Number, Number>[][][] populationFinancialDataToGraph = new Two[iterations][1][duration];
		Two<Number, Number>[][][] senarioMoneyDataToGraph = new Two[iterations][1][duration];
		Two<Number, Number>[][][] governmentRevenueDataToGraph = new Two[iterations][][];
		Two<Number, Number>[][][] governmentTaxRatesDataToGraph = new Two[iterations][][];
		Two<Number, Number>[][][] productionRateData = new Two[iterations][][];
		ArrayList<String> multipleRunResults = new ArrayList<String>();
		
		double forCalculatingAveragePopulationTheSecondDecade = 0;
		double forCalculatingAveragePopulationTwentySecondDecade = 0;
		
		double forCalculatingAverageRawMaterialEmployeesSecondDecade =  0;
		double forCalculatingAverageRawMaterialEmployeesTwentySecondDecade = 0;
		double forCalculatingAverageNecessityEmployeesSecondDecade =  0;
		double forCalculatingAverageNecessityEmployeesTwentySecondDecade = 0;
		double forCalculatingAverageDiscretionaryEmployeesSecondDecade = 0;
		double forCalculatingAverageDiscretionaryEmployeesTwentySecondDecade = 0;
		
		double forCalculatingAverageRawMaterialSalePriceSecondDecade = 0;
		double forCalculatingAverageRawMaterialSalePriceTwentySecondDecade = 0;
		double forCalculatingAverageNecessitySalePriceSecondDecade = 0;
		double forCalculatingAverageNecessitySalePriceTwentySecondDecade = 0;
		double forCalculatingAverageDiscretionarySalePriceSecondDecade = 0;
		double forCalculatingAverageDiscretionarySalePriceTwentySecondDecade = 0;
		
		//int fails = 0;
		long timeBeforeRunningSimulations = System.currentTimeMillis();
		for(int i = 0; i < iterations; i++) {
			testResults.SetTitle("Processing Simulation Run " + (i + 1) + " out of " + iterations + "...");
			window.repaint();
			testResults.repaint();
			THE_WORLD.getInstance().reset();
			Setup();
			scenario.Play(duration);
			if(THE_WORLD.getInstance().getPeople().size() < 100) {
				//fails++;
			}
			unemploymentDataToGraph[i] = MyReport.getUnemploymentData();
			percentOfLaborDataToGraph[i] = MyReport.getEmployeesByIndustryData();
			financialStandingDataToGraph[i] = MyReport.getFinancialStandingData();
			industryDataToGraph[i] = MyReport.get_Inventory_Production__Demand_UnfilledPositions_ByIndustryData(EConsumable.NECESSITY);
			industryDataToGraph2[i] = MyReport.get_Inventory_Production__Demand_UnfilledPositions_ByIndustryData(EConsumable.RAW_MATERIAL);
			industryDataToGraph3[i] = MyReport.get_Inventory_Production__Demand_UnfilledPositions_ByIndustryData(EConsumable.DISCRETIONARY);
			populationDataToGraph[i] = MyReport.getPopulationData();
			productPriceDataToGraph[i] = MyReport.getProductPriceData();
			parentToANewChildPaymentDataToGraph[i] = MyReport.getParentToANewChildPaymentData();
			retirementAgeDataToGraph[i] = MyReport.getRetirmentAgeData();
			lengthOfWorkdayDataToGraph[i] = MyReport.getLengthOfWorkdayData();
			adjustedGDPDataToGraph[i] = MyReport.getGDPAdjustedByPurchasingPowerBasedOnAverageConsumption();
			populationFinancialDataToGraph[i] = MyReport.getPopulationFinancialData();
			senarioMoneyDataToGraph[i] = MyReport.getSenarioMoneyData();
			governmentRevenueDataToGraph[i] = MyReport.getGovernmentRevenueData();
			governmentTaxRatesDataToGraph[i] = MyReport.getGovernmentTaxRatesData();
			productionRateData[i] = MyReport.getProductionRateData();
			
			forCalculatingAveragePopulationTheSecondDecade += MyReport.getAveragePopulationWithinTimeRange(120, 240);
			forCalculatingAveragePopulationTwentySecondDecade += MyReport.getAveragePopulationWithinTimeRange(2520, 2640);
			
			forCalculatingAverageRawMaterialEmployeesSecondDecade += MyReport.getAverageEmployeesByIndustryWithinTimeRange(EConsumable.RAW_MATERIAL, 120, 240);
			forCalculatingAverageRawMaterialEmployeesTwentySecondDecade += MyReport.getAverageEmployeesByIndustryWithinTimeRange(EConsumable.RAW_MATERIAL, 2520, 2640);
			forCalculatingAverageNecessityEmployeesSecondDecade += MyReport.getAverageEmployeesByIndustryWithinTimeRange(EConsumable.NECESSITY, 120, 240);
			forCalculatingAverageNecessityEmployeesTwentySecondDecade += MyReport.getAverageEmployeesByIndustryWithinTimeRange(EConsumable.NECESSITY, 2520, 2640);
			forCalculatingAverageDiscretionaryEmployeesSecondDecade += MyReport.getAverageEmployeesByIndustryWithinTimeRange(EConsumable.DISCRETIONARY, 120, 240);
			forCalculatingAverageDiscretionaryEmployeesTwentySecondDecade += MyReport.getAverageEmployeesByIndustryWithinTimeRange(EConsumable.DISCRETIONARY, 2520, 2640);
			
			forCalculatingAverageRawMaterialSalePriceSecondDecade += MyReport.getAverageConsumableSalePriceWithinTimeRange(EConsumable.RAW_MATERIAL, 120, 240);
			forCalculatingAverageRawMaterialSalePriceTwentySecondDecade += MyReport.getAverageConsumableSalePriceWithinTimeRange(EConsumable.RAW_MATERIAL, 2520, 2640);
			forCalculatingAverageNecessitySalePriceSecondDecade += MyReport.getAverageConsumableSalePriceWithinTimeRange(EConsumable.NECESSITY, 120, 240);
			forCalculatingAverageNecessitySalePriceTwentySecondDecade += MyReport.getAverageConsumableSalePriceWithinTimeRange(EConsumable.NECESSITY, 2520, 2640);
			forCalculatingAverageDiscretionarySalePriceSecondDecade += MyReport.getAverageConsumableSalePriceWithinTimeRange(EConsumable.DISCRETIONARY, 120, 240);
			forCalculatingAverageDiscretionarySalePriceTwentySecondDecade += MyReport.getAverageConsumableSalePriceWithinTimeRange(EConsumable.DISCRETIONARY, 2520, 2640);
		}
		long timeAfterRunningSimulations = System.currentTimeMillis();
		String note = getMultipleRunChartNote(iterations);
		list.add( new OverlappingChartArguments( MyReport.getFinancialStandingChartKey(note), financialStandingDataToGraph, EChart.LINE_GRAPH) );
		list.add( new OverlappingChartArguments( MyReport.get_Inventory_Production__Demand_UnfilledPositions_ByIndustryChartKey(EConsumable.NECESSITY, note), industryDataToGraph, EChart.LINE_GRAPH) );
		list.add( new OverlappingChartArguments( MyReport.get_Inventory_Production__Demand_UnfilledPositions_ByIndustryChartKey(EConsumable.RAW_MATERIAL, note), industryDataToGraph2, EChart.LINE_GRAPH) );
		list.add( new OverlappingChartArguments( MyReport.get_Inventory_Production__Demand_UnfilledPositions_ByIndustryChartKey(EConsumable.DISCRETIONARY, note), industryDataToGraph3, EChart.LINE_GRAPH) );
		list.add( new OverlappingChartArguments( MyReport.getPopulationChartKey(note), populationDataToGraph, EChart.LINE_GRAPH));
		list.add( new OverlappingChartArguments( MyReport.getProductPriceChartKey(note), productPriceDataToGraph, EChart.LINE_GRAPH));
		list.add( new OverlappingChartArguments( MyReport.getParentToANewChildPaymentChartKey(note), parentToANewChildPaymentDataToGraph, EChart.LINE_GRAPH));
		list.add( new OverlappingChartArguments( MyReport.getRetirmentAgeChartKey(note), retirementAgeDataToGraph, EChart.LINE_GRAPH));
		list.add( new OverlappingChartArguments( MyReport.getLengthOfWorkdayChartKey(note), lengthOfWorkdayDataToGraph, EChart.LINE_GRAPH));
		list.add( new OverlappingChartArguments( MyReport.getGDPAdjustedByPurchasingPowerBasedOnAverageConsumptionChartKey(note), adjustedGDPDataToGraph, EChart.LINE_GRAPH));
		list.add( new OverlappingChartArguments( MyReport.getPopulationFinancialChartKey(note), populationFinancialDataToGraph, EChart.LINE_GRAPH));
		list.add(new OverlappingChartArguments( MyReport.getGovernmentTaxRatesChartKey(note), governmentTaxRatesDataToGraph, EChart.LINE_GRAPH));
		list.add( new OverlappingChartArguments( MyReport.getUnemploymentChartKey(note), unemploymentDataToGraph, EChart.FILLED_LINE_GRAPH) );
		list.add(new OverlappingChartArguments( MyReport.getGovernmentRevenueChartKey(note), governmentRevenueDataToGraph, EChart.STACKED_AREA_CHART));
		list.add(new OverlappingChartArguments( MyReport.getEmployeesByIndustryChartKey(note), percentOfLaborDataToGraph, EChart.STACKED_AREA_CHART));
		list.add(new OverlappingChartArguments( MyReport.getSenarioMoneyChartKey(note), senarioMoneyDataToGraph, EChart.STACKED_AREA_CHART));
		list.add(new OverlappingChartArguments( MyReport.getProductionRateChartKey(note), productionRateData, EChart.LINE_GRAPH));

		
		multipleRunResults.add("Average Population of second decade :" + String.format("%3.1f",forCalculatingAveragePopulationTheSecondDecade/iterations));
		multipleRunResults.add("Average Population of twenty-second decade  :" + String.format("%3.1f",forCalculatingAveragePopulationTwentySecondDecade/iterations));
		
		forCalculatingAverageRawMaterialEmployeesSecondDecade /= iterations;
		forCalculatingAverageRawMaterialEmployeesTwentySecondDecade /= iterations;
		forCalculatingAverageNecessityEmployeesSecondDecade /= iterations;
		forCalculatingAverageNecessityEmployeesTwentySecondDecade /= iterations;
		forCalculatingAverageDiscretionaryEmployeesSecondDecade /= iterations;
		forCalculatingAverageDiscretionaryEmployeesTwentySecondDecade /= iterations;
		
		double averageTotalEmployeesSecondDecade = forCalculatingAverageRawMaterialEmployeesSecondDecade + forCalculatingAverageNecessityEmployeesSecondDecade + forCalculatingAverageDiscretionaryEmployeesSecondDecade;
		double averageTotalEmployeesTwentySecondDecade = forCalculatingAverageRawMaterialEmployeesTwentySecondDecade + forCalculatingAverageNecessityEmployeesTwentySecondDecade + forCalculatingAverageDiscretionaryEmployeesTwentySecondDecade;

		
		multipleRunResults.add("Average Share of workforce in RawMaterials for second decade :" + String.format("%3.1f", forCalculatingAverageRawMaterialEmployeesSecondDecade/averageTotalEmployeesSecondDecade * 100) + "%");
		multipleRunResults.add("Average Share of workforce in RawMaterials for twenty-second decade :" + String.format("%3.1f", forCalculatingAverageRawMaterialEmployeesTwentySecondDecade/averageTotalEmployeesTwentySecondDecade * 100) + "%");
		multipleRunResults.add("Average Share of workforce in Neccessities for second decade :" + String.format("%3.1f", forCalculatingAverageNecessityEmployeesSecondDecade/averageTotalEmployeesSecondDecade * 100) + "%");
		multipleRunResults.add("Average Share of workforce in Neccessities for twenty-second decade :" + String.format("%3.1f", forCalculatingAverageNecessityEmployeesTwentySecondDecade/averageTotalEmployeesTwentySecondDecade * 100) + "%");
		multipleRunResults.add("Average Share of workforce in Discretionaries for second decade :" + String.format("%3.1f", forCalculatingAverageDiscretionaryEmployeesSecondDecade/averageTotalEmployeesSecondDecade * 100) + "%");
		multipleRunResults.add("Average Share of workforce in Discretionaries for twenty-second decade :" + String.format("%3.1f", forCalculatingAverageDiscretionaryEmployeesTwentySecondDecade/averageTotalEmployeesTwentySecondDecade * 100) + "%");
		
		multipleRunResults.add("Average Raw Material Unit Sale Price for the second decade :" + String.format("%3.1f", forCalculatingAverageRawMaterialSalePriceSecondDecade/iterations));
		multipleRunResults.add("Average Raw Material Unit Sale Price for twenty-second decade :" + String.format("%3.1f", forCalculatingAverageRawMaterialSalePriceTwentySecondDecade/iterations));
		multipleRunResults.add("Average Necessity Unit Sale Price for the second decade :" + String.format("%3.1f", forCalculatingAverageNecessitySalePriceSecondDecade/iterations));
		multipleRunResults.add("Average Necessity Unit Sale Price for twenty-second decade :" + String.format("%3.1f", forCalculatingAverageNecessitySalePriceTwentySecondDecade/iterations));
		multipleRunResults.add("Average Discretionary Unit Sale Price for the second decade :" + String.format("%3.1f", forCalculatingAverageDiscretionarySalePriceSecondDecade/iterations));
		multipleRunResults.add("Average Discretionary Unit Sale Price for twenty-second decade :" + String.format("%3.1f", forCalculatingAverageDiscretionarySalePriceTwentySecondDecade/iterations));
		
		multipleRunResults.add("      Simulation ran " + iterations + " times in " + (timeAfterRunningSimulations - timeBeforeRunningSimulations)/1000 + " seconds" );
		multipleRunResults.add("   ( To see the results of just a single simulation run, scroll halfway --> )");
		
		window.remove(mainPanel);
		MyReport.displayOverlappingGraphsReport(window, list, "Test Results For " + iterations + " Simulation Runs", multipleRunResults);
		
		THE_WORLD.getInstance().reset();
	}
	
	/*
	 * This method executes the bulk of the main loop.
	 */
	protected void updateAgents() {
		for(int i = THE_WORLD.getInstance().getPeople().size() - 1; i >= 0; i--){ //allows individual removal
			THE_WORLD.getInstance().getPeople().get(i).executiveFunction();
		}
		for(int i = 0; i < THE_WORLD.getInstance().getPeople().size(); i++){
			THE_WORLD.getInstance().getPeople().get(i).updateRecords();
		}
		for(EConsumable consumable : DeterminationHelper.INDUSTRY_ORDER){
			THE_WORLD.getInstance().getIndustry(consumable).executiveFunction();
		}
		for(EConsumable consumable : DeterminationHelper.INDUSTRY_ORDER){ 
			THE_WORLD.getInstance().getIndustry(consumable).payProfitTaxAndUpdateRecords();
		}
		THE_WORLD.getInstance().getGovernment().executiveFunction();
	}
	
	/*
	 * Provides the text for displaying as a note on graphs that display results of multiple runs
	 */
	private String getMultipleRunChartNote(int iterations) {
		return "(Chart based on " + iterations + " runs of the simulation)";
	}
}












