package simulate;
import helper.DeterminationHelper;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 * Includes very basic tests to help ensure important values are within certain ranges or maintain certain relationships to each other.
 * The included checks are simple and are only meant to potentially help limit the unavoidable headache that would come with changing these values.
 * 
 * @author Ian Stuart
 */
public class ScenarioValuesTest {

	
	public static void checkDeterminationHelperValues() throws Exception {
		ensureNonNegativeValue(DeterminationHelper.LIFE_EXPECTANCY);
		ensureNonNegativeValue(DeterminationHelper.LEGAL_ADULT_AGE);
		ensureFirstValueIsLessThanSecond(DeterminationHelper.LEGAL_ADULT_AGE, DeterminationHelper.LIFE_EXPECTANCY);
		ensureNonNegativeValue(DeterminationHelper.EXPECTED_RATIO_OF_MALE_BIRTHS);
		ensureNonNegativeValue(DeterminationHelper.MAX_NUMBER_OF_CHILDREN);
		ensureNonNegativeValue(DeterminationHelper.MAX_FERTILE_AGE);
		ensureNonNegativeValue(DeterminationHelper.CHANCE_OF_FERTILE_COUPLE_GETTING_PREGNANT);
		ensureNonNegativeValue(DeterminationHelper.INCREASE_CHANCE_OF_COUPLE_REPRODUCING_BY);
		ensureAboveOne(DeterminationHelper.INCREASE_CHANCE_OF_COUPLE_REPRODUCING_BY);
		ensureNonNegativeValue(DeterminationHelper.CHILDREN_TO_ADULT_RATIO_THRESHOLD_TO_BOOST_REPRODUCTION);
		ensureNonNegativeValue(DeterminationHelper.REDUCE_CHANCE_OF_COUPLE_REPRODUCING_BY);
		ensureBelowOne(DeterminationHelper.REDUCE_CHANCE_OF_COUPLE_REPRODUCING_BY);
		ensureNonNegativeValue(DeterminationHelper.TIME_TO_WAIT_BETWEEN_REPRODUCTION_RATE_ADJUSTMENTS);
		ensureNonNegativeValue(DeterminationHelper.NUMBER_OF_GENERATIONS_INCEST_TABOO_EXTENDS);
		ensureNonNegativeValue(DeterminationHelper.CHANCE_OF_COMPATIBLE_PAIR_ENTERING_MUTUAL_ROMANTIC_RELATIONSHIP_DURING_SENARIO_INITIALIZATION);
		ensureNonNegativeValue(DeterminationHelper.CHANCE_OF_COMPATIBLE_PAIR_ENTERING_MUTUAL_ROMANTIC_RELATIONSHIP);
		ensureFirstValueIsLessThanSecond(DeterminationHelper.CHANCE_OF_COMPATIBLE_PAIR_ENTERING_MUTUAL_ROMANTIC_RELATIONSHIP, 
														DeterminationHelper.CHANCE_OF_COMPATIBLE_PAIR_ENTERING_MUTUAL_ROMANTIC_RELATIONSHIP_DURING_SENARIO_INITIALIZATION);
		for(int i = 0; i < DeterminationHelper.AGE_DIFFERENCE_FILTERS.length; i++) {
			ensureNonNegativeValue(DeterminationHelper.AGE_DIFFERENCE_FILTERS[i]);
			if(i > 0) {
				ensureFirstValueIsLessThanSecond(DeterminationHelper.AGE_DIFFERENCE_FILTERS[i-1], DeterminationHelper.AGE_DIFFERENCE_FILTERS[i]);
			}
		}
		ensureNonNegativeValue(DeterminationHelper.INDEX_AND_AGE_OF_MATURITY_FOR_AGE_DIFFERENCE_FILTER.getFirst());
		ensureFirstValueIsLessThanSecond(DeterminationHelper.INDEX_AND_AGE_OF_MATURITY_FOR_AGE_DIFFERENCE_FILTER.getFirst(), DeterminationHelper.AGE_DIFFERENCE_FILTERS.length);
		ensureNonNegativeValue(DeterminationHelper.INDEX_AND_AGE_OF_MATURITY_FOR_AGE_DIFFERENCE_FILTER.getSecond());
		ensureFirstValueIsLessThanOrEqaulToSecond(DeterminationHelper.LEGAL_ADULT_AGE, DeterminationHelper.INDEX_AND_AGE_OF_MATURITY_FOR_AGE_DIFFERENCE_FILTER.getSecond());
		ensureNonNegativeValue(DeterminationHelper.CHANCE_OF_INITIAL_COUPLE_HAVING_HAD_A_KID_IN_A_GIVEN_YEAR);
		ensureNonNegativeValue(DeterminationHelper.CHANCE_OF_DYING_FROM_FAILING_TO_MEET_NEEDS);
		ensureNonNegativeValue(DeterminationHelper.CHANCE_OF_NON_ESSIENTIAL_EMPLOYEE_SWITCHING_TO_ESSENTIAL_INDUSTRY_IN_AN_EMERGENCY);
		ensureNonNegativeValue(DeterminationHelper.INDUSTRY_SAMPLE_SIZE_FOR_AVERAGE);
		ensureNonNegativeValue(DeterminationHelper.MAX_ADJUSTED_PRODUCTION_RATE);
		ensureNonNegativeValue(DeterminationHelper.ADJUST_ADJUST_PRODUCTION_RATE_BY);
		ensureNonNegativeValue(DeterminationHelper.MULTIPLIER_TO_DETERMINE_TARGET_INPUT_MATERIAL_INVENTORY);
		ensureNonNegativeValue(DeterminationHelper.MULTIPLIER_OF_EXPENSES_TO_DETERMINE_DIVISOR_OF_MONEY_FOR_DETERMINING_FINANCIAL_STANDING);
		ensureNonNegativeValue(DeterminationHelper.MULTIPLIER_OF_DEMAND_TO_DETERMINE_TARGET_PRODUCT_INVENTORY);
		ensureNonNegativeValue(DeterminationHelper.FINANCIAL_STANDING_CAP);
		ensureNonNegativeValue(DeterminationHelper.TIME_TO_WAIT_BETWEEN_CHANGES_IN_THE_NUMBER_OF_JOBS);
		ensureNonNegativeValue(DeterminationHelper.MINIMUM_FINANACIAL_STANDING_FOR_DESCRETIONARY_INDUSTRY_TO_ADD_JOBS);
		ensureNonNegativeValue(DeterminationHelper.AVERAGE_FINANCIAL_STANDING_AND_PRODUCT_QUANTITY_THRESHOLD_TO_SIGNAL_INSUFFICIENT_ASSETS.getFirst());
		ensureBelowOne(DeterminationHelper.AVERAGE_FINANCIAL_STANDING_AND_PRODUCT_QUANTITY_THRESHOLD_TO_SIGNAL_INSUFFICIENT_ASSETS.getFirst());
		ensureNonNegativeValue(DeterminationHelper.AVERAGE_FINANCIAL_STANDING_AND_PRODUCT_QUANTITY_THRESHOLD_TO_SIGNAL_INSUFFICIENT_ASSETS.getSecond());
		ensureNonNegativeValue(DeterminationHelper.MINIMUM_TIME_BETWEEN_SALE_PRICE_CHANGES);
		
		ensureNonNegativeValue(DeterminationHelper.MULTIPLICAR_TO_DETERMINE_LOWEST_AND_SECOND_LOWEST_SALE_PRICE);
		ensureBelowOne(DeterminationHelper.MULTIPLICAR_TO_DETERMINE_LOWEST_AND_SECOND_LOWEST_SALE_PRICE);
		ensureNonNegativeValue(DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_SECOND_LOWEST_AND_MIDDLE_SALE_PRICE);
		ensureNonNegativeValue(DeterminationHelper.LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD);
		ensureAboveOne(DeterminationHelper.LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD);
		ensureNonNegativeValue(DeterminationHelper.SECOND_LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD);
		ensureAboveOne(DeterminationHelper.SECOND_LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD);
		ensureFirstValueIsLessThanSecond(DeterminationHelper.SECOND_LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD, DeterminationHelper.LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD);
		ensureNonNegativeValue(DeterminationHelper.SECOND_HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD);
		ensureBelowOne(DeterminationHelper.SECOND_HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD);
		ensureFirstValueIsLessThanSecond(DeterminationHelper.SECOND_HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD, DeterminationHelper.SECOND_LOWEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD);
		ensureNonNegativeValue(DeterminationHelper.HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD);
		ensureBelowOne(DeterminationHelper.HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD);
		ensureFirstValueIsLessThanSecond(DeterminationHelper.HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD, DeterminationHelper.SECOND_HIGHEST_SALE_PRICE_AVERAGE_FINANCIAL_STANDING_THRESHOLD);
		ensureNonNegativeValue(DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_SECOND_HIGHEST_SALE_PRICE);
		ensureFirstValueIsLessThanSecond(DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_SECOND_LOWEST_AND_MIDDLE_SALE_PRICE, DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_SECOND_HIGHEST_SALE_PRICE);
		ensureNonNegativeValue(DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_HIGHEST_SALE_PRICE);
		ensureFirstValueIsLessThanSecond(DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_SECOND_HIGHEST_SALE_PRICE, DeterminationHelper.AMOUNT_TO_ADD_TO_DETERMINE_HIGHEST_SALE_PRICE);
		ensureNonNegativeValue(DeterminationHelper.SECOND_HIGHEST_SALE_PRICE_PERCENT_OF_TARGET_INVENTORY_OTHER_POSSIBLE_THRESHOLD);
		ensureBelowOne(DeterminationHelper.SECOND_HIGHEST_SALE_PRICE_PERCENT_OF_TARGET_INVENTORY_OTHER_POSSIBLE_THRESHOLD);
		ensureNonNegativeValue(DeterminationHelper.HIGHEST_SALE_PRICE_PERCENT_OF_TARGET_INVENTORY_OTHER_POSSIBLE_THRESHOLD);
		ensureBelowOne(DeterminationHelper.HIGHEST_SALE_PRICE_PERCENT_OF_TARGET_INVENTORY_OTHER_POSSIBLE_THRESHOLD);
		ensureFirstValueIsLessThanSecond(DeterminationHelper.HIGHEST_SALE_PRICE_PERCENT_OF_TARGET_INVENTORY_OTHER_POSSIBLE_THRESHOLD, DeterminationHelper.SECOND_HIGHEST_SALE_PRICE_PERCENT_OF_TARGET_INVENTORY_OTHER_POSSIBLE_THRESHOLD);
		
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_INVENTORY_FOR_THESHOLD_TO_REMOVE_JOBS);
		ensureAboveOne(DeterminationHelper.RATIO_OF_TARGET_INVENTORY_FOR_THESHOLD_TO_REMOVE_JOBS);
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_INVENTORY_FOR_THESHOLD_TO_ADD_JOBS);
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_INVENTORY_FOR_THESHOLD_TO_ADD_JOBS);
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_INVENTORY_CEILING_FOR_THESHOLD_TO_CALCULATE_JOBS_TO_ADD_WITH_CURRENT_INSTEAD_OF_AVERAGE_DEMAND);
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_INVENTORY_CEILING_FOR_THESHOLD_TO_CALCULATE_JOBS_TO_ADD_WITH_CURRENT_INSTEAD_OF_AVERAGE_DEMAND);
		ensureNonNegativeValue(DeterminationHelper.DIVISOR_OF_AMOUNT_NEEDED_TO_MEET_TARGET_INVENTORY_TO_CALCULATE_ADDITIONAL_JOBS_BASED_ONLY_ON_THIS);
		
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_INVENTORY_AND_RATIO_OF_PRODUCTION_OVER_AVERAGE_DEMAND_FOR_THESHOLD_TO_REMOVE_JOBS.getFirst());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_INVENTORY_AND_RATIO_OF_PRODUCTION_OVER_AVERAGE_DEMAND_FOR_THESHOLD_TO_REMOVE_JOBS.getFirst());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_INVENTORY_AND_RATIO_OF_PRODUCTION_OVER_AVERAGE_DEMAND_FOR_THESHOLD_TO_REMOVE_JOBS.getSecond());
		ensureAboveOne(DeterminationHelper.RATIO_OF_TARGET_INVENTORY_AND_RATIO_OF_PRODUCTION_OVER_AVERAGE_DEMAND_FOR_THESHOLD_TO_REMOVE_JOBS.getSecond());
		
		ensureNonNegativeValue(DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_REMOVED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst());
		ensureBelowOne(DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_REMOVED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst());
		ensureNonNegativeValue(DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_REMOVED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getSecond());
		
		ensureNonNegativeValue(DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_DISCRETIONARY_INDUSTRY_CAN_ADD_AT_ONCE_IF_IT_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst());
		ensureBelowOne(DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_DISCRETIONARY_INDUSTRY_CAN_ADD_AT_ONCE_IF_IT_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst());
		ensureNonNegativeValue(DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_DISCRETIONARY_INDUSTRY_CAN_ADD_AT_ONCE_IF_IT_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getSecond());
		
		ensureNonNegativeValue(DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_ADDED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst());
		ensureBelowOne(DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_ADDED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getFirst());
		ensureNonNegativeValue(DeterminationHelper.MAXIMUM_PERCENT_OF_JOBS_THAT_CAN_BE_ADDED_AT_ONCE_IF_INDUSTRY_HAS_ATLEAST_THIS_MANY_EMPLOYEES.getSecond());
		
		ensureNonNegativeValue(DeterminationHelper.REQUIRED_NECESSITY_CONSUMABLE_FOR_FIRST_ADULT_OF_A_HOME);
		ensureNonNegativeValue(DeterminationHelper.ADDITIONAL_NECESSITY_CONSUMABLE_REQUIRED_FOR__THE_SECOND_ADULT_OF_A_HOME_WHERE_THE_TOTAL_IS_ROUNDED_DOWN);
		ensureNonNegativeValue(DeterminationHelper.ADDITIONAL_NECESSITY_CONSUMABLE_REQUIRED_FOR_EACH_PERSON_UNDER_18_WHERE_THE_TOTAL_FOR_A_HOME_IS_ROUNDED_DOWN);
		ensureNonNegativeValue(DeterminationHelper.AMOUNT_OF_MONEY_PEOPLE_MAKE_SURE_TO_SAVE_WHEN_SPENDING_MONEY_ON_DESCRETIONARIES);
		ensureNonNegativeValue(DeterminationHelper.AMOUNT_TO_PAY_TO_ASSIST_AND_IMPOVERISHED_INDUSTRY);
		ensureNonNegativeValue(DeterminationHelper.INITIAL_PARENT_TO_A_NEW_CHILD_PAYMENT);
		ensureNonNegativeValue(DeterminationHelper.MAXIMUM_PARENT_TO_A_NEW_CHILD_PAYMENT);
		ensureFirstValueIsLessThanOrEqaulToSecond(DeterminationHelper.INITIAL_PARENT_TO_A_NEW_CHILD_PAYMENT, DeterminationHelper.MAXIMUM_PARENT_TO_A_NEW_CHILD_PAYMENT);
		ensureNonNegativeValue(DeterminationHelper.PARENT_TO_A_NEW_CHILD_PAYMENT_ADJUSTMENT_INCREMENTS);
		ensureFirstValueIsLessThanSecond(DeterminationHelper.PARENT_TO_A_NEW_CHILD_PAYMENT_ADJUSTMENT_INCREMENTS, DeterminationHelper.MAXIMUM_PARENT_TO_A_NEW_CHILD_PAYMENT);
		ensureNonNegativeValue(DeterminationHelper.PARENT_TO_A_NEW_CHILD_PAYMENT_AVERAGE_PERSONAL_BALANCE_THRESHOLDS_TO__LOWER_AND_RAISE__.getFirst());
		ensureNonNegativeValue(DeterminationHelper.PARENT_TO_A_NEW_CHILD_PAYMENT_AVERAGE_PERSONAL_BALANCE_THRESHOLDS_TO__LOWER_AND_RAISE__.getSecond());
		ensureFirstValueIsLessThanSecond(DeterminationHelper.PARENT_TO_A_NEW_CHILD_PAYMENT_AVERAGE_PERSONAL_BALANCE_THRESHOLDS_TO__LOWER_AND_RAISE__.getSecond(), 
																DeterminationHelper.PARENT_TO_A_NEW_CHILD_PAYMENT_AVERAGE_PERSONAL_BALANCE_THRESHOLDS_TO__LOWER_AND_RAISE__.getFirst());
		ensureNonNegativeValue(DeterminationHelper.MINIMUM_TIME_BETWEEN_PARENT_TO_A_NEW_CHILD_PAYMENT_ADJUSTMENTS);
		
		ensureNonNegativeValue(DeterminationHelper.INITIAL_SALES_TAX_RATE);
		ensureBelowOne(DeterminationHelper.INITIAL_SALES_TAX_RATE);
		ensureNonNegativeValue(DeterminationHelper.MINIMUM_ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_REQUIRED_TO_RAISE_SALES_TAX);
		ensureBelowOne(DeterminationHelper.MINIMUM_ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_REQUIRED_TO_RAISE_SALES_TAX);
		ensureNonNegativeValue(DeterminationHelper.MINIMUM_TIME_BETWEEN_CHANGES_TO_SALES_TAX);
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getFirst());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getFirst());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getSecond());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getSecond());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getThird());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getThird());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getFirst());
		ensureAboveOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getFirst());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getSecond());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getSecond());
		ensureFirstValueIsLessThanSecond(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getSecond(),
													DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getSecond());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getThird());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getThird());
		ensureFirstValueIsLessThanOrEqaulToSecond(DeterminationHelper.INITIAL_SALES_TAX_RATE, DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_SALES_TAX.getSecond());
		ensureFirstValueIsLessThanOrEqaulToSecond(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_SALES_TAX.getSecond(), DeterminationHelper.INITIAL_SALES_TAX_RATE);
		
		ensureNonNegativeValue(DeterminationHelper.INITIAL_PROFIT_TAX_RATE);
		ensureBelowOne(DeterminationHelper.INITIAL_PROFIT_TAX_RATE);
		ensureNonNegativeValue(DeterminationHelper.ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_THRESHOLD_TO_CHANGE_PROFIT_TAX_RATE);
		ensureBelowOne(DeterminationHelper.ESSENTIAL_INDUSTRY_FINANCIAL_STANDING_THRESHOLD_TO_CHANGE_PROFIT_TAX_RATE);
		ensureNonNegativeValue(DeterminationHelper.MINIMUM_TIME_BETWEEN_CHANGES_TO_PROFIT_TAX_RATE);
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getFirst());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getFirst());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getSecond());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getSecond());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getThird());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getThird());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getFirst());
		ensureAboveOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getFirst());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getSecond());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getSecond());
		ensureFirstValueIsLessThanSecond(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getSecond(),
													DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getSecond());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getThird());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getThird());
		ensureFirstValueIsLessThanOrEqaulToSecond(DeterminationHelper.INITIAL_PROFIT_TAX_RATE, DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_PROFIT_TAX.getSecond());
		ensureFirstValueIsLessThanOrEqaulToSecond(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_PROFIT_TAX.getSecond(), DeterminationHelper.INITIAL_PROFIT_TAX_RATE);
		
		ensureNonNegativeValue(DeterminationHelper.INITIAL_INCOME_TAX_RATE);
		ensureBelowOne(DeterminationHelper.INITIAL_INCOME_TAX_RATE);
		ensureNonNegativeValue(DeterminationHelper.MINIMUM_TIME_BETWEEN_CHANGES_TO_INCOME_TAX_RATE);
		
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getFirst());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getFirst());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getSecond());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getSecond());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getThird());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getThird());
		
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_INCOME_TAX.getFirst());
		ensureAboveOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_INCOME_TAX.getFirst());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_INCOME_TAX.getSecond());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_INCOME_TAX.getSecond());
		ensureFirstValueIsLessThanSecond(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_INCOME_TAX.getSecond(),
													DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getSecond());
		ensureNonNegativeValue(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_INCOME_TAX.getThird());
		ensureBelowOne(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_INCOME_TAX.getThird());
		ensureFirstValueIsLessThanOrEqaulToSecond(DeterminationHelper.INITIAL_INCOME_TAX_RATE, DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MAXIUM_RATE_AND_INCREMENT__FOR_RAISING_INCOME_TAX.getSecond());
		ensureFirstValueIsLessThanOrEqaulToSecond(DeterminationHelper.RATIO_OF_TARGET_BALANCE_AND_MINIMUM_RATE_AND_INCREMENT__FOR_LOWERING_INCOME_TAX.getSecond(), DeterminationHelper.INITIAL_INCOME_TAX_RATE);
		
		ensureNonNegativeValue(DeterminationHelper.INITIAL_INHERITANCE_TAX);
	
		ensureNonNegativeValue(DeterminationHelper.INITIAL_RETIREMENT_AGE);
		ensureNonNegativeValue(DeterminationHelper.MINIMUM_RETIREMENT_AGE);
		ensureFirstValueIsLessThanOrEqaulToSecond(DeterminationHelper.MINIMUM_RETIREMENT_AGE, DeterminationHelper.INITIAL_RETIREMENT_AGE);
		ensureNonNegativeValue(DeterminationHelper.AMOUNT_TO_PAY_FOR_SOCIAL_SECURITY);
		ensureNonNegativeValue(DeterminationHelper.MINIMUM_WAGE);
		for(int i = 0; i < DeterminationHelper.RETIREMENT_AGE__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS.length; i++) {
			ensureNonNegativeValue(DeterminationHelper.RETIREMENT_AGE__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i].getFirst());
			ensureBelowOne(DeterminationHelper.RETIREMENT_AGE__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i].getFirst());
			ensureNonNegativeValue(DeterminationHelper.RETIREMENT_AGE__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i].getSecond());
			ensureNonNegativeValue(DeterminationHelper.RETIREMENT_AGE__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i].getThird());
			if(i > 0) {
				ensureFirstValueIsLessThanSecond(DeterminationHelper.RETIREMENT_AGE__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i].getFirst(),
															DeterminationHelper.RETIREMENT_AGE__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i - 1].getFirst());
			}
		}
		ensureNonNegativeValue(DeterminationHelper.MINIMUM_LENGTH_OF_WORKDAY);
		for(int i = 0; i < DeterminationHelper.WORKDAY_LENGTH__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS.length; i++) {
			ensureNonNegativeValue(DeterminationHelper.WORKDAY_LENGTH__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i].getFirst());
			ensureBelowOne(DeterminationHelper.WORKDAY_LENGTH__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i].getFirst());
			ensureNonNegativeValue(DeterminationHelper.WORKDAY_LENGTH__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i].getSecond());
			ensureNonNegativeValue(DeterminationHelper.WORKDAY_LENGTH__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i].getThird());
			ensureBelowOne(DeterminationHelper.WORKDAY_LENGTH__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i].getThird());
			if(i > 0) {
				ensureFirstValueIsLessThanSecond(DeterminationHelper.WORKDAY_LENGTH__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i].getFirst(),
															DeterminationHelper.WORKDAY_LENGTH__UNEMPLOYMENT_AND_THE_TIME_BETWEEN_CHANGES_AND_INCREMENT_BY__THRESHOLDS[i - 1].getFirst());
			}
		}
		ensureNonNegativeValue(DeterminationHelper.TIME_FOR_SENARIO_TO_SEE_EARLY_INDUSTRY_SHIFT);
		ensureNonNegativeValue(DeterminationHelper.TIME_FOR_SENARIO_IN_RANGE_OF_EXPONENTIAL_PRODUCTION_RATE_INCREASE);
		ensureNonNegativeValue(DeterminationHelper.TIME_FOR_SENARIO_EXTENDING_PAST_PRODUCTION_RATE_AND_POPULATION_PLATEAU);
		ensureNonNegativeValue(DeterminationHelper.MINIMUM_ADULT_STARTING_MONEY);
		ensureNonNegativeValue(DeterminationHelper.EXPECTED_ADULT_EXTRA_STARTING_MONEY_IF_UNIFORM_DISTRIBUTION);
	}
	
	private static void ensureAboveOne(double number) throws Exception {
		if(number <= 1) {
			throw new Exception("Encountered value that should have been above 1.");
		}
	}
	
	private static void ensureBelowOne(double number) throws Exception {
		if(number >= 1) {
			throw new Exception("Encountered value that should have been below 1.");
		}
	}
	
	private static void ensureFirstValueIsLessThanSecond(Number first, Number second) throws Exception {
		if(first.doubleValue() >= second.doubleValue()) {
			throw new Exception("Encountered two values that did not maintain their required relative relationship to eachother.");
		}
	}
	
	private static void ensureFirstValueIsLessThanOrEqaulToSecond(Number first, Number second) throws Exception {
		if(first.doubleValue() > second.doubleValue()) {
			throw new Exception("Encountered two values that did not maintain their required relative relationship to eachother.");
		}
	}
	
	private static void ensureNonNegativeValue(Number number) throws Exception {
		if( number.doubleValue() < 0 ) {
			throw new Exception("Unexpected Negative Value");
		}
	}

}
