package utility;

import java.util.Collections;
import java.util.LinkedList;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 *  ExtendedCountSort uses count sort for values under or equal to the common range, uses Collections.sort for all values above the common range.
 *  Values are to be added one at a time using addValue. For reuse zeroValues needs to be called to delete current values.
 *  
 * @author Ian Stuart
 */
public class ExtendedCountSort {
	int[] countSortArray;
	private int numberOfValues = 0;
	private int total = 0;
	public LinkedList<Integer> valuesOverCommonRange = new LinkedList<Integer>();
	
	public ExtendedCountSort(int commonRange) {
		this.countSortArray = new int[commonRange+1];
	}
	
	public void addValue(int value) {
		if(value < countSortArray.length) {
			countSortArray[value]++;
		}else {
			valuesOverCommonRange.add(value);
		}
		numberOfValues++;
		total += value;
	}
	
	public int[] getSortedArrayOfCurrentValues() {
		int[] sortedArray = new int[numberOfValues];
		int sortedArrayIndex = 0;
		for(int i = 0; i < this.countSortArray.length; i++) {
			if(countSortArray[i] > 0) {
				for(int j = 0; j < countSortArray[i]; j++) {
					sortedArray[sortedArrayIndex] = i;
					sortedArrayIndex++;
				}
			}
		}
		Collections.sort(this.valuesOverCommonRange);
		
		for(int i = 0; i < this.valuesOverCommonRange.size(); i++) {
			sortedArray[sortedArrayIndex] = valuesOverCommonRange.get(i);
			sortedArrayIndex++;
		}
		return sortedArray;
	}
	
	public int getAverage() {
		return total/(numberOfValues > 0 ? numberOfValues : 1);
	}
	
	public void zeroValues() { 
		for(int i = 0; i < countSortArray.length; i++) {
			countSortArray[i] = 0;
		}
		this.numberOfValues = 0;
		if(!this.valuesOverCommonRange.isEmpty()) {
			valuesOverCommonRange = new LinkedList<Integer>();
		}
	}
}
