package utility;
import people.THE_WORLD;
/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

Three Industry ABCE Model is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Three Industry ABCE Model is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/**
 * Created to improve performance where an item usually doesn't need to be updated more can once in a time step.
 * 
 * @author Ian Stuart
 *
 * @param <T> type of object being held/updated.
 */
public class LazyUpdateUpToOncePerTick<T> {
	private final IReturnsItem<T> returnsItem;
	private int lastTimeItemWasUpdated = Integer.MIN_VALUE;
	private T item;
	public LazyUpdateUpToOncePerTick(IReturnsItem<T> returnsItem) {
		this.returnsItem = returnsItem;
	}
	
	public T get() {
		if(lastTimeItemWasUpdated != THE_WORLD.getInstance().getTime()) {
			this.item = returnsItem.getItem();
			lastTimeItemWasUpdated = THE_WORLD.getInstance().getTime();
		}
		return this.item;
	}
	
	public T forceUpdate() {
		this.item = returnsItem.getItem();
		return this.item;
	}
}
