package utility;
import java.util.Random;

/*	Copyright 2021 Ian Stuart

	This file is part of Three Industry ABCE Model.

	Three Industry ABCE Model is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	Three Industry ABCE Model is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with Three Industry ABCE Model.  If not, see <https://www.gnu.org/licenses/>.
*/

/*
 * This class provides a way to provide uncertainty between two objects of the same type.
 * In an Agent Based Model this can provide a balance where the consistent order of two objects is not desired. 
 */

/**
 * A generic class that is useful for providing uncertainty (of which is which) between two object of the same type.
 * In an agent based model this can provide a balance where consistent order of two objects is not desired.
 * 
 * @author Ian Stuart
 *
 * @param <T> Type of objects to obscure which is which.
 */
public class ObscureWhichIsWhich <T> {
	private final T itemA;
	private final T item1;
	
	/**
	 * This class provides a way to provide uncertainty between two objects of the same type.
	 * In an Agent Based Model this can provide a balance where the consistent order of two objects is not desired. 
	 * 
	 * @param objectUno object uno.
	 * @param objectI object I.
	 */
	public ObscureWhichIsWhich(T objectUno, T objectI) {
		Random random = new Random();
		if(random.nextDouble() < 0.5) {
			this.itemA = objectUno;
			this.item1 = objectI;
		}else {
			this.item1 = objectUno;
			this.itemA = objectI;
		}
	}
	
	/**
	 * @return itemA. Wait... was this objectUno or objectI?
	 */
	public T getItemA() { return this.itemA;}
	
	/**
	 * @return item1. Wait... was this objectUno or objectI?
	 */
	public T getItem1() { return this.item1;}
}

